@@ -7,7 +7,7 @@
 
 (require 'mew)
 
-(defsubst mew-case-default-p (case)
+(defun mew-case-default-p (case)
   (or (null case) (string= case mew-case-default)))
 
 (defvar mew-generate-mail-address-list nil)
@@ -18,20 +18,31 @@
 ;;
 ;;
 
-(defun mew-cfent-value (case key def &optional symbol)
+(defun mew-cfent-value (case key def &optional type)
   (let ((cases (and case (mapcar 'mew-chop (mew-split case ?,))))
-	ent ret)
+	ent ret cfent)
     (or (member mew-case-default cases)
 	(setq cases (cons mew-case-default cases)))
     (setq cases (nreverse cases))
     (catch 'loop
-      (while cases
-	(setq ent (assoc key (mew-cfent-by-case (car cases))))
+      (dolist (cs cases)
+	(setq cfent (mew-cfent-by-case cs))
+	(setq ent (or (assoc key cfent) ;; old
+		      (assoc (intern key) cfent))) ;; new
 	(when ent
-	  (setq ret (cdr ent))
-	  (if symbol (setq ret (symbol-value ret)))
-	  (if ent (throw 'loop nil)))
-	(setq cases (cdr cases)))
+	  (setq ret (cdr ent)) ;; old
+	  (cond
+	   ((eq type 'symbol)
+	    (if (consp ret) (setq ret (car ret)))
+	    (setq ret (symbol-value ret)))
+	   ((eq type 'list)
+	    (if (and (consp (car ret)) (null (cdr ret))) ;; new
+		(setq ret (car ret)))
+	    (if (equal ret '(nil))
+		(setq ret nil)))
+	   (t
+	    (if (consp ret) (setq ret (car ret))))) ;; new
+	  (if ent (throw 'loop nil))))
       (setq ret def))
     ret))
 
@@ -46,6 +57,12 @@
   (if (null mew-mail-domain-list) (setq mew-generate-mail-domain-list t))
   (if (null mew-from-list) (setq mew-generate-from-list t)))
 
+(defun mew-config-case-name (ent)
+  (let ((case (car ent)))
+    (if (symbolp case)
+	(symbol-name case)
+      case)))
+
 (defun mew-config-setup ()
   (if mew-generate-mail-address-list (setq mew-mail-address-list nil))
   (if mew-generate-mail-domain-list (setq mew-mail-domain-list nil))
@@ -53,17 +70,15 @@
   (setq mew-inbox-folders nil)
   (setq mew-queue-folders nil)
   (setq mew-postq-folders nil)
-  (setq mew-config-cases (mapcar 'car mew-config-alist))
+  (setq mew-config-cases (mapcar 'mew-config-case-name mew-config-alist))
   (or (member mew-case-default mew-config-cases)
       (setq mew-config-cases (cons mew-case-default mew-config-cases)))
   (setq mew-config-cases2
 	(append mew-folder-prefixes
 		(mapcar (lambda (x) (concat x ":")) mew-config-cases)))
   (let ((cases (reverse mew-config-cases)) ;; must copy
-	case val) 
-    (while cases
-      (setq case (car cases))
-      (setq cases (cdr cases))
+	val)
+    (dolist (case cases)
       (setq val (mew-inbox-folder case))
       (or (mew-folder-inboxp val)
 	  (setq mew-inbox-folders (cons val mew-inbox-folders)))
@@ -121,6 +136,9 @@
 (defun mew-fcc (&optional case)
   (mew-cfent-value case "fcc" mew-fcc))
 
+(defun mew-bcc (&optional case)
+  (mew-cfent-value case "bcc" mew-bcc))
+
 (defun mew-dcc (&optional case)
   (let ((dcc (mew-cfent-value case "dcc" mew-dcc)))
     (if (eq dcc 'me) (mew-mail-address case) dcc)))
@@ -159,11 +177,18 @@
 (defun mew-smtp-helo-domain (&optional case)
   (mew-cfent-value case "smtp-helo-domain" mew-smtp-helo-domain))
 
+(defun mew-smtp-user-only (&optional case)
+  (let ((user-domain (mew-smtp-user case)))
+    (if (string-match "^\\([^@]+\\)@" user-domain)
+	(mew-match-string 1 user-domain)
+      user-domain)))
+
 (defun mew-smtp-user (&optional case)
-  (mew-cfent-value case "smtp-user" mew-smtp-user))
+  (or (mew-cfent-value case "smtp-user" mew-smtp-user)
+      (mew-mail-address case)))
 
 (defun mew-smtp-auth-list (&optional case)
-  (mew-cfent-value case "smtp-auth-list" mew-smtp-auth-list))
+  (mew-cfent-value case "smtp-auth-list" mew-smtp-auth-list 'list))
 
 (defun mew-smtp-mail-from (&optional case)
   (mew-cfent-value case "smtp-mail-from" mew-smtp-mail-from))
@@ -183,6 +208,12 @@
 	 (time (mew-time-ctz-to-msgid (current-time))))
     (concat "<" time "." random "." user "@" domain ">")))
 
+(defun mew-use-smtp-auth (&optional case)
+  (mew-cfent-value case "use-smtp-auth" mew-use-smtp-auth))
+
+(defun mew-smtp-auth-plain-authorize-id (&optional case)
+  (mew-cfent-value case "smtp-auth-plain-authorize-id" mew-smtp-auth-plain-authorize-id))
+
 ;;
 
 (defun mew-mailbox-type (&optional case)
@@ -226,7 +257,7 @@
   (mew-cfent-value case "pop-user" mew-pop-user))
 
 (defun mew-pop-auth-list (&optional case)
-  (mew-cfent-value case "pop-auth-list" mew-pop-auth-list))
+  (mew-cfent-value case "pop-auth-list" mew-pop-auth-list 'list))
 
 (defun mew-pop-size (&optional case)
   (mew-cfent-value case "pop-size" mew-pop-size))
@@ -260,11 +291,17 @@
 (defun mew-imap-ssl-port (&optional case)
   (mew-cfent-value case "imap-ssl-port" mew-imap-ssl-port))
 
+(defun mew-imap-proxy-server (&optional case)
+  (mew-cfent-value case "imap-proxy-server" mew-imap-proxy-server))
+
+(defun mew-imap-proxy-port (&optional case)
+  (mew-cfent-value case "imap-proxy-port" mew-imap-proxy-port))
+
 (defun mew-imap-user (&optional case)
   (mew-cfent-value case "imap-user" mew-imap-user))
 
 (defun mew-imap-auth-list (&optional case)
-  (mew-cfent-value case "imap-auth-list" mew-imap-auth-list))
+  (mew-cfent-value case "imap-auth-list" mew-imap-auth-list 'list))
 
 (defun mew-imap-size (&optional case)
   (mew-cfent-value case "imap-size" mew-imap-size))
@@ -276,7 +313,7 @@
   (mew-cfent-value case "imap-header-only" mew-imap-header-only))
 
 (defun mew-imap-prefix-list (&optional case)
-  (mew-cfent-value case "imap-prefix-list" mew-imap-prefix-list))
+  (mew-cfent-value case "imap-prefix-list" mew-imap-prefix-list 'list))
 
 ;;
 
@@ -323,6 +360,9 @@
 (defun mew-inbox-folder (&optional case)
   (mew-cfent-value case "inbox-folder" mew-inbox-folder))
 
+(defun mew-imap-inbox-folder (&optional case)
+  (mew-cfent-value case "imap-inbox-folder" mew-imap-inbox-folder))
+
 (defun mew-nntp-newsgroup (&optional case)
   (mew-cfent-value case "nntp-newsgroup" mew-nntp-newsgroup))
 
@@ -342,10 +382,10 @@
   (mew-cfent-value case "imap-trash-folder" mew-imap-trash-folder))
 
 (defun mew-imap-trash-folder-list (&optional case)
-  (mew-cfent-value case "imap-trash-folder-list" mew-imap-trash-folder-list))
+  (mew-cfent-value case "imap-trash-folder-list" mew-imap-trash-folder-list 'list))
 
 (defun mew-header-alist (&optional case)
-  (mew-cfent-value case "header-alist" mew-header-alist))
+  (mew-cfent-value case "header-alist" mew-header-alist 'list))
 
 (defun mew-signature-file (&optional case)
   (mew-cfent-value case "signature-file" mew-signature-file))
@@ -353,20 +393,34 @@
 (defun mew-content-type (&optional case)
   (mew-capitalize (mew-cfent-value case "content-type" mew-content-type)))
 
-(defun mew-use-retrieve (&optional case)
-  (mew-cfent-value case "use-retrieve" mew-use-retrieve))
-
 (defun mew-imap-spam-field (&optional case)
   (mew-cfent-value case "imap-spam-field" mew-imap-spam-field))
 
 (defun mew-imap-spam-word (&optional case)
   (mew-cfent-value case "imap-spam-word" mew-imap-spam-word))
 
+(defun mew-imap-spam-pattern (&optional case)
+  (mew-cfent-value case "imap-spam-pattern" mew-imap-spam-pattern))
+
 (defun mew-imap-spam-folder (&optional case)
   (mew-cfent-value case "imap-spam-folder" mew-imap-spam-folder))
 
 ;;
 
+(defun mew-warn-addresses (&optional case)
+  (mew-cfent-value case "warn-addresses" mew-warn-addresses))
+
+(defun mew-safe-addresses (&optional case)
+  (mew-cfent-value case "safe-addresses" mew-safe-addresses))
+
+(defun mew-warn-domains (&optional case)
+  (mew-cfent-value case "warn-domains" mew-warn-domains))
+
+(defun mew-safe-domains (&optional case)
+  (mew-cfent-value case "safe-domains" mew-safe-domains))
+
+;;
+
 (defun mew-proto (&optional case)
   (mew-cfent-value case "proto" mew-proto))
 
@@ -403,77 +457,74 @@
 (defun mew-use-old-pgp (&optional case)
   (mew-cfent-value case "use-old-pgp" mew-use-old-pgp))
 
+;;
+
+(defun mew-inbox-action-alist (&optional case)
+  (mew-cfent-value case "inbox-action-alist" mew-inbox-action-alist 'list))
+
+;;
+
+(defun mew-draft-privacy-method (&optional case)
+  (mew-cfent-value case "draft-privacy-method" mew-draft-privacy-method))
+
+(defun mew-protect-privacy-always (&optional case)
+  (mew-cfent-value case "protect-privacy-always" mew-protect-privacy-always))
+
+(defun mew-protect-privacy-always-type (&optional case)
+  (mew-cfent-value case "protect-privacy-always-type" mew-protect-privacy-always-type))
+
+(defun mew-protect-privacy-encrypted (&optional case)
+  (mew-cfent-value case "protect-privacy-encrypted" mew-protect-privacy-encrypted))
+
+(defun mew-protect-privacy-encrypted-type (&optional case)
+  (mew-cfent-value case "protect-privacy-encrypted-type" mew-protect-privacy-encrypted-type))
+
+(defun mew-protect-privacy-with-old-pgp-signature (&optional case)
+  (mew-cfent-value case "protect-privacy-with-old-pgp-signature" mew-protect-privacy-with-old-pgp-signature))
+
+;;
+
+(defun mew-pgp-signer (&optional case)
+  (mew-cfent-value case "pgp-signer" mew-pgp-signer))
+
+(defun mew-smime-signer (&optional case)
+  (mew-cfent-value case "smime-signer" mew-smime-signer))
+
+(defun mew-use-x-mailer (&optional case)
+  (mew-cfent-value case "use-x-mailer" mew-use-x-mailer))
+
+(defun mew-use-format-flowed (&optional case)
+  (mew-cfent-value case "use-format-flowed" mew-use-format-flowed))
+
 ;;;
 ;;; Setting case
 ;;;
 
-(defvar mew-case-input nil
-  "\\<mew-summary-mode-map>This is used for
-1. '\\[mew-summary-retrieve]' to select a POP server (or something)
-2. Biff to select a server (or somthing)
-3. If 'mew-use-case-input-completion' is non-nil, this value 
-   is automatically inserted when inputting a folder>")
+(defvar mew-case nil)
 
-(defvar mew-case-output nil
-  "This is used to
-1. a draft. 
-   It is used to complete Newsgroups, 
-                 choose default Content-Type: for a file to be attached,
-                 choose a signature file, 
-                 select a SMTP server (or something).
-2. flushing a message queue.")
-
-(defun mew-case-set-both ()
+(defun mew-case-set ()
   "Set the case both for input and output."
   (interactive)
-  (let ((case (mew-input-case mew-case-input "Input/Output")))
-    (setq mew-case-input case)
-    (setq mew-case-output case)))
+  (setq mew-case (mew-input-case mew-case)))
 
-(defun mew-case-set-input ()
-  "Set the case for input. If 'mew-case-synchronize' is
-non-nil, set the case for output, too."
+(defun mew-summary-set-case ()
+  "Set the case."
   (interactive)
-  (if mew-case-synchronize
-      (mew-case-set-both)
-    (setq mew-case-input (mew-input-case mew-case-input "Input"))))
-
-(defun mew-case-set-output ()
-  "Set the case for output. If 'mew-case-synchronize' is
-non-nil, set the case for input, too."
-  (interactive)
-  (if mew-case-synchronize
-      (mew-case-set-both)
-    (setq mew-case-output (mew-input-case mew-case-output "Output"))))
-
-(defun mew-summary-set-case (&optional output)
-  "Set the case. 
- (1) If 'mew-case-synchronize' is non-nil, set the case both for
- input and output.
- (2) If called with '\\[universal-argument]', set the case for output.
- (3) Otherwise, set the case for input."
-  (interactive "P")
-  (let (case inbox)
-    (if output
-	(setq case (mew-case-set-output))
-      (setq case (mew-case-set-input)))
+  (let ((case (mew-case-set))) ;; side effect
     (save-excursion
-      (let ((bufs mew-buffers))
-	(while bufs
-	  (if (get-buffer (car bufs))
-	      (progn
-		(set-buffer (car bufs))
-		(cond
-		 ((mew-summary-p)
-		  (mew-summary-mode-name mew-mode-name-summary))
-		 ((mew-virtual-p)
-		  (mew-summary-mode-name mew-mode-name-virtual)))))
-	  (setq bufs (cdr bufs)))))
+      (dolist (buf mew-buffers)
+	(when (get-buffer buf)
+	  (set-buffer buf)
+	  (cond
+	   ((mew-summary-p)
+	    (mew-summary-mode-name mew-mode-name-summary))
+	   ((mew-virtual-p)
+	    (mew-summary-mode-name mew-mode-name-virtual))))))
     (when mew-visit-inbox-after-setting-case
-      (setq inbox (mew-case-folder
-		   case
-		   (mew-proto-inbox-folder (mew-proto case) case)))
-      (mew-summary-visit-folder inbox))))
+      (let ((inbox (mew-case-folder
+		    case
+		    (mew-proto-inbox-folder (mew-proto case) case))))
+	(mew-summary-visit-folder inbox)))))
 
 (defun mew-draft-set-case (&optional arg)
   "Guess case and set the case for output to it. The value is
@@ -483,13 +534,13 @@ fields in the header according to the new value."
   (interactive "P")
   (let ((old-case (mew-tinfo-get-case)) new-case)
     (if arg
-	(setq new-case (mew-input-case old-case "This draft" 'edit))
+	(setq new-case (mew-input-case old-case 'edit))
       (setq new-case (mew-draft-get-case-by-guess))
       (if mew-case-guess-addition
 	  (setq new-case (mew-draft-add-case old-case new-case)))
-      (setq new-case
-	    (mew-input-case (or new-case mew-case-output) "This draft")))
+      (setq new-case (mew-input-case (or new-case mew-case))))
     (mew-tinfo-set-case new-case)
+    (mew-tinfo-set-use-flowed (mew-use-format-flowed new-case))
     (mew-draft-mode-name (mew-tinfo-get-hdr-file))
     (mew-draft-replace-fields old-case)
     (mew-highlight-header)
@@ -500,7 +551,7 @@ fields in the header according to the new value."
   (save-excursion
     (goto-char (point-min))
     (let ((new-case (mew-tinfo-get-case))
-	  from dcc eoh)
+	  from fcc dcc eoh)
       ;; (mew-header-end) cannot be used here
       (mew-header-goto-end)
       (setq eoh (point))
@@ -509,24 +560,28 @@ fields in the header according to the new value."
        ((mew-draft-resent-p eoh)
 	(setq from mew-resent-from:)
 	;; (setq cc mew-resent-cc:)
+	(setq fcc mew-resent-fcc:)
 	(setq dcc mew-resent-dcc:))
        (t
 	(setq from mew-from:)
 	;; (setq cc mew-cc:)
+	(setq fcc mew-fcc:)
 	(setq dcc mew-dcc:)))
       (mew-header-replace-value from (mew-from new-case))
       ;; (mew-header-replace-value cc (mew-cc new-case))
       (mew-header-replace-value dcc (mew-dcc new-case))
-      (mew-header-replace-value mew-fcc: (mew-fcc new-case))
+      (mew-header-replace-value fcc (mew-fcc new-case))
       (mew-header-replace-value mew-reply-to: (mew-reply-to new-case))
       (mew-header-replace-value mew-organization: (mew-organization new-case))
-      (mew-header-delete-lines (mapcar 'car (mew-header-alist old-case)))
-      (mew-header-delete-lines (mapcar 'car (mew-header-alist new-case)))
-      (mew-header-delete-lines (list mew-x-mailer:))
+      (mew-header-delete-lines (mapcar 'mew-alist-get-key (mew-header-alist old-case)))
+      (mew-header-delete-lines (mapcar 'mew-alist-get-key (mew-header-alist new-case)))
+      (mew-header-delete-lines (list mew-x-face: mew-x-mailer:))
       (mew-header-goto-end)
+      (mew-draft-header-insert-xface)
       (mew-draft-header-insert-alist (mew-header-alist new-case))
       ;; X-Mailer: must be the last
-      (mew-draft-header-insert mew-x-mailer: mew-x-mailer))))
+      (if (mew-use-x-mailer new-case)
+	  (mew-draft-header-insert mew-x-mailer: mew-x-mailer)))))
 
 (defun mew-draft-get-case-by-guess (&optional alist)
   "Guess case according to 'mew-case-guess-alist'."
@@ -556,13 +611,13 @@ fields in the header according to the new value."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2005 Mew developing team.
+;; Copyright (C) 1999-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -571,7 +626,7 @@ fields in the header according to the new value."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
