@@ -14,7 +14,8 @@
 
 (defvar mew-tinfo-list
   '("header-keymap" "attach-keymap" "case" "encrypted-p" "privacy-err"
-    "encode-err" "privacy-type" "hdr-file" "field-del" "other-frame"))
+    "encode-err" "privacy-type" "hdr-file" "field-del" "other-frame"
+    "preserved-header" "src-folder" "flowed" "use-flowed"))
 
 (mew-blinfo-defun 'mew-tinfo mew-tinfo-list)
 
@@ -44,7 +45,6 @@
   (setq comment-start mew-comment-start)
   (make-local-variable 'comment-start-skip)
   (setq comment-start-skip mew-comment-start-skip)
-  (make-local-hook 'after-change-functions)
   (add-hook 'after-change-functions 'mew-draft-dynamic-highlight nil 'local)
   (if (boundp 'write-file-functions)
       (add-hook 'write-file-functions 'mew-encode-make-backup nil 'local)
@@ -54,9 +54,7 @@
     (make-local-variable 'require-final-newline)
     (setq require-final-newline t))
   (when (featurep 'dnd)
-    (make-variable-buffer-local 'x-dnd-test-function)
-    (make-variable-buffer-local 'dnd-protocol-alist)
-    (setq x-dnd-test-function 'mew-draft-dnd-test-function)
+    (make-local-variable 'dnd-protocol-alist)
     (setq dnd-protocol-alist
 	  (append '(("^file:///" . mew-draft-dnd-handle-local-file)
 		    ("^file://"  . mew-draft-dnd-handle-file)
@@ -64,135 +62,12 @@
 		  dnd-protocol-alist))))
 
 (defun mew-draft-mode (&optional encrypted)
-  "\\<mew-draft-mode-map>
-Mew Draft mode:: major mode for composing a MIME message.
-Key actions are different in each region: Header, Body, and Attachment.
+  "A major mode for composing a MIME message.
 
-To send a draft, type \\[mew-draft-make-message] and \\[mew-draft-send-message].  To make multipart, type
-\\[mew-draft-prepare-attachments], edit attachments, type \\[mew-draft-make-message] and \\[mew-draft-send-message].
-
-*Whole buffer key assignment:
-
-\\[mew-draft-make-message]	Compose a MIME message then put it into a queue folder.
-\\[mew-draft-send-message]	Compose a MIME message then send it.
-
-\\[mew-draft-prepare-attachments]	Prepare an attachment region in the bottom of the draft.
-	To compose a multipart message, you should execute this 
-	command first.
-
-\\[mew-draft-set-case]	Guess case and set the case for output to it.
-\\[mew-draft-rehighlight]  Highlight header and body again.
-
-\\[mew-draft-kill]	Kill this draft.
-
-\\[mew-pgp-sign-message]	Sign the entire draft with PGP. Input your passphrase.
-\\[mew-pgp-encrypt-message]	Encrypt the entire draft with PGP.
-\\[mew-pgp-sign-encrypt-message]	Sign then encrypt the entire draft with PGP.
-	Input your passphrase.
-\\[mew-pgp-encrypt-sign-message]	Encrypt then sign the entire draft with PGP.
-	Input your passphrase.
-
-\\[mew-draft-toggle-privacy-always]	Toggle whether or not all drafts are protected.
-\\[mew-draft-toggle-privacy-encrypted]	Toggle whether or not drafts replying to encrypted messages 
-		are protected.
-\\[mew-draft-set-privacy-type]	Set privacy service which will be effective when \\[mew-draft-make-message].
-\\<mew-draft-header-map>
-*Header region key assignment:
-
-\\[mew-draft-header-comp]	Complete field keys.
-	Complete and expand an address short name.
-	Complete folder names.
-\\[mew-draft-circular-comp]	Complete your mail domain.
-\\[mew-draft-expand]	Replace an address with 'NAME <address>'.
-
-*Body region key assignment:
-
-\\<mew-draft-body-map>\\[mew-draft-insert-signature]	Insert '~/.signature' on the cursor point.
-\\<mew-draft-mode-map>\\[mew-draft-cite]	Copy and paste a part of message from Message mode with
-	citation prefix and label.
-	1. Roughly speaking, it copies the body in Message mode. 
-	   For example, if text/plain is displayed, the entire Message 
-	   mode is copied. If message/rfc822 is displayed, the body 
-	   without the header is copied.
-	2. If called with '\\[universal-argument]', the header is also copied if exists.
-	3. If an Emacs mark exists, the target is the region between 
-	   the mark and the cursor.
-\\[mew-draft-yank]	Copy and paste a part of message from Message mode WITHOUT
-	citation prefix and label.
-
-*Attachments region Key assignment:
-\\<mew-draft-attach-map>
-\\[mew-attach-forward]	Go to the first subdirectory.
-\\[mew-attach-backforward]	Go to the parent directory.
-\\[mew-attach-next]	Go to the next file in the current directory.
-\\[mew-attach-previous]	Go to the previous file in the current directory.
-
-\\[mew-attach-copy]	Copy a file (via networks) on '.'.
-	To copy a remote file, use the '/[user@]hostname:/filepath' syntax.
-\\[mew-attach-link]	Link a file with a symbolic link on '.'.
-\\[mew-attach-link-message]	Link the message displayed in Message mode with a symbolic link on '.'.
-\\[mew-attach-delete]	Delete this file or this directory.
-\\[mew-attach-multipart]	Create a subdirectory(i.e. multipart) on '.'.
-\\[mew-attach-find-file]	Open this file into a buffer.
-\\[mew-attach-find-new-file]	Open a new file into a buffer on '.'.
-\\[mew-attach-external-body]	Input external-body on '.'.
-\\[mew-attach-audio]	Sampling voice and insert as audio file on '.'.
-\\[mew-attach-pgp-public-key]	Extract the PGP key for the specified user on '.'.
-\\[mew-attach-description]	Input a description(Content-Description:).
-\\[mew-attach-disposition]	Change the file name(Content-Disposition:).
-\\[mew-attach-type]	Change the data type(Content-Type:).
-\\[mew-attach-charset]	Specify the charset parameter for a Text/* object.
-\\[mew-attach-icharset]	Specify a input coding-system for a text file.
-
-\\[mew-attach-base64]	Put the 'B' mark to encode with Base64.
-\\[mew-attach-quoted-printable]	Put the 'Q' mark to encode with Quoted-Printable.
-\\[mew-attach-gzip64]	Put the 'G' mark to encode with Gzip64. This is applicable 
-	only to Text/Plain and Application/Postscript since compression 
-	is not effective other objects. For example, JPEG is already 
-	compressed.
-\\[mew-attach-pgp-sign]	Put the 'PS' mark to sign with PGP.
-\\[mew-attach-pgp-enc]	Put the 'PE' mark to encrypt with PGP. 
-	Input decryptors' addresses.
-\\[mew-attach-undo]	Unmark. The original mark appears.
-
-* Fill blanks
-\\<mew-draft-mode-map>
-Prepare '~/.mew-fib' like;
-
-	name:  Kazuhiko Yamamoto
-	email: kazu@example.org
-
-If you receive a message like;
-
-	Your name : |>name<|
-	Your e-mail address: |>email<|
-
-Type \\<mew-summary-mode-map>\\[mew-summary-reply] in Summary mode to enter Draft mode. 
-Then type \\<mew-draft-mode-map>\\[mew-draft-yank], \\[mew-fib-fill-default], and \\[mew-fib-delete-frame] makes following
-draft.
-
-	Your name : Kazuhiko Yamamoto
-	Your e-mail address: kazu@example.org
-
-In this way, mew-fil fills up items quoted like |> <| from '~/.mew-fib'.
-The fill functions described below.
-
-\\[mew-fib-fill-default]	Fill |>item<| from '~/.mew-fib'.
-\\[mew-fib-delete-frame]	Delete all quotations, i.e. |> <|.
-\\[mew-fib-next-item]	Jump to the next fib item.
-\\[mew-fib-previous-item]	Jump to the previous fib item.
-\\[mew-fib-flush-input]	Flush input from '~/.mew-fib'.
-
-Moreover, '~/.mew-fib' supports aliases like;
-
-	email: kazu@example.org
-	e-mail:
-
-"
+\\{mew-draft-mode-map}"
   (interactive)
   (setq major-mode 'mew-draft-mode)
   (setq mode-line-buffer-identification mew-mode-line-id)
-  (mew-draft-mode-name)
   (mew-draft-set-local-variables)
   (use-local-map mew-draft-mode-map)
   (set-syntax-table mew-draft-mode-syntax-table)
@@ -202,6 +77,8 @@ Moreover, '~/.mew-fib' supports aliases like;
   (mew-tinfo-set-encrypted-p encrypted)
   (mew-tinfo-set-privacy-err nil)
   (mew-tinfo-set-privacy-type nil)
+  (mew-tinfo-set-use-flowed (mew-use-format-flowed (mew-tinfo-get-case)))
+  (mew-draft-mode-name) ;; must be after (mew-tinfo-set-encrypted-p encrypted)
   (run-hooks 'text-mode-hook 'mew-draft-mode-hook)
   ;; auto-fill-function is set by mew-draft-mode-hook
   (when auto-fill-function
@@ -210,22 +87,25 @@ Moreover, '~/.mew-fib' supports aliases like;
   (setq buffer-undo-list nil))
 
 (defun mew-draft-mode-name (&optional header)
-  (let (pcdb sub)
+  (let ((case (mew-tinfo-get-case))
+	pcdb sub)
     (cond
      ((or (mew-tinfo-get-privacy-type) (mew-tinfo-get-privacy-err))
       ;; If privacy err, don't display mew-protect-privacy-always-type etc.
       (setq pcdb (mew-pcdb-by-service (mew-tinfo-get-privacy-type)))
       (setq sub (mew-pcdb-mark pcdb)))
-     ((and (mew-tinfo-get-encrypted-p) mew-protect-privacy-encrypted)
-      (setq pcdb (mew-pcdb-by-service mew-protect-privacy-encrypted-type))
+     ((and (mew-tinfo-get-encrypted-p) (mew-protect-privacy-encrypted case))
+      (setq pcdb (mew-pcdb-by-service (mew-protect-privacy-encrypted-type case)))
       (setq sub (mew-pcdb-mark pcdb)))
-     (mew-protect-privacy-always
-      (setq pcdb (mew-pcdb-by-service mew-protect-privacy-always-type))
+     ((mew-protect-privacy-always case)
+      (setq pcdb (mew-pcdb-by-service (mew-protect-privacy-always-type case)))
       (setq sub (mew-pcdb-mark pcdb))))
     (setq mode-name (if header mew-mode-name-header mew-mode-name-draft))
     (if sub (setq mode-name (concat mode-name " " sub)))
     (unless (mew-case-default-p (mew-tinfo-get-case))
       (setq mode-name (concat mode-name " " (mew-tinfo-get-case))))
+    (if (mew-tinfo-get-use-flowed)
+	(setq mode-name (concat mode-name " F")))
     (force-mode-line-update)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -274,9 +154,8 @@ Moreover, '~/.mew-fib' supports aliases like;
       (setq switch-func 'switch-to-buffer))
      ((eq switch-func t)
       (setq switch-func 'switch-to-buffer-other-window)))
-    (mew-frwlet
-     mew-cs-m17n mew-cs-dummy
-     (funcall switch-func (mew-find-file-noselect draft-path)))
+    (mew-frwlet mew-cs-m17n mew-cs-dummy
+      (funcall switch-func (mew-find-file-noselect draft-path)))
     ;; draft buffer
     (mew-set-buffer-cs mew-cs-m17n)
     ;; copy config, first
@@ -299,10 +178,9 @@ Moreover, '~/.mew-fib' supports aliases like;
 the Body: field."
   (let ((case-fold-search t)
 	key val ret)
-    (while halist
-      (setq key (car (car halist)))
-      (setq val (cdr (car halist)))
-      (setq halist (cdr halist))
+    (dolist (ent halist)
+      (setq key (mew-alist-get-key ent))
+      (setq val (mew-alist-get-value ent))
       (unless (string-match ":$" key)
 	(setq key (concat key ":")))
       (if (string-match mew-body: key)
@@ -311,9 +189,8 @@ the Body: field."
     ret))
 
 (defun mew-insert-address-list (field adrs del force-insert)
-  (let ((cnt 0) (beg (point)) med adr)
-    (while adrs
-      (setq adr (car adrs) adrs (cdr adrs))
+  (let ((cnt 0) (beg (point)) med)
+    (dolist (adr adrs)
       (unless (mew-is-my-address del adr)
 	(if (= cnt 0)
 	    (insert adr)
@@ -336,9 +213,8 @@ the Body: field."
       (setq med (point))
       (insert (car adrs))
       (setq adrs (cdr adrs))
-      (while adrs
-	(insert ", " (car adrs))
-	(setq adrs (cdr adrs)))
+      (dolist (adr adrs)
+	(insert ", " adr))
       (insert "\n")
       (mew-header-fold-region beg (point) med 'use-tab))))
 
@@ -373,7 +249,7 @@ the Body: field."
        ((listp to)
 	(setq del (mew-insert-address-list mew-to: to del t))))
       (cond
-       ((null cc) ()) ;; do nothing 
+       ((null cc) ()) ;; do nothing
        ((stringp cc) ;; Cc: specified from the mini-buffer.
 	(mew-draft-header-insert mew-cc: cc))
        ((listp cc) ;; Cc: collected by reply.
@@ -393,22 +269,16 @@ the Body: field."
     (unless newsgroups
       (mew-draft-header-insert mew-in-reply-to: in-reply-to))
     (mew-draft-header-insert mew-references: references)
-    (if (and mew-x-face-file
-	     (file-exists-p (expand-file-name mew-x-face-file)))
-	(let (xface)
-	  (with-temp-buffer
-	    (mew-insert-file-contents (expand-file-name mew-x-face-file))
-	    (setq xface (mew-buffer-substring (point-min)
-					      (max (buffer-size) 1))))
-	  (mew-draft-header-insert mew-x-face: xface)))
+    (mew-draft-header-insert-xface)
     (mew-draft-header-insert mew-organization: (mew-organization case))
     (setq body (mew-draft-header-insert-alist other-headers))
     ;; Deleting fields defined in mew-header-alist to replace them.
-    (mew-header-delete-lines (mapcar 'car (mew-header-alist case)))
+    (mew-header-delete-lines (mapcar 'mew-alist-get-key (mew-header-alist case)))
     (mew-header-goto-end)
     (mew-draft-header-insert-alist (mew-header-alist case))
     ;; X-Mailer: must be the last
-    (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
+    (if (mew-use-x-mailer case)
+	(mew-draft-header-insert mew-x-mailer: mew-x-mailer))
     ;; (mew-header-set "\n") is enough. But highlighting delayed.
     (mew-header-prepared)
     ;; on the body
@@ -418,6 +288,16 @@ the Body: field."
     (goto-char (point-min))
     (search-forward ": " nil t)))
 
+(defun mew-draft-header-insert-xface ()
+  (if (and mew-x-face-file
+	   (file-exists-p (expand-file-name mew-x-face-file)))
+      (let (xface)
+	(with-temp-buffer
+	  (mew-insert-file-contents (expand-file-name mew-x-face-file))
+	  (setq xface (mew-buffer-substring (point-min)
+					    (max (buffer-size) 1))))
+	(mew-draft-header-insert mew-x-face: xface))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Citation
@@ -440,7 +320,7 @@ citation prefix and label.
    if text/plain is displayed, the entire Message mode is copied.
    If message/rfc822 is displayed, the body without the header is copied.
 2. If called with '\\[universal-argument]', the header is also copied if exists.
-3. If an Emacs mark exists, the target is the region between the mark and 
+3. If an Emacs mark exists, the target is the region between the mark and
    the cursor."
 ;; MUST take care of C-x C-x
 ;; MUST be able to cancel by C-x u
@@ -459,7 +339,7 @@ citation prefix and label.
 	  (widen)
 	  (let ((mark-active t))
 	    (cond
-	     (arg 
+	     (arg
 	      (setq beg (point-min) end (point-max)))
 	     ((and (not mew-cite-ignore-region)
 		   (mew-mark)
@@ -491,7 +371,7 @@ citation prefix and label.
    if text/plain is displayed, the entire Message mode is copied.
    If message/rfc822 is displayed, the body without the header is copied.
 2. If called with '\\[universal-argument]', the header is also copied if exists.
-3. If an Emacs mark exists, the target is the region between the mark and 
+3. If an Emacs mark exists, the target is the region between the mark and
    the cursor."
 ;; MUST take care of C-x C-x
 ;; MUST be able to cancel by C-x u
@@ -499,7 +379,7 @@ citation prefix and label.
   (if (and (not force) (or (mew-in-header-p) (mew-in-attach-p)))
       (message "Cannot cite a message here")
     (let* ((nonmewbuf mew-message-citation-buffer) ;; may be buffer local
-	   (fid (or mew-message-citation-frame-id (mew-frame-id))) 
+	   (fid (or mew-message-citation-frame-id (mew-frame-id)))
 	   (fld (mew-current-get-fld fid))
 	   (msg (mew-current-get-msg fid))
 	   (msg-buf (mew-buffer-message))
@@ -550,11 +430,11 @@ citation prefix and label.
 	(save-restriction
 	  (widen)
 	  (mew-header-goto-end)
-	  (setq cite (concat (mew-buffer-substring (point-min) (point)) 
+	  (setq cite (concat (mew-buffer-substring (point-min) (point))
 			     "\n" cite))
           (setq irt-msgid (mew-idstr-get-first-id
 			   (mew-header-get-value mew-message-id:)))))
-      ;; 
+      ;;
       ;; Draft mode, insert the header and the body.
       ;;
 
@@ -573,7 +453,7 @@ citation prefix and label.
 	      (mew-draft-header-insert mew-in-reply-to: irtstr))))
       (save-restriction
 	;; this gets complicated due to supercite, please do not care
-	(narrow-to-region (point)(point)) ;; for (goto-char (point-min))
+	(narrow-to-region (point) (point)) ;; for (goto-char (point-min))
 	(insert cite)
 	;; not for C-x C-x. Do not use mew-push-mark.
 	(push-mark (point) t t)
@@ -587,6 +467,8 @@ citation prefix and label.
       (or force (mew-highlight-body-region (mark-marker) (point) 'draft))
       (mew-draft-auto-set-input-method))))
 
+(defconst mew-cite-default-prefix "> ")
+
 (defun mew-cite-original (&optional arg)
   (if (< (marker-position (mark-marker)) (point))
       (exchange-point-and-mark))
@@ -598,23 +480,38 @@ citation prefix and label.
           (setq label (funcall mew-cite-strings-function))
         (error
 	 (error "Syntax of mew-cite-format was changed. Read explanation of mew-cite-fields")))
-      (if (null mew-cite-prefix-function)
-          (setq prefix mew-cite-prefix)
-        (setq prefix (funcall mew-cite-prefix-function)))
-      (if mew-cite-prefix-confirmp
-          (let ((ask (read-string 
+      (cond
+       (mew-cite-prefix-function
+	(setq prefix (funcall mew-cite-prefix-function)))
+       (mew-cite-prefix
+	(setq prefix mew-cite-prefix))
+       (t
+	(setq prefix mew-cite-default-prefix)))
+      (if (and mew-cite-prefix-confirmp (not mew-use-format-flowed))
+          (let ((ask (read-string
                       (format "Prefix (\"%s\"): " prefix) "")))
             (if (not (string= ask "")) (setq prefix ask))))
       ;; C-u C-c C-y cites body with header.
-      (if (eq arg nil) 
+      (if (eq arg nil)
 	  ;; header has been already cited. So, delete it.
 	  (delete-region beg (progn (mew-header-goto-body) (point))))
       (insert label)
       (mew-push-mark)
-      (and (bolp) (insert prefix))
-      (while (= 0 (forward-line))
-	(or (= (point) (point-max))
-	    (insert prefix))))))
+      (if (or mew-cite-prefix-function mew-cite-prefix)
+	  (progn
+	    (and (bolp) (insert prefix))
+	    (while (= 0 (forward-line))
+	      (or (= (point) (point-max))
+		  (insert prefix))))
+	(if (bolp) (mew-cite-format-flowed))
+	(while (= 0 (forward-line))
+	  (unless (= (point) (point-max))
+	    (mew-cite-format-flowed)))))))
+
+(defun mew-cite-format-flowed ()
+  (insert mew-flowed-quoted)
+  (unless (char-equal (char-after) mew-flowed-quoted)
+    (insert mew-flowed-stuffed)))
 
 (defun mew-cite-get-value (field)
   (let ((value (mew-header-get-value field))
@@ -626,7 +523,7 @@ citation prefix and label.
     (or value "")))
 
 (defun mew-cite-strings ()
-  "A function to create cite labels according to 
+  "A function to create cite labels according to
 'mew-cite-format' and 'mew-cite-fields'."
   (if (null mew-cite-fields)
       ""
@@ -646,7 +543,7 @@ citation prefix and label.
 	    (while (not (eobp))
 	      (cond
 	       ((eq mew-draft-cite-fill-mode 'truncate)
-		(end-of-line)            
+		(end-of-line)
 		(if (>= fill-column (current-column))
 		    ()
 		  (setq eol (point))
@@ -659,9 +556,7 @@ citation prefix and label.
 		(end-of-line)
 		(if (= (current-column) 0)
 		    ()
-		  (fill-region beg (point))
-		  (if (= (current-column) 0) ;; for XEmacs
-		      (delete-backward-char 1)))))
+		  (fill-region beg (point)))))
 	      (forward-line)))
 	  (buffer-string))))))
 
@@ -678,6 +573,38 @@ The citation style is 'from_address> ', e.g. 'kazu> '"
 	(setq prefix (read-string "Citation prefix: " prefix)))
     (concat prefix mew-cite-prefix)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; format=flowed
+;;;
+
+(defun mew-draft-encode-flowed (&optional arg)
+  "Manually encode the body with format=flowed.
+If called with '\\[universal-argument]', toggle whether or not
+format=flowed is used on composing."
+  (interactive "P")
+  (if arg
+      (progn
+	(mew-tinfo-set-use-flowed (not (mew-tinfo-get-use-flowed)))
+	(mew-draft-mode-name))
+    (save-excursion
+      (goto-char (mew-header-end))
+      (forward-line)
+      (if (mew-tinfo-get-flowed)
+	  (progn
+	    (mew-decode-flowed (point) (point-max)
+			       (if (string= (mew-tinfo-get-flowed) "yes") t nil))
+	    (mew-tinfo-set-flowed nil))
+	(let* ((charset (mew-charset-guess-region (point) (point-max)))
+	       (flowed-delsp (mew-encode-flowed (point) (point-max) charset))
+	       flowed delsp)
+	  (mew-set '(flowed delsp) flowed-delsp)
+	  (if (not flowed)
+	      (message "No line folded")
+	    (mew-tinfo-set-flowed (if delsp "yes" "no")))))
+      (mew-draft-rehighlight)
+      (setq buffer-undo-list nil))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Misc
@@ -719,7 +646,7 @@ The citation style is 'from_address> ', e.g. 'kazu> '"
 (defun mew-draft-insert-signature (&optional arg)
   "Insert the signature file specified by mew-signature-file.
 If attachments exist and mew-signature-as-lastpart is *non-nil*,
-the file is attached to the last part. Otherwise, the file is 
+the file is attached to the last part. Otherwise, the file is
 inserted into the body. If mew-signature-insert-last is *non-nil*,
 the file is inserted to the end of the body. Otherwise, inserted
 the cursor position. If executed with '\\[universal-argument]',
@@ -747,7 +674,7 @@ you can set the case."
 	      (mew-syntax-set-cdp syntax nil)
 	      (mew-syntax-set-cd  syntax mew-signature-description))
 	    (mew-encode-syntax-print mew-encode-syntax))
-	(when mew-signature-insert-last 
+	(when mew-signature-insert-last
 	  (if (null (mew-attach-p))
 	      (goto-char (point-max))
 	    (goto-char (1- (mew-attach-begin))))
@@ -758,7 +685,7 @@ you can set the case."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Re-highlight
-;;; 
+;;;
 
 (defun mew-draft-rehighlight-body ()
   (save-excursion
@@ -789,7 +716,7 @@ you can set the case."
   (mew-draft-mode-name))
 
 (defun mew-draft-toggle-privacy-encrypted ()
-  "Toggle whether or not drafts replying to encrypted messages are 
+  "Toggle whether or not drafts replying to encrypted messages are
 protected."
   (interactive)
   (setq mew-protect-privacy-encrypted (not mew-protect-privacy-encrypted))
@@ -818,20 +745,24 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 (defun mew-draft-make-message (&optional privacy signer)
   "Compose a MIME message then put it into a queue folder."
   (interactive)
-  (if (and (interactive-p) ;; prevent the loop
-	   mew-use-old-pgp
-	   mew-protect-privacy-with-old-pgp-signature)
-      (mew-pgp-sign-message)
-    (mew-draft-process-message 'queue privacy signer)))
+  (if (string= mode-name "Edit")
+      (mew-edit-make)
+    (if (and (interactive-p) ;; prevent the loop
+	     mew-use-old-pgp
+	     mew-protect-privacy-with-old-pgp-signature)
+	(mew-pgp-sign-message)
+      (mew-draft-process-message 'queue privacy signer))))
 
 (defun mew-draft-send-message ()
   "Compose a MIME message then send it."
   (interactive)
-  (if (and (interactive-p) ;; just in case
-	   mew-use-old-pgp
-	   mew-protect-privacy-with-old-pgp-signature)
-      (mew-pgp-sign-message)
-    (mew-draft-process-message 'send)))
+  (if (string= mode-name "Edit")
+      (mew-edit-make)
+    (if (and (interactive-p) ;; just in case
+	     mew-use-old-pgp
+	     mew-protect-privacy-with-old-pgp-signature)
+	(mew-pgp-sign-message)
+      (mew-draft-process-message 'send))))
 
 (defun mew-draft-process-message (action &optional privacy signer)
   (run-hooks 'mew-make-message-hook)
@@ -991,64 +922,65 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 ;;; Privacy
 ;;;
 
-(defvar mew-draft-privacy-method 'pgp)
+(defvar mew-draft-privacy-method-alist '(("pgp" . pgp) ("smime" . smime)))
+
+(defun mew-draft-set-privacy-method ()
+  "Set mew-draft-privacy-method. 'pgp or 'smime."
+  (interactive)
+  (let ((method (completing-read "Privacy method: " mew-draft-privacy-method-alist nil t)))
+    (setq mew-draft-privacy-method
+	  (cdr (assoc method mew-draft-privacy-method-alist)))))
+
+(defmacro mew-draft-privacy-switch (&rest form)
+  `(let ((method (mew-draft-privacy-method (mew-tinfo-get-case))))
+     (cond
+      ,@(mapcar
+	 (lambda (x)
+	   (if (eq (car x) t)
+	       x
+	     `((eq method ',(car x)) ,(car (cdr x)))))
+	 form)
+      (t (message "'%s' is not supported" method)))))
 
 (defun mew-draft-sign-message (&optional arg)
   "Sign the entire draft. Input your passphrase."
   (interactive "P")
-  (cond
-   ((eq mew-draft-privacy-method 'pgp)
-    (mew-pgp-sign-message arg))
-   ((eq mew-draft-privacy-method 'smime)
-    (mew-smime-sign-message arg))
-   (t
-    (message "'%s' is not supported" mew-draft-privacy-method))))
+  (mew-draft-privacy-switch
+   (pgp   (mew-pgp-sign-message arg))
+   (smime (mew-smime-sign-message arg))))
 
 (defun mew-draft-encrypt-message ()
   "Encrypt the entire draft with PGP."
   (interactive)
-  (cond
-   ((eq mew-draft-privacy-method 'pgp)
-    (mew-pgp-encrypt-message))
-;;   ((eq mew-draft-privacy-method 'smime)
-;;    (mew-smime-encrypt-message))
-   (t
-    (message "'%s' is not supported" mew-draft-privacy-method))))
+  (mew-draft-privacy-switch
+   (pgp   (mew-pgp-encrypt-message))
+   (smime (mew-smime-encrypt-message))))
 
 (defun mew-draft-sign-encrypt-message (&optional arg)
   "Sign then encrypt the entire draft. Input your passphrase."
   (interactive "P")
-  (cond
-   ((eq mew-draft-privacy-method 'pgp)
-    (mew-pgp-sign-encrypt-message arg))
-;;   ((eq mew-draft-privacy-method 'smime)
-;;    (mew-smime-sign-encrypt-message arg))
-   (t
-    (message "'%s' is not supported" mew-draft-privacy-method))))
-
+  (mew-draft-privacy-switch
+   (pgp   (mew-pgp-sign-encrypt-message arg))
+   (smime (mew-smime-sign-encrypt-message arg))))
 
 (defun mew-draft-encrypt-sign-message (&optional arg)
   "Encrypt then sign the entire draft. Input your passphrase."
   (interactive "P")
-  (cond
-   ((eq mew-draft-privacy-method 'pgp)
-    (mew-pgp-encrypt-sign-message arg))
-;;   ((eq mew-draft-privacy-method 'smime)
-;;    (mew-smime-encrypt-sign-message arg))
-   (t
-    (message "'%s' is not supported" mew-draft-privacy-method))))
+  (mew-draft-privacy-switch
+   (pgp   (mew-pgp-encrypt-sign-message arg))
+   (smime (mew-smime-encrypt-sign-message arg))))
 
 (provide 'mew-draft)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2006 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1057,7 +989,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
