@@ -682,7 +682,7 @@ See also mew-summary-edit-again."
       (t
        (if part
 	   (let* ((cache (mew-cache-hit fld msg 'must-hit))
-		  (alt (mew-cache-dinfo-get-use-alt cache))
+		  (alt nil)
 		  (syntax (mew-cache-decode-syntax cache))
 		  (stx (mew-syntax-get-entry syntax part))
 		  (ct (mew-syntax-get-value (mew-syntax-get-ct stx) 'cap)))
@@ -895,7 +895,7 @@ mew-summary-reedit."
       (set-buffer buf)
       (mew-erase-buffer)
       (mew-insert-message fld msg mew-cs-text-for-read nil)
-      (mew-dinfo-set (1+ (length part)) 'no-cs-conv t alt)
+      (mew-dinfo-set (1+ (length part)) 'no-cs-conv t alt 'encap-html)
       (mew-decode-for-edit)
       (setq syntax (mew-syntax-get-entry mew-decode-syntax part))
       ;; syntax is now for the message
@@ -1031,6 +1031,145 @@ mew-summary-reedit."
     (mew-syntax-set-cdp syntax nil)
     (mew-syntax-set-part syntax nil)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Edit mode
+;;;
+
+(defun mew-summary-edit ()
+  (interactive)
+  (mew-summary-not-in-queue
+   (mew-summary-not-in-draft
+    (mew-summary-msg-or-part
+     (let ((fld (mew-summary-folder-name))
+	   (msg (mew-summary-message-number2))
+	   (part (mew-syntax-nums)))
+       (if part
+	   (let* ((cache (mew-cache-hit fld msg 'must-hit))
+		  (alt nil)
+		  (syntax (mew-cache-decode-syntax cache))
+		  (stx (mew-syntax-get-entry syntax part))
+		  (ct (mew-syntax-get-value (mew-syntax-get-ct stx) 'cap)))
+	     (if (not (string= mew-ct-msg ct))
+		 (message "Cannot edit here")
+	       (mew-summary-edit-for-message fld msg part alt)
+	       (run-hooks 'mew-draft-mode-edit-hook)))
+	 (mew-summary-edit-for-message fld msg nil nil)
+	 (setq mode-name "Edit")
+	 (force-mode-line-update)
+	 (run-hooks 'mew-draft-mode-edit-hook)))))))
+
+(defun mew-summary-edit-for-message (fld msg part alt)
+  (mew-current-set-window-config)
+  (mew-window-configure 'draft)
+  ;; main part
+  (mew-summary-prepare-draft
+   ;; case is copied here
+   (mew-summary-edit-message fld msg part alt)
+   ;; get info before backup the original
+   (mew-summary-edit-header-for-edit)
+   (let ((fields (mew-summary-edit-obtain-fields-key)))
+     (mew-tinfo-set-preserved-header (mew-summary-edit-obtain-fields fld msg fields)))
+   (mew-tinfo-set-src-folder fld)
+   (mew-draft-mode)
+   (mew-draft-rehighlight-body)
+   (when mew-encode-syntax
+     (save-excursion
+       (mew-draft-prepare-attachments t)))
+   (mew-draft-mode-name)))
+
+(defun mew-summary-edit-header-for-edit ()
+  (mew-elet
+   (mew-header-delete-lines '("From " ">From"))
+   (mew-header-delete-lines mew-field-delete-for-editing)
+   (mew-header-sort mew-field-order-for-editing)
+   (mew-header-clear) ;; erase the old header separator
+   (mew-header-set "\n")))
+
+(defun mew-summary-edit-obtain-fields-key ()
+  (let ((case-fold-search t) fields)
+    (save-excursion
+      (mew-header-goto-end)
+      (save-restriction
+	(narrow-to-region (point-min) (point))
+	(goto-char (point-min))
+	(while (not (eobp))
+	  (if (not (looking-at mew-keyval))
+	      (forward-line)
+	    (setq fields (cons (mew-match-string 1) fields))
+            (forward-line)
+	    (mew-header-goto-next))))
+      fields)))
+
+(defun mew-summary-edit-obtain-fields (fld msg fields)
+  (with-temp-buffer
+    (mew-insert-message
+     fld msg mew-cs-text-for-read mew-header-reasonable-size)
+    (mew-header-goto-end)
+    (delete-region (point) (point-max))
+    (mew-header-delete-lines fields)
+    (mew-header-delete-lines (list mew-mv: "Content-" "X-Mew-"))
+    (mew-buffer-substring (point-min) (point-max))))
+
+(defun mew-edit-make-delete-window (buf)
+  (if (and (mew-tinfo-get-other-frame) (> (length (frame-list)) 1))
+      (delete-frame)
+    (mew-current-get-window-config)
+    (delete-windows-on buf))) ;; just in case
+
+(defun mew-edit-make-message (buf)
+  (let (multip)
+    (save-excursion
+      (save-window-excursion
+	(set-buffer buf)
+	(mew-set-buffer-multibyte t)
+	(if (buffer-modified-p) (save-buffer)) ;; to make backup
+	(widen)
+	(if (mew-attach-p)
+	    (progn
+	      (setq multip t)
+	      (mew-attach-clear))
+	  (unless mew-encode-syntax
+	    (setq mew-encode-syntax
+		  (mew-encode-syntax-single "text.txt" (list mew-ct-txt)))))
+	(goto-char (mew-header-end))
+	(forward-line)
+	(if multip
+	    (mew-encode-make-multi)
+	  (mew-encode-make-single))
+	(mew-encode-make-header)
+	(mew-encode-save-draft)
+	(mew-overlay-delete-buffer)
+	(goto-char (point-min))
+	(insert (mew-tinfo-get-preserved-header))
+	(save-buffer)
+	(buffer-file-name)))))
+
+(defun mew-edit-make-refile (case:srcfld srcfile)
+  (let* ((case:dstfld (car (mew-input-folders case:srcfld)))
+	 (dstfld (mew-case:folder-folder case:dstfld))
+	 (case (mew-case:folder-case case:dstfld))
+	 dstfile)
+    (cond
+     ((mew-folder-imapp dstfld)
+      (mew-imap2-copy-message case 'move (list srcfile) dstfld))
+     (t
+      (setq dstfile (mew-folder-new-message dstfld))
+      (rename-file srcfile dstfile)
+      (mew-set-file-modes dstfile)
+      (mew-touch-folder dstfld)
+      (message "Saved in %s" dstfld)))))
+
+(defun mew-edit-make ()
+  (interactive)
+  (let ((buf (current-buffer))
+	(case:srcfld (mew-tinfo-get-src-folder))
+	srcfile)
+    (mew-edit-make-delete-window buf)
+    (setq srcfile (mew-edit-make-message buf))
+    (mew-remove-buffer buf)
+    (mew-edit-make-refile case:srcfld srcfile)))
+
 (provide 'mew-edit)
 
 ;;; Copyright Notice:
