@@ -7,6 +7,22 @@
 
 (require 'mew)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Suppressing warnings at compile
+;;;
+
+(defmacro mew-no-waring-defvar (var-name)
+  `(unless (boundp ',var-name) (defvar ,var-name nil)))
+
+(defmacro mew-no-waring-defun (func-name)
+  `(unless (fboundp ',func-name) (defun ,func-name(&rest args) nil)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;;
+;;;
+
 ;; to avoid competition with mh-e.el, sigh.
 (let ((ent (rassq 'mh-letter-mode auto-mode-alist)))
   (and ent (setq auto-mode-alist (delq ent auto-mode-alist))))
@@ -67,11 +83,20 @@ requires PTY.")
     (defalias 'mew-minibuf-point-min 'minibuffer-prompt-end)
   (defalias 'mew-minibuf-point-min 'point-min))
 
+(eval-when-compile
+  (unless (fboundp 'with-no-warnings)
+    (defmacro with-no-warnings (&rest body) `(progn ,@body))))
+
 (if (fboundp 'set-process-query-on-exit-flag)
     (defun mew-process-silent-exit (pro)
       (set-process-query-on-exit-flag pro nil))
-  (defun mew-process-silent-exit (pro)
-    (process-kill-without-query pro))) ;; Emacs 21.4
+  (with-no-warnings
+    (defun mew-process-silent-exit (pro)
+      (process-kill-without-query pro)))) ;; Emacs 21.4
+
+(with-no-warnings
+  (defun mew-set-coding-priority (pri)
+    (set-coding-priority pri)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -106,7 +131,7 @@ requires PTY.")
 (defun mew-overlay-delete-region (beg end)
   "Delete overlays in the region."
   (interactive "r")
-  (mapcar 'mew-overlay-delete (overlays-in beg end))
+  (mapc 'mew-overlay-delete (overlays-in beg end))
   (if (fboundp 'remove-images) (remove-images beg end))) ;; xxx
 
 (defun mew-overlay-delete-buffer ()
@@ -188,6 +213,19 @@ requires PTY.")
        (string= current-language-environment "Korean"))
   (require 'mew-lang-kr)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Unix and Mac
+;;;
+
+(if (fboundp 'unix-sync)
+    (defalias 'mew-unix-sync 'unix-sync)
+  (defmacro mew-unix-sync (&rest args) nil))
+
+(if (fboundp 'mac-set-file-type)
+    (defalias 'mew-mac-set-file-type 'mac-set-file-type)
+  (defmacro mew-mac-set-file-type (&rest args) nil))
+
 (provide 'mew-env)
 
 ;;; Copyright Notice:
