@@ -21,21 +21,18 @@
 
 (defun mew-mark-exec-refile (src msgs)
   "Refile MSGs from the SRC folder."
-  (let (dsts tmp msg msg-dsts dsts-msgses)
-    (while msgs
-      (setq msg (car msgs))
-      (setq msgs (cdr msgs))
+  (let (dsts tmp msg-dsts dsts-msgses)
+    (dolist (msg msgs)
       (setq msg-dsts (mew-refile-get msg))
       (setq dsts (cdr msg-dsts))
       (if dsts ;; sanity check
 	  (if (setq tmp (assoc dsts dsts-msgses))
-	      (nconc tmp (list msg))	      
+	      (nconc tmp (list msg))	
 	    (setq dsts-msgses (cons (list dsts msg) dsts-msgses))))
       (mew-refile-reset msg))
     ;; refile at once
-    (while dsts-msgses
-      (mew-summary-mv-msgs src (car dsts-msgses))
-      (setq dsts-msgses (cdr dsts-msgses)))))
+    (dolist (dsts-msgs dsts-msgses)
+      (mew-summary-mv-msgs src dsts-msgs))))
 
 (defun mew-mark-exec-unlink (src dels)
   "Unlink DELS from the SRC folder.
@@ -48,7 +45,7 @@ DELS represents the messages to be deleted."
 and 'mew-trash-folder-list'.
 DELS represents the messages to be deleted."
   (when dels
-    ;; see also mew-summary-exec-remote-get-info.
+    ;; see also mew-summary-exec-remote-get-execinfo.
     (cond
      ((null mew-trash-folder)
       (mew-summary-unlink-msgs src dels))
@@ -69,19 +66,13 @@ DELS represents the messages to be deleted."
 
 (defun mew-mark-sanity-refile (msgs)
   "Check the destination folder for MSGS."
-  (let (msg dst dsts uniq-dsts udst err-dsts dir)
-    (while msgs
-      (setq msg (car msgs))
-      (setq msgs (cdr msgs))
-      (setq dsts (cdr (mew-refile-get msg)))
-      (while dsts
-	(setq dst (car dsts))
-	(setq dsts (cdr dsts))
+  (let (uniq-dsts udst err-dsts dir)
+    (dolist (msg msgs)
+      (dolist (dst (cdr (mew-refile-get msg)))
 	(mew-addq uniq-dsts dst)))
     (mew-addq uniq-dsts mew-trash-folder)
-    (while uniq-dsts
-      (setq udst (mew-case-folder mew-inherit-exec-case (car uniq-dsts)))
-      (setq uniq-dsts (cdr uniq-dsts))
+    (dolist (uniq-dst uniq-dsts)
+      (setq udst (mew-case-folder mew-inherit-exec-case uniq-dst))
       (setq dir (mew-expand-folder udst))
       (if (file-exists-p dir)
 	  (if (file-directory-p dir)
@@ -105,9 +96,8 @@ DELS represents the messages to be deleted."
 (defun mew-summary-unlink-msgs (src dels)
   (let ((suffix (mew-folder-suffix src))
 	file infofile)
-    (while dels
-      (setq file (mew-expand-msg src (car dels)))
-      (setq dels (cdr dels))
+    (dolist (del dels)
+      (setq file (mew-expand-msg src del))
       ;; if file does not exist, the marked line will be deleted anyway.
       (mew-delete-file file)
       (when suffix
@@ -126,14 +116,12 @@ DELS represents the messages to be deleted."
 	 (myselfp (member srcx dsts))
 	 (cache mew-inherit-offline)
 	 (suffix (mew-folder-suffix srcx))
-	 msgs- msg srcfile dstfile dst num strnum infofile)
+	 msgs- srcfile dstfile dst num strnum infofile)
     (if myselfp
 	;; msg stays in the src folder with the same number
 	(progn
 	  (setq dsts (delete srcx dsts))
-	  (while msgs
-	    (setq msg (car msgs))
-	    (setq msgs (cdr msgs))
+	  (dolist (msg msgs)
 	    (if (file-regular-p (mew-expand-msg src msg))
 		(setq msgs- (cons msg msgs-))))
 	  (setq msgs- (nreverse msgs-))
@@ -141,29 +129,26 @@ DELS represents the messages to be deleted."
       (setq dst (mew-case-folder mew-inherit-exec-case (car dsts)))
       (setq dsts (cdr dsts))
       (setq num (string-to-number (mew-folder-new-message dst 'num-only cache)))
-      (while msgs
-	(setq srcfile (mew-expand-msg src (car msgs)))
-	(setq msgs (cdr msgs))
+      (dolist (msg msgs)
+	(setq srcfile (mew-expand-msg src msg))
 	(when (and (file-exists-p srcfile) (file-writable-p srcfile))
 	  (setq strnum (mew-i2s num cache))
 	  (setq msgs- (cons strnum msgs-))
 	  (setq dstfile (mew-expand-new-msg dst strnum))
 	  (setq num (1+ num))
 	  (rename-file srcfile dstfile 'override)
+	  (mew-set-file-times dstfile)
 	  (when suffix
 	    (setq infofile (concat srcfile suffix))
 	    (mew-delete-file infofile))))
       (mew-touch-folder dst)
       (setq msgs- (nreverse msgs-))
       (setq src dst)) ;; myselfp
-    (while dsts
-      (setq dst (mew-case-folder mew-inherit-exec-case (car dsts)))
-      (setq dsts (cdr dsts))
+    (dolist (ent dsts)
+      (setq dst (mew-case-folder mew-inherit-exec-case ent))
       (setq num (string-to-number (mew-folder-new-message dst 'num-only cache)))
-      (setq msgs msgs-)
-      (while msgs
-	(setq srcfile (mew-expand-msg src (car msgs)))
-	(setq msgs (cdr msgs))
+      (dolist (msg msgs-)
+	(setq srcfile (mew-expand-msg src msg))
 	(setq strnum (mew-i2s num cache))
 	(setq dstfile (mew-expand-new-msg dst strnum))
 	(setq num (1+ num))
@@ -179,14 +164,14 @@ DELS represents the messages to be deleted."
     (unless (file-directory-p trashdir) (make-directory trashdir))
     (setq num (string-to-number (mew-folder-new-message trash 'num-only cache)))
     ;; must be here after ensuring that +trash exists.
-    (while dels
-      (setq srcfile (mew-expand-msg src (car dels))) ;; xxx
-      (setq dels (cdr dels))
+    (dolist (del dels)
+      (setq srcfile (mew-expand-msg src del)) ;; xxx
       (setq dstfile (mew-expand-new-msg trash (mew-i2s num cache)))
       (setq num (1+ num))
-      (if (file-regular-p srcfile)
-	  ;; if not, the marked line will be deleted anyway.
-	  (rename-file srcfile dstfile 'override))
+      (when (file-regular-p srcfile)
+	;; if not, the marked line will be deleted anyway.
+	(rename-file srcfile dstfile 'override)
+	(mew-set-file-times dstfile))
       (when suffix
 	(setq infofile (concat srcfile suffix))
 	(mew-delete-file infofile)))
@@ -217,7 +202,7 @@ region. To cancel the '*' mark, use '\\[mew-summary-undo]' or
 
 (defun mew-summary-exec-one (&optional arg)
   "Process the current marked message.
-When called with '\\[universal-argument]', process 
+When called with '\\[universal-argument]', process
 all marked messages before the current message."
   (interactive "P")
   (mew-summary-goto-message)
@@ -292,7 +277,7 @@ destination is the same as that of the current message are processed."
      ((mew-folder-remotep folder)
       (message "Refiling and deleting...")
       (force-mode-line-update)
-      (mew-summary-exec-remote beg end))
+      (mew-summary-exec-remote beg end func))
      ((mew-virtual-p)
       (mew-summary-go-back-summary
        (mew-substitute-for-summary "\\[mew-summary-exec]")))
@@ -310,11 +295,10 @@ destination is the same as that of the current message are processed."
   (when (get-buffer buf)
     (save-excursion
       (set-buffer buf)
-      (mew-frwlet
-       mew-cs-dummy mew-cs-m17n
-       (write-region (point-min) (point-max)
-		     (expand-file-name mew-refile-log-file mew-conf-path)
-		     'append 'no-msg)))
+      (mew-frwlet mew-cs-dummy mew-cs-m17n
+	(write-region (point-min) (point-max)
+		      (expand-file-name mew-refile-log-file mew-conf-path)
+		      'append 'no-msg)))
     (mew-remove-buffer buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -322,7 +306,7 @@ destination is the same as that of the current message are processed."
 ;;; Processing marks for local
 ;;;
 
-(defsubst mew-mark-kill-line ()
+(defun mew-mark-kill-line ()
   (let (start)
     (beginning-of-line)
     (setq start (point))
@@ -339,10 +323,8 @@ destination is the same as that of the current message are processed."
 		 (m (copy-marker end))
 		 (cnt 0)
 		 (case-fold-search nil)
-		 msg msgs err-folders mark func-exec func-sanity killp regex)
-	    (while marks
-	      (setq mark (car marks))
-	      (setq marks (cdr marks))
+		 msg msgs err-folders func-exec func-sanity killp regex)
+	    (dolist (mark marks)
 	      (setq func-exec (mew-markdb-func-exec mark))
 	      (when func-exec
 		(setq killp (mew-markdb-killp mark))
@@ -399,7 +381,7 @@ destination is the same as that of the current message are processed."
 ;;; Processing marks for remote
 ;;;
 
-(defsubst mew-mark-invisible-line ()
+(defun mew-mark-invisible-line ()
   (let (start)
     (beginning-of-line)
     (setq start (point))
@@ -409,7 +391,10 @@ destination is the same as that of the current message are processed."
      ;; For Emacs only. To make invisible text act as we want
      (put-text-property start (point) 'front-sticky t))))
 
-(defun mew-summary-exec-remote-get-info (beg end modify)
+(mew-defstruct execinfo remove kill refile move rttl dttl)
+(mew-defstruct refileinfo uid size delete folders)
+
+(defun mew-summary-exec-remote-get-execinfo (beg end modify &optional func-decide)
   ;; for POP or IMAP
   ;; If modify, marked lines get invisible. They are removed
   ;; IMAP's sentinel.
@@ -418,9 +403,11 @@ destination is the same as that of the current message are processed."
   (let* ((case (mew-sinfo-get-case))
 	 (folder (mew-sinfo-get-folder))
 	 (m (copy-marker end))
+	 (case-fold-search nil)
+	 (rttl 0) (dttl 0)
 	 trash trash-list lomov
 	 siz msg uid ref flds del regex trss rmvs kils refs movs
-	 refinfo refile-back mark rttl dttl)
+	 refinfo refile-back)
     (when (mew-folder-imapp folder)
       ;; see also mew-mark-exec-delete.
       (setq trash (mew-imap-trash-folder case))
@@ -438,71 +425,109 @@ destination is the same as that of the current message are processed."
 	    (setq trash nil)))))
     (save-excursion
       (mew-decode-syntax-delete)
-      (setq regex (mew-mark-regex mew-mark-refile))
-      (goto-char beg)
-      (while (re-search-forward regex m t)
-	(re-search-forward mew-regex-sumsyn-long)
-	(setq msg (mew-sumsyn-message-number))
-	(setq uid (mew-sumsyn-message-uid))
-	(setq siz (mew-sumsyn-message-size))
-	(setq refinfo (mew-refile-get msg))
-	(setq refile-back (cons refinfo refile-back))
-	(setq flds (cdr refinfo))
-	;;
-	(setq del nil)
-	(if (member folder flds)
-	    (setq flds (delete folder flds)) ;; destructive
-	  (setq del t)
-	  (when modify
-	    (if lomov
-		(setq movs (cons msg movs))
-	      (setq kils (cons msg kils)))))
-	(when flds
-	  (setq ref (cons uid (cons siz (cons del flds))))
-	  (setq refs (cons ref refs)))
-	(if modify
-	    (if del
-		(mew-mark-invisible-line)
-	      (mew-summary-refile-remove-body)
-	      (mew-mark-remove))))
-      (setq refs (nreverse refs))
-      (setq rttl (length refs))
-      (setq movs (nreverse movs))
-      (if modify (mew-sinfo-set-refile-back refile-back))
-      (when lomov
-	(let ((mew-inherit-exec-case case))
-	  (mew-mark-sanity-refile movs)))
       ;;
-      (setq regex (mew-mark-list-regex (list mew-mark-delete mew-mark-unlink)))
-      (goto-char beg)
-      (setq dttl 0)
-      (while (re-search-forward regex m t)
-	(setq mark (match-string 0))
-	(re-search-forward mew-regex-sumsyn-long)
-	(setq msg (mew-sumsyn-message-number))
-	(setq uid (mew-sumsyn-message-uid))
-	(setq siz (mew-sumsyn-message-size))
-	(if (and trash (char-equal (string-to-char mark) mew-mark-delete))
-	    (setq trss (cons (list uid siz t trash) trss))
-	  (setq rmvs (cons uid rmvs)))
-	(when modify
-	  (setq kils (cons msg kils))
-	  (mew-mark-invisible-line))
-	(setq dttl (1+ dttl)))
+      ;; refile
+      ;;
+      (when (mew-markdb-by-mark mew-mark-refile)
+	(setq regex (mew-mark-regex mew-mark-refile))
+	(goto-char beg)
+	(while (re-search-forward regex m t)
+	  (re-search-forward mew-regex-sumsyn-long)
+	  (setq msg (mew-sumsyn-message-number))
+	  (setq uid (mew-sumsyn-message-uid))
+	  (setq siz (mew-sumsyn-message-size))
+	  (when (or (not func-decide) (funcall func-decide msg))	
+	    (setq refinfo (mew-refile-get msg))
+	    (setq refile-back (cons refinfo refile-back))
+	    (setq flds (cdr refinfo))
+	    ;;
+	    (setq del nil)
+	    (if (member folder flds)
+		(setq flds (delete folder flds)) ;; destructive
+	      (setq del t)
+	      (when modify
+		(if lomov
+		    (setq movs (cons msg movs))
+		  (setq kils (cons msg kils)))))
+	    (when flds
+	      (setq ref (mew-make-refileinfo :uid uid
+					     :size siz
+					     :delete del
+					     :folders flds))
+	      (setq refs (cons ref refs)))
+	    (if modify
+		(if del
+		    (mew-mark-invisible-line)
+		  (mew-summary-refile-remove-body)
+		  (mew-mark-remove)))))
+	(setq refs (nreverse refs))
+	(setq rttl (length refs))
+	(setq movs (nreverse movs))
+	(if modify (mew-sinfo-set-refile-back refile-back))
+	(when lomov
+	  (let ((mew-inherit-exec-case case))
+	    (mew-mark-sanity-refile movs))))
+      ;;
+      ;; delete
+      ;;
+      (when (mew-markdb-by-mark mew-mark-delete)
+	(setq regex (mew-mark-regex mew-mark-delete))
+	(goto-char beg)
+	(while (re-search-forward regex m t)
+	  (re-search-forward mew-regex-sumsyn-long)
+	  (setq msg (mew-sumsyn-message-number))
+	  (setq uid (mew-sumsyn-message-uid))
+	  (setq siz (mew-sumsyn-message-size))
+	  (if trash
+	      (setq trss (cons (mew-make-refileinfo :uid uid
+						    :size siz
+						    :delete t
+						    :folders (list trash))
+			       trss))
+	    (setq rmvs (cons uid rmvs)))
+	  (when modify
+	    (setq kils (cons msg kils))
+	    (mew-mark-invisible-line))
+	  (setq dttl (1+ dttl))))
+      ;;
+      ;; unlink
+      ;;
+      (when (mew-markdb-by-mark mew-mark-unlink)
+	(setq regex (mew-mark-regex mew-mark-unlink))
+	(goto-char beg)
+	(while (re-search-forward regex m t)
+	  (re-search-forward mew-regex-sumsyn-long)
+	  (setq msg (mew-sumsyn-message-number))
+	  (setq uid (mew-sumsyn-message-uid))
+	  (setq siz (mew-sumsyn-message-size))
+	  (setq rmvs (cons uid rmvs))
+	  (when modify
+	    (setq kils (cons msg kils))
+	    (mew-mark-invisible-line))
+	  (setq dttl (1+ dttl))))
+      ;;
       (setq rmvs (nreverse rmvs))
       (setq trss (nreverse trss))
-      (when trss
-	(setq refs (nconc trss refs)))
+      (if trss (setq refs (nconc trss refs)))
+      ;;
       (set-buffer-modified-p nil)
       (cond
        ((and (null rmvs) (null refs)) nil)
        (modify
 	(force-mode-line-update) ;; xxx
-	(list rmvs kils refs movs rttl dttl))
+	(mew-make-execinfo :remove rmvs
+			   :kill   kils
+			   :refile refs
+			   :move   movs
+			   :rttl   rttl
+			   :dttl   dttl))
        (t
-	(list rmvs nil refs nil 0 0))))))
+	(mew-make-execinfo :remove rmvs
+			   :refile refs
+			   :rttl   0
+			   :dttl   0))))))
 
-(defun mew-summary-exec-remote (beg end)
+(defun mew-summary-exec-remote (beg end &optional func-decide)
   (if (mew-folder-nntpp (mew-sinfo-get-folder))
       (message "This command cannot be used in this folder")
     (when (mew-summary-exclusive-p)
@@ -510,17 +535,17 @@ destination is the same as that of the current message are processed."
 	     (folder (mew-sinfo-get-folder))
 	     (bnm (mew-summary-folder-name 'ext))
 	     (lbuf (mew-refile-log-buffer bnm))
-	     (info (mew-summary-exec-remote-get-info beg end 'modify)))
-	(if (not info)
+	     (execinfo (mew-summary-exec-remote-get-execinfo beg end 'modify func-decide)))
+	(if (not execinfo)
 	    (message "No messages to be processed")
 	  (mew-summary-reset)
 	  (while (eq (get-text-property (point) 'invisible) t)
 	    (forward-line))
 	  (cond
 	   ((mew-folder-popp folder)
-	    (mew-pop-retrieve case 'exec bnm info))
+	    (mew-pop-retrieve case 'exec bnm execinfo))
 	   ((mew-folder-imapp folder)
-	    (mew-imap-retrieve case 'exec bnm info)))
+	    (mew-imap-retrieve case 'exec bnm execinfo)))
 	  (mew-exec-refile-log lbuf))))))
 
 (defun mew-mark-kill-invisible ()
@@ -537,12 +562,12 @@ destination is the same as that of the current message are processed."
 ;;;
 
 (defun mew-summary-exec-offline (&optional arg)
-  "\\<mew-summary-mode-map>Process marked messages *offline*. Messages to be refiled in 
+  "\\<mew-summary-mode-map>Process marked messages in an IMAP folder *offline*. Messages to be refiled in
 a remote folder are moved to the corresponding folder but they are
 marked invalid. Invalid message are marked with '#'.
-Invalid messages will be overridden when the remote folder is scanned online. 
-A job to delete or refile messages, which is created by this command, 
-is queued in a queue folder (%queue for IMAP). To flush jobs in 
+Invalid messages will be overridden when the remote folder is scanned online.
+A job to delete or refile messages, which is created by this command,
+is queued in a queue folder (%queue for IMAP). To flush jobs in
 the queue, type '\\[mew-summary-send-message]' in the queue online."
   (interactive "P")
   (cond
@@ -566,46 +591,47 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
       (force-mode-line-update)
       (mew-summary-exec-offline-region beg end)))))
 
+(mew-defstruct jobinfo mailbox execinfo mark-hist)
+
 (defun mew-summary-exec-offline-region (beg end)
   "Process the current marked message offline."
   (interactive)
-  (let* ((info (mew-summary-exec-remote-get-info beg end nil))
-	 (rmvs (nth 0 info))
-	 (refs (nth 2 info)))
-    (if (not info)
+  (let* ((execinfo (mew-summary-exec-remote-get-execinfo beg end nil))
+	 (rmvs (mew-execinfo-get-remove execinfo))
+	 (refs (mew-execinfo-get-refile execinfo))
+	 job)
+    (if (not execinfo)
 	(message "No messages to be processed")
       (let* ((fld (mew-sinfo-get-folder))
 	     (case (mew-sinfo-get-case))
 	     (imapq (mew-case-folder case (mew-imap-queue-folder case)))
 	     (imapq-dir (mew-expand-folder imapq))
-	     file file-info ref)
+	     file file-info)
 	(unless (file-directory-p imapq-dir) (mew-make-directory imapq-dir))
 	(setq file (mew-folder-new-message imapq))
 	(setq file-info (concat file mew-imapq-info-suffix))
-	(mew-lisp-save file-info (cons fld info) 'nobackup 'unlimit)
+	(setq job (mew-make-jobinfo :mailbox fld
+				    :execinfo execinfo
+				    :mark-hist (mew-sinfo-get-mark-hist)))
+	(mew-lisp-save file-info job 'nobackup 'unlimit)
 	(with-temp-buffer
 	  (mew-header-insert mew-subj: (format "IMAP jobs for %s" fld))
 	  (mew-header-insert mew-date: (mew-time-ctz-to-rfc (current-time)))
 	  (mew-header-insert mew-from: "Mew IMAP manager")
 	  (insert "\n")
 	  (insert "Messages to be refiled:\n")
-	  (while refs
-	    (setq ref (car refs))
-	    (setq refs (cdr refs))
-	    (if (nth 2 ref)
-		(insert (format "\t%s will be copied to %s\n"
-				(nth 0 ref)
-				(mew-join "," (nthcdr 3 ref))))
-	      (insert (format "\t%s will be moved to %s\n"
-			      (nth 0 ref)
-			      (mew-join "," (nthcdr 3 ref))))))
+	  (dolist (ref refs)
+	    (insert (format (if (mew-refileinfo-get-delete ref)
+				"\t%s will be copied to %s\n"
+			      "\t%s will be moved to %s\n")
+			    (mew-refileinfo-get-uid ref)
+			    (mew-join "," (mew-refileinfo-get-folders ref)))))
 	  (insert "\n")
 	  (when rmvs
 	    (insert "Messages to be deleted:\n")
 	    (insert "\t" (mew-join "," rmvs) "\n"))
-	  (mew-frwlet
-	   mew-cs-dummy nil
-	   (write-region (point-min) (point-max) file nil 'no-msg)))
+	  (mew-frwlet mew-cs-dummy nil
+	    (write-region (point-min) (point-max) file nil 'no-msg)))
 	(mew-touch-folder imapq)
 	;; refile log is not necessary since mew-summary-exec-local does it.
 	(let ((mew-inherit-exec-case case)
@@ -636,13 +662,13 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2006 Mew developing team.
+;; Copyright (C) 1997-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -651,7 +677,7 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
