@@ -1,4 +1,4 @@
-;;; mew-header.el --- Mail header stuff for Mew
+;; mew-header.el --- Mail header stuff for Mew
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
@@ -14,18 +14,18 @@
 ;;; Header functions
 ;;;
 
-(defsubst mew-header-goto-next ()
+(defun mew-header-goto-next ()
   (while (looking-at mew-lwsp) (forward-line)))
 
 ;; see also mew-header-end
-(defsubst mew-header-goto-end ()
+(defun mew-header-goto-end ()
   (goto-char (point-min))
   (if (re-search-forward mew-eoh nil t)
       (beginning-of-line)
     (goto-char (point-max))
     (if (eolp) (insert "\n"))))
 
-(defsubst mew-header-goto-body ()
+(defun mew-header-goto-body ()
   (mew-header-goto-end)
   (forward-line))
 
@@ -75,10 +75,10 @@
 	(if (and (listp value)
 		 (stringp (car value)))
 	    (setq value (cons 'format value)))
-	(setq value 
+	(setq value
 	      (condition-case e
 		  (eval value)
-		(error 
+		(error
 		 (format "*** ERROR: %s: %s ***" (car e) (car (cdr e))))))
 	(unless (or (stringp value)
 		    (null value))
@@ -155,7 +155,6 @@
     (let* ((case-fold-search t)
 	   (len (length order))
 	   (vec (make-vector len nil))
-	   (i 0)
 	   key beg n line other)
       (mew-header-goto-end)
       (save-restriction
@@ -174,9 +173,8 @@
 	    (if n
 		(aset vec n (concat (aref vec n) line))
 	      (setq other (concat other line)))))
-	(while (< i len)
-	  (and (aref vec i) (insert (aref vec i)))
-	  (setq i (1+ i)))
+	(dotimes (i len)
+	  (and (aref vec i) (insert (aref vec i))))
 	(and other (insert other))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -184,16 +182,16 @@
 ;;; Header insertion
 ;;;
 
-(defsubst mew-draft-header-insert (field value)
+(defun mew-draft-header-insert (field value)
   "No encoding. Just insert."
   (if (and value (stringp field)) (insert field " " value "\n")))
 
-(defsubst mew-draft-header-fill (field value)
+(defun mew-draft-header-fill (field value)
   (unless (mew-header-existp field) (mew-draft-header-insert field value)))
 
 (defun mew-header-insert (key value &optional no-fold)
   (if (and value (stringp key))
-      (let ((beg (point)) params med par parname parval)
+      (let ((beg (point)) params med parname parval)
 	(when (listp value)
 	  (setq params (cdr value))
 	  (setq value (car value)))
@@ -203,10 +201,8 @@
 	(if (string-match "^[\t -~]*$" value)
 	    (insert value)
 	  (mew-header-encode-text value nil (length key)))
-	(while params
-	  (setq par (car params))
+	(dolist (par params)
 	  (mew-set '(parname parval) par)
-	  (setq params (cdr params))
 	  (insert ";")
 	  (cond
 	   ((string-match "^[-a-zA-Z0-9]+$" parval)
@@ -246,17 +242,17 @@
 ;; 'quoted-string', 'comment', and 'domain-literal' is self-delimiting.
 ;; But 'domain-literal' (e.g. [1.2.3.4]) is meaningless.
 ;; So, the delimiters consists of SEP, SP, TAB, CRLF, "(", ")", and <">.
-;; 
+;;
 ;; 'qtext' remains. (It will be removed by mew-param-decode.)
 ;; ## how about 'quoted-pair'?
 ;;
 ;; 'comment' must be symmetric. That is, it must have the exact number of
-;; ")" against that of ")". The entire 'comment', even if nested, is 
+;; ")" against that of ")". The entire 'comment', even if nested, is
 ;; ignored.
 ;;
 ;; 'encoded-word' is meaningless.
 ;; (1) Remove 'comment' so 'encoded-word' in 'comment' is meaningless.
-;; (2) 'phrase' of 'mailbox' will be ignored because 'addr-spec' in 
+;; (2) 'phrase' of 'mailbox' will be ignored because 'addr-spec' in
 ;; 'route-addr' will be extracted. So, 'encoded-word' in 'phrase' is
 ;; meaningless.
 
@@ -270,51 +266,60 @@
        (mew-warn "Too many values. Truncate values over mew-header-max-depth")
        (throw 'max nil))))
 
-(defun mew-addrstr-parse-syntax-list (str sep addrp &optional depth)
-  (when str
-    (if (featurep 'mew-temacs)
-	(setq str (string-as-unibyte str))) ;; A bug of Emacs 20.7
-    (let* ((i 0) (len (length str))
-	   (par-cnt 0) (tmp-cnt 0) (sep-cnt 0)
-	   (tmp (make-string len ?x))
-	   c ret)
-      (catch 'max
-	(while (< i len)
-	  (setq c (aref str i))
-	  (cond
-	   ((char-equal c ?\")
+(defun mew-addrstr-parse-syntax-list (str sep addrp &optional depth allow-spc)
+  (if str (mew-addrstr-parse-syntax-list1 str sep addrp depth allow-spc)))
+
+(defun mew-addrstr-parse-syntax-list1 (str sep addrp depth allow-spc)
+  (let* ((i 0) (len (length str))
+	 (par-cnt 0) (tmp-cnt 0) (sep-cnt 0)
+	 ;; Emacs 23 has non-safe aset.
+	 (tmp (mew-set-string-multibyte (mew-make-string len)))
+	 c ret prevc
+	 (do-clear
+	  (lambda ()
+	    (setq tmp-cnt 0)
+	    (mew-addrstr-parse-syntax-list-check-depth depth)))
+	 (do-cons-ret
+	  (lambda ()
+	    (setq ret (cons (substring tmp 0 tmp-cnt) ret))))
+	 (do-copy
+	  (lambda ()
 	    (aset tmp tmp-cnt c)
-	    (setq tmp-cnt (1+ tmp-cnt))
+	    (setq tmp-cnt (1+ tmp-cnt))))
+	 (in-fold-quote
+	  (lambda ()
 	    (setq i (1+ i))
-	    (catch 'quote
+	    (catch 'fold-quote
 	      (while (< i len)
 		(setq c (aref str i))
 		(cond
+		 ;; ((or (char-equal c ?\t) (char-equal c mew-sp))
+		 ;; (setq i (1+ i)))
 		 ((char-equal c ?\")
-		  (aset tmp tmp-cnt c)
-		  (setq tmp-cnt (1+ tmp-cnt))
+		  (funcall do-copy)
+		  (throw 'quote nil))
+		 (t
+		  (funcall do-copy)
+		  (throw 'fold-quote nil)))))))
+	 (in-quote
+	  (lambda ()
+	    (funcall do-copy)
+	    (setq i (1+ i))
+	    (catch 'quote
+	      (while (< i len)
+		(setq prevc c)
+		(setq c (aref str i))
+		(cond
+		 ((and (char-equal c ?\") (not (char-equal prevc ?\\)))
+		  (funcall do-copy)
 		  (throw 'quote nil))
 		 ((char-equal c ?\n)
-		  (setq i (1+ i))
-		  (catch 'fold-quote
-		    (while (< i len)
-		      (setq c (aref str i))
-		      (cond
-		       ;; ((or (char-equal c ?\t) (char-equal c mew-sp))
-		       ;; (setq i (1+ i)))
-		       ((char-equal c ?\")
-			(aset tmp tmp-cnt c)
-			(setq tmp-cnt (1+ tmp-cnt))
-			(throw 'quote nil))
-		       (t
-			(aset tmp tmp-cnt c)
-			(setq tmp-cnt (1+ tmp-cnt))
-			(throw 'fold-quote nil))))))
+		  (funcall in-fold-quote))
 		 (t
-		  (aset tmp tmp-cnt c)
-		  (setq tmp-cnt (1+ tmp-cnt))))
-		(setq i (1+ i)))))
-	   ((char-equal c ?\()
+		  (funcall do-copy)))
+		(setq i (1+ i))))))
+	 (in-comment
+	  (lambda ()
 	    (setq par-cnt 1)
 	    (setq i (1+ i))
 	    (catch 'comment
@@ -326,106 +331,120 @@
 		 ((char-equal c ?\))
 		  (setq par-cnt (1- par-cnt))
 		  (if (eq par-cnt 0) (throw 'comment nil))))
-		(setq i (1+ i)))))
-	   ((char-equal c ?<)
+		(setq i (1+ i))))))
+	 (do-skip-until
+	  (lambda (target)
+	    (while (and (< i len) (not (char-equal (aref str i) target)))
+	      (setq i (1+ i)))))
+	 (in-addr
+	  (lambda ()
 	    (cond
 	     (addrp
 	      (let (rbeg rend)
 		(setq i (1+ i))
 		(setq rbeg i)
-		(while (and (< i len) (not (char-equal (aref str i) ?>)))
-		  (setq i (1+ i)))
+		(funcall do-skip-until ?>)
 		(when (> i rbeg)
-		  (setq rend i)	;; note: to be used for substring, so not 1-.
+		  (setq rend i)
+		  ;; note: to be used for substring, so not 1-.
 		  ;; should not be nested but easy to implement...
-		  (setq ret (cons (car (mew-addrstr-parse-syntax-list
-					(substring str rbeg rend) sep t))
-				  ret))))
-	      (while (and (< i len) (not (char-equal (aref str i) sep)))
-		(setq i (1+ i)))
-	      (setq tmp-cnt 0)
-	      (mew-addrstr-parse-syntax-list-check-depth depth))
+		  (setq ret (cons (car (mew-addrstr-parse-syntax-list (substring str rbeg rend) sep t)) ret))))
+	      (funcall do-skip-until sep)
+	      (funcall do-clear))
 	     (t
 	      ;; just ignore
-	      (while (and (< i len) (not (char-equal (aref str i) ?>)))
-		(setq i (1+ i))))))
-	   ((char-equal c ?\n))
-	   ((char-equal c ?\t))
-	   ((char-equal c mew-sp))
-	   ((char-equal c sep)
+	      (funcall do-skip-until ?>)))))
+	 (do-sep
+	  (lambda ()
 	    ;; broken quoted string cannot be rescued because
 	    ;; the separator cannot be distinguished
-	    (if (> tmp-cnt 0)
-		(setq ret (cons (substring tmp 0 tmp-cnt) ret)))
-	    (setq tmp-cnt 0)
-	    (mew-addrstr-parse-syntax-list-check-depth depth))
-	   (t 
-	    (aset tmp tmp-cnt c)
-	    (setq tmp-cnt (1+ tmp-cnt)))) ;; end of cond
-	  (setq i (1+ i)))		  ;; end of while
-	(when (> tmp-cnt 0)
-	  ;; broken quoted string can be rescued if it appears solely
-	  (save-match-data
-	    (if (string-match "<\\([^>]+\\)>" tmp)
-		(setq ret (cons (mew-match-string 1 tmp) ret))
-	      (setq ret (cons (substring tmp 0 tmp-cnt) ret)))))) ;; end of catch
-      (setq ret (delete nil ret))
-      (if (featurep 'mew-temacs)
-	  (mapcar 'string-as-multibyte (nreverse ret)) ;; A bug of Emacs 20.7
-	(nreverse ret)))))
+	    (if (> tmp-cnt 0) (funcall do-cons-ret))
+	    (funcall do-clear))))
+    ;; main
+    (catch 'max
+      (while (< i len)
+	(setq c (aref str i))
+	(cond
+	 ((char-equal c ?\")
+	  (funcall in-quote))
+	 ((char-equal c ?\()
+	  (funcall in-comment))
+	 ((char-equal c ?<)
+	  (funcall in-addr))
+	 ((or (char-equal c ?\n) (char-equal c ?\t)
+	      (and (not allow-spc) (char-equal c mew-sp)))
+	  ) ;; do nothing
+	 ((char-equal c sep)
+	  (funcall do-sep))
+	 (t
+	  (funcall do-copy)))
+	(setq i (1+ i)))
+      (when (> tmp-cnt 0)
+	;; broken quoted string can be rescued if it appears solely
+	(save-match-data
+	  (if (and addrp (string-match "<\\([^>]+\\)>" tmp))
+	      (setq ret (cons (mew-match-string 1 tmp) ret))
+	    (funcall do-cons-ret)))))
+    (setq ret (delete nil ret))
+    (when allow-spc
+      (setq ret (mapcar (lambda (str)
+			  (if (string-match "^ +\\(.*\\)" str)
+			      (mew-match-string 1 str)
+			    str))
+			ret)))
+    (nreverse ret)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; High-level functions to parse fields
 ;;;
 
-(defsubst mew-addrstr-parse-value (value)
+(defun mew-addrstr-parse-value (value)
   (car (mew-addrstr-parse-syntax-list value ?\; nil 1)))
 
-(defsubst mew-addrstr-parse-value-list (value)
+(defun mew-addrstr-parse-value-list (value)
   (mew-addrstr-parse-syntax-list value ?\; nil))
 
-(defsubst mew-addrstr-parse-value2 (value)
+(defun mew-addrstr-parse-value2 (value)
   (car (mew-addrstr-parse-syntax-list value ?, nil 1)))
 
-(defsubst mew-addrstr-parse-value-list2 (value)
-  (mew-addrstr-parse-syntax-list value ?, nil))
+(defun mew-addrstr-parse-value-list2 (value)
+  (mew-addrstr-parse-syntax-list value ?, nil nil t))
 
 ;;
 
-(defsubst mew-addrstr-parse-address (address)
+(defun mew-addrstr-parse-address (address)
   (car (mew-addrstr-parse-syntax-list address ?, t 1)))
 
-(defsubst mew-addrstr-parse-address-list (address)
+(defun mew-addrstr-parse-address-list (address)
   (mew-addrstr-parse-syntax-list address ?, t))
 
-(defsubst mew-header-parse-address (field)
+(defun mew-header-parse-address (field)
   (car
    (mew-addrstr-parse-syntax-list
     (mapconcat 'mew-header-get-value (list field) ",")
     ?, t 1)))
 
-(defsubst mew-header-parse-address-list (field-list)
+(defun mew-header-parse-address-list (field-list)
   (mew-addrstr-parse-syntax-list
    (mapconcat 'mew-header-get-value field-list ",")
    ?, t))
 
 (defun mew-header-parse-address-list2 (field-list)
-  "Collect addresses from FIELD-LIST. 
+  "Collect addresses from FIELD-LIST.
 Remove anonymous addresses."
   (let ((vals (mew-addrstr-parse-syntax-list
 	       (mapconcat 'mew-header-get-value field-list ",")
 	       ?, t))
 	ret)
-    (while vals
-      (unless (string-match mew-anonymous-recipients (car vals))
-	(setq ret (cons (car vals) ret)))
-      (setq vals (cdr vals)))
+    (dolist (val vals)
+      (unless (string-match mew-anonymous-recipients val)
+	(setq ret (cons val ret))))
     (nreverse ret)))
 
 ;;
 
-(defsubst mew-addrstr-extract-user-list (addr-list)
+(defun mew-addrstr-extract-user-list (addr-list)
   (mapcar 'mew-addrstr-extract-user addr-list))
 
 (defun mew-addrstr-extract-user (addr)
@@ -483,7 +502,7 @@ Remove anonymous addresses."
   (let (skip)
     (if (integerp mew-references-max-count)
 	(setq skip (- (length id-list) mew-references-max-count)))
-    (if (and (numberp skip) (> skip 0))
+    (if (and (integerp skip) (> skip 0))
 	(setq id-list (nthcdr skip id-list)))
     (mew-join "\n\t" id-list)))
 
@@ -491,13 +510,13 @@ Remove anonymous addresses."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2006 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -506,7 +525,7 @@ Remove anonymous addresses."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
