@@ -14,7 +14,7 @@
 ;;; IMAP2 info
 ;;;
 
-(defvar mew-imap2-info-list 
+(defvar mew-imap2-info-list
   '("case" "directive" "mailbox"
     ;;
     "src" "messages" "src-list" "src-list-orig"
@@ -48,10 +48,10 @@
     ("post-append"   ("OK" . "done") ("\\[TRYCREATE\\]" . "create"))
     ("logout"        ("OK" . "noop"))))
 
-(defsubst mew-imap2-fsm-by-status (status)
+(defun mew-imap2-fsm-by-status (status)
   (assoc status mew-imap2-fsm))
 
-(defsubst mew-imap2-fsm-next (status code)
+(defun mew-imap2-fsm-next (status code)
   (cdr (mew-assoc-match2 code (nthcdr 1 (mew-imap2-fsm-by-status status)) 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -121,7 +121,7 @@
 	src dst mailbox msg src-file)
     (if msgs
 	(setq src (mew-imap2-get-src pnm))
-      (if (eq directive 'copy)
+      (if (or (eq directive 'copy) (eq directive 'move))
 	  (progn
 	    (setq msgs src-list)
 	    (mew-imap2-set-src-list pnm nil)
@@ -132,7 +132,7 @@
 	    (setq mailbox (mew-imap-utf-7-encode-string mailbox))
 	    (mew-imap2-set-mailbox pnm mailbox))
 	(catch 'break
-	  (while src-list
+	  (while src-list ;; cannot use dolist
 	    (setq src (car src-list))
 	    (setq src-list (cdr src-list))
 	    (mew-imap2-set-src pnm src)
@@ -157,13 +157,12 @@
 	  (setq msg (car msgs))
 	  (setq msgs (cdr msgs))
 	  (cond
-	   ((eq directive 'copy)
+	   ((or (eq directive 'copy) (eq directive 'move))
 	    (setq src-file msg))
 	   (t
 	    (setq src-file (mew-expand-msg (mew-case-folder case src) msg))))
-	  (mew-frwlet
-	   mew-cs-text-for-read mew-cs-dummy
-	   (set-buffer (mew-find-file-noselect src-file)))
+	  (mew-frwlet mew-cs-text-for-read mew-cs-dummy
+	    (set-buffer (mew-find-file-noselect src-file)))
 	  (mew-set-buffer-multibyte nil)
 	  (mew-info-clean-up pnm mew-imap2-info-list-clean-length)
 	  (mew-imap2-set-messages pnm msgs)
@@ -244,7 +243,7 @@
   '(("CRAM-MD5" mew-imap2-command-auth-cram-md5)
     ("LOGIN"    mew-imap2-command-auth-login)))
 
-(defsubst mew-imap2-auth-get-func (auth)
+(defun mew-imap2-auth-get-func (auth)
   (nth 1 (mew-assoc-case-equal auth mew-imap2-auth-alist 0)))
 
 (defun mew-imap2-command-auth-cram-md5 (pro pnm)
@@ -286,19 +285,19 @@
 
 (defconst mew-imap2-info-prefix "mew-imap2-info-")
 
-(defsubst mew-imap2-info-name (case)
+(defun mew-imap2-info-name (case)
   (let ((server (mew-imap-server case))
-	(port (mew-imap-port case))
+	(port (mew-*-to-string (mew-imap-port case)))
 	(sshsrv (mew-imap-ssh-server case))
 	(name mew-imap2-info-prefix))
     (setq name (concat name server))
-    (unless (string= port mew-imap-port)
+    (unless (mew-port-equal port mew-imap-port)
       (setq name (concat name ":" port)))
     (if sshsrv
 	(concat name "%" sshsrv)
       name)))
 
-(defsubst mew-imap2-buffer-name (pnm)
+(defun mew-imap2-buffer-name (pnm)
   (concat mew-buffer-prefix pnm))
 
 (defun mew-imap2-process-send-string (pro pnm &rest args)
@@ -362,14 +361,14 @@
 ;;;
 
 (defun mew-imap2-open (pnm server port)
-  (let ((sprt (mew-port-sanity-check port))
+  (let ((sprt (mew-*-to-port port))
 	pro tm)
     (condition-case emsg
 	(progn
 	  (setq tm (run-at-time mew-imap-timeout-time nil 'mew-imap2-timeout))
 	  (message "Connecting to the IMAP server...")
 	  (setq pro (open-network-stream pnm nil server sprt))
-	  (process-kill-without-query pro)
+	  (mew-process-silent-exit pro)
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
 	  (message "Connecting to the IMAP server...done"))
       (quit
@@ -392,11 +391,13 @@
 (defun mew-imap2-copy-message (case directive src-list &optional dst)
   (let* ((server (mew-imap-server case))
          (user (mew-imap-user case))
-	 (port (mew-imap-port case))
+	 (port (mew-*-to-string (mew-imap-port case)))
 	 (pnm (mew-imap2-info-name case))
 	 (sshsrv (mew-imap-ssh-server case))
 	 (sslp (mew-imap-ssl case))
 	 (sslport (mew-imap-ssl-port case))
+	 (proxysrv (mew-imap-proxy-server case))
+	 (proxyport (mew-imap-proxy-port case))
 	 process sshname sshpro sslname sslpro lport tls)
     (cond
      (sshsrv
@@ -407,13 +408,15 @@
 	(when lport
 	  (setq process (mew-imap2-open pnm "localhost" lport)))))
      (sslp
-      (if (string= port sslport) (setq tls mew-tls-imap))
+      (if (mew-port-equal port sslport) (setq tls mew-tls-imap))
       (setq sslpro (mew-open-ssl-stream case server sslport tls))
       (when sslpro
 	(setq sslname (process-name sslpro))
 	(setq lport (mew-ssl-pnm-to-lport sslname))
 	(when lport
 	  (setq process (mew-imap2-open pnm mew-ssl-localhost lport)))))
+     (proxysrv
+      (setq process (mew-imap2-open pnm proxysrv proxyport)))
      (t
       (setq process (mew-imap2-open pnm server port))))
     (if (null process)
@@ -455,12 +458,10 @@ time (even several days) if there are many local messages."
   (let* ((prefix (mew-input-local-folder mew-folder-local))
 	 (case (mew-input-case mew-case-default "IMAP"))
 	 (alist (mew-local-folder-alist))
-	 regex ent src-list fld)
+	 regex src-list fld)
     (unless prefix (setq prefix mew-folder-local))
     (setq regex (concat "^" (regexp-quote prefix)))
-    (while alist
-      (setq ent (car alist))
-      (setq alist (cdr alist))
+    (dolist (ent alist)
       (setq fld (car ent))
       (if (string-match regex fld)
 	  (setq src-list (cons fld src-list))))
@@ -536,39 +537,40 @@ with '*' in the region are handled."
 	 (str (concat (mew-imap2-get-string pnm) string))
 	 (buf (process-buffer process))
 	 aux stay next func code resp)
-    (mew-imap2-debug (upcase status) string)
-    (if (and buf (get-buffer buf)) (set-buffer buf))
-    (while (string-match "^\\*[^\n]*\n" str)
-      (setq aux (substring str 0 (match-end 0)))
-      (setq str (substring str (match-end 0)))
+    (save-excursion
+      (mew-imap2-debug (upcase status) string)
+      (if (and buf (get-buffer buf)) (set-buffer buf))
+      (while (string-match "^\\*[^\n]*\n" str)
+	(setq aux (substring str 0 (match-end 0)))
+	(setq str (substring str (match-end 0)))
+	(cond
+	 ((string= status "greeting")
+	  (setq next (mew-imap2-fsm-next "greeting" "OK")))
+	 ((string= status "capability")
+	  (mew-imap2-set-aux pnm (concat (mew-imap2-get-aux pnm) aux)))))
+      (mew-imap2-set-string pnm str)
       (cond
-       ((string= status "greeting")
-	(setq next (mew-imap2-fsm-next "greeting" "OK")))
-       ((string= status "capability")
-	(mew-imap2-set-aux pnm (concat (mew-imap2-get-aux pnm) aux)))))
-    (mew-imap2-set-string pnm str)
-    (cond
-     (next
-      nil)
-     ((string-match "^\\+" str)
-      (mew-imap2-set-aux pnm str)
-      (setq next (mew-imap2-fsm-next status "OK")))
-     ((string-match eos str)
-      (mew-imap2-set-tag pnm nil)
-      (setq code (mew-match-string 1 str))
-      (setq resp (mew-match-string 2 str))
-      (setq next (mew-imap2-fsm-next status code))
-      (unless next (setq next (mew-imap2-fsm-next status resp))))
-     (t
-      (setq stay t)))
-    (unless stay
-      (unless next (setq next "logout"))
-      (mew-imap2-set-status pnm next)
-      (setq func (intern-soft (concat "mew-imap2-command-" next)))
-      (if (fboundp func)
-	  (funcall func process pnm)
-	(error "No function called %s" (symbol-name func)))
-      (mew-imap2-set-string pnm nil))))
+       (next
+	nil)
+       ((string-match "^\\+" str)
+	(mew-imap2-set-aux pnm str)
+	(setq next (mew-imap2-fsm-next status "OK")))
+       ((string-match eos str)
+	(mew-imap2-set-tag pnm nil)
+	(setq code (mew-match-string 1 str))
+	(setq resp (mew-match-string 2 str))
+	(setq next (mew-imap2-fsm-next status code))
+	(unless next (setq next (mew-imap2-fsm-next status resp))))
+       (t
+	(setq stay t)))
+      (unless stay
+	(unless next (setq next "logout"))
+	(mew-imap2-set-status pnm next)
+	(setq func (intern-soft (concat "mew-imap2-command-" next)))
+	(if (fboundp func)
+	    (funcall func process pnm)
+	  (error "No function called %s" (symbol-name func)))
+	(mew-imap2-set-string pnm nil)))))
 
 (defun mew-imap2-sentinel (process event)
   (let* ((pnm (process-name process))
@@ -578,45 +580,45 @@ with '*' in the region are handled."
 	 (error (mew-imap2-get-error pnm))
 	 (sshpro (mew-imap2-get-ssh-process pnm))
 	 (sslpro (mew-imap2-get-ssl-process pnm))
-	 (srcflds (mew-imap2-get-src-list-orig pnm))
-	 srcfld)
-    (mew-imap2-debug "IMAP SENTINEL" event)
-    (cond
-     (error
-      (message "%s" error))
-     (done
-      (when (eq directive 'fcc)
-	(while srcflds
-	  (setq srcfld (car srcflds))
-	  (setq srcflds (cdr srcflds))
-	  (mew-imap2-copy-fcc-cache case srcfld))
-	(mew-delete-directory-recursively 
-	 (mew-folder-imap-fcc-dir case)))
-      (message "Copying in background...done"))
-     (t 
-      (message "IMAP connection is lost")))
-    (mew-info-clean-up pnm)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
-	(delete-process sslpro))
-    (run-hooks 'mew-imap2-sentinel-hook)))
+	 (srcflds (mew-imap2-get-src-list-orig pnm)))
+    (save-excursion
+      (mew-imap2-debug "IMAP SENTINEL" event)
+      (cond
+       (error
+	(message "%s" error))
+       (done
+	(cond
+	 ((eq directive 'fcc)
+	  (dolist (srcfld srcflds)
+	    (mew-imap2-copy-fcc-cache case srcfld))
+	  (mew-delete-directory-recursively
+	   (mew-folder-imap-fcc-dir case)))
+	 ((eq directive 'move)
+	  (let ((file (car (mew-imap2-get-src-list-orig pnm))))
+	    (if (file-exists-p file) (delete-file file)))))
+	(message "Copying in background...done"))
+       (t
+	(message "IMAP connection is lost")))
+      (mew-info-clean-up pnm)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (run-hooks 'mew-imap2-sentinel-hook))))
 
 (defun mew-imap2-copy-fcc-cache (case srcfld)
   (let* ((case:srcfld (mew-case-folder case srcfld))
 	 (srcs (mew-folder-messages case:srcfld))
 	 (prefix (file-name-as-directory mew-folder-imap-fcc-prefix))
 	 (regex (concat "^" (regexp-quote prefix)))
-	 src dst dstfld case:dstfld i)
+	 dst dstfld case:dstfld i)
     (string-match regex srcfld)
     (setq dstfld (concat mew-folder-imap (substring srcfld (match-end 0))))
     (setq case:dstfld (mew-case-folder case dstfld))
     (mew-local-folder-check case:dstfld)
     (setq i (string-to-number (mew-folder-new-message case:dstfld 'num 'cache)))
     ;; to make code simple, we just copy the file.
-    (while srcs
-      (setq src (car srcs))
-      (setq srcs (cdr srcs))
+    (dolist (src srcs)
       (setq dst (mew-i2s i 'cache))
       (copy-file (mew-expand-msg case:srcfld src)
 		 (mew-expand-new-msg case:dstfld dst))
@@ -627,13 +629,13 @@ with '*' in the region are handled."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2003-2006 Mew developing team.
+;; Copyright (C) 2003-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -642,7 +644,7 @@ with '*' in the region are handled."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
