@@ -21,23 +21,13 @@
 
 ;; iso-2022-7bit-ss2 is iso-2022-jp-2
 
-(cond
- (mew-xemacs-p
-  (copy-coding-system 'no-conversion      'mew-cs-text)
-  (copy-coding-system 'no-conversion-unix 'mew-cs-text-lf)
-  (copy-coding-system 'no-conversion-dos  'mew-cs-text-crlf)
-  (copy-coding-system 'no-conversion-mac  'mew-cs-text-cr)
-  (copy-coding-system 'no-conversion-dos  'mew-cs-text-net)
-  (defun mew-cs-raw-p (cs)
-    (memq cs '(no-conversion no-conversion-unix no-conversion-dos no-conversion-mac))))
- (t
-  (define-coding-system-alias 'mew-cs-text      'raw-text)
-  (define-coding-system-alias 'mew-cs-text-lf   'raw-text-unix)
-  (define-coding-system-alias 'mew-cs-text-crlf 'raw-text-dos)
-  (define-coding-system-alias 'mew-cs-text-cr   'raw-text-mac)
-  (define-coding-system-alias 'mew-cs-text-net  'raw-text-dos)
-  (defun mew-cs-raw-p (cs)
-    (memq cs '(raw-text raw-text-unix raw-text-dos raw-text-mac)))))
+(define-coding-system-alias 'mew-cs-text      'raw-text)
+(define-coding-system-alias 'mew-cs-text-lf   'raw-text-unix)
+(define-coding-system-alias 'mew-cs-text-crlf 'raw-text-dos)
+(define-coding-system-alias 'mew-cs-text-cr   'raw-text-mac)
+(define-coding-system-alias 'mew-cs-text-net  'raw-text-dos)
+(defun mew-cs-raw-p (cs)
+  (memq cs '(raw-text raw-text-unix raw-text-dos raw-text-mac)))
 
 (defvar mew-cs-dummy          'binary)
 (defvar mew-cs-binary         'binary)
@@ -45,7 +35,7 @@
 (defvar mew-cs-text-for-write 'mew-cs-text-lf)
 (defvar mew-cs-text-for-net   'mew-cs-text-net)
 (defvar mew-cs-autoconv       'undecided)
-;; ctext for consistency --unibyte, -unix for XEmacs's ^M
+;; xxx should be utf-8-unix someday.
 (defvar mew-cs-m17n (if (mew-coding-system-p 'ctext-unix) 'ctext-unix 'ctext))
 (defvar mew-cs-utf-16be
   (cond
@@ -87,107 +77,116 @@
 (defvar mew-charset-utf-8-encoding "base64")
 (defvar mew-charset-utf-8-header-encoding "B")
 
+(mew-defstruct ecsdb lcs cs cte b-or-q delsp)
+
+(defun mew-ecsdb-hcs (ecsdb)
+  (mew-ecsdb-get-cs ecsdb))
+
 (defvar mew-cs-database-for-encoding
-  `(((ascii)                    nil         "7bit"             "7bit")
+  `(((ascii)                    nil         "7bit"             "7bit" nil)
     ;; West European (Latin 1)
-    ((ascii latin-iso8859-1)    iso-8859-1  "quoted-printable" "Q")
+    ((ascii latin-iso8859-1)    iso-8859-1  "quoted-printable" "Q" nil)
     ;; East European (Latin 2)
-    ((ascii latin-iso8859-2)    iso-8859-2  "quoted-printable" "Q")
+    ((ascii latin-iso8859-2)    iso-8859-2  "quoted-printable" "Q" nil)
     ;; South European (Latin 3)
-    ((ascii latin-iso8859-3)    iso-8859-3  "quoted-printable" "Q")
+    ((ascii latin-iso8859-3)    iso-8859-3  "quoted-printable" "Q" nil)
     ;; North European (Latin 4)
-    ((ascii latin-iso8859-4)    iso-8859-4  "quoted-printable" "Q")
+    ((ascii latin-iso8859-4)    iso-8859-4  "quoted-printable" "Q" nil)
     ;; Cyrillic
-    ((ascii cyrillic-iso8859-5) koi8-r      "quoted-printable" "Q")
+    ((ascii cyrillic-iso8859-5) koi8-r      "quoted-printable" "Q" nil)
     ;; Arabic
-    ((ascii arabic-iso8859-6)   iso-8859-6  "base64"           "B")
+    ((ascii arabic-iso8859-6)   iso-8859-6  "base64"           "B" t)
     ;; Greek
-    ((ascii greek-iso8859-7)    iso-8859-7  "base64"           "B")
+    ((ascii greek-iso8859-7)    iso-8859-7  "base64"           "B" t)
     ;; Hebrew
-    ((ascii hebrew-iso8859-8)   iso-8859-8  "base64"           "B")
+    ((ascii hebrew-iso8859-8)   iso-8859-8  "base64"           "B" t)
     ;; Turkish (Latin 5)
-    ((ascii latin-iso8859-9)    iso-8859-9  "quoted-printable" "Q")
+    ((ascii latin-iso8859-9)    iso-8859-9  "quoted-printable" "Q" nil)
     ;; Celtic (Latin 8)
-    ((ascii latin-iso8859-14)   iso-8859-14 "quoted-printable" "Q") ;; xxx
+    ((ascii latin-iso8859-14)   iso-8859-14 "quoted-printable" "Q" nil) ;; xxx
     ;; New West European (Latin 9 or Latin 0)
-    ((ascii latin-iso8859-15)   iso-8859-15 "quoted-printable" "Q")
-    ((ascii thai-tis620 composition) ;; composition for Emacs 20
-                                tis-620     "base64"           "B")
-    ((ascii thai-xtis) ;; thai-xtis for XEmacs
-                                tis-620     "base64"           "B")
+    ((ascii latin-iso8859-15)   iso-8859-15 "quoted-printable" "Q" nil)
+    ((ascii thai-tis620)        tis-620     "base64"           "B" t)
     ((ascii latin-jisx0201 japanese-jisx0208 japanese-jisx0208-1978)
-                               iso-2022-jp "7bit"             "B")
-    ((ascii korean-ksc5601)     euc-kr     "8bit"             "B")
-    ((ascii chinese-gb2312)     cn-gb-2312 "base64"           "B")
+                               iso-2022-jp "7bit"             "B" t)
+    ((ascii korean-ksc5601)     euc-kr     "8bit"             "B" t)
+    ((ascii chinese-gb2312)     cn-gb-2312 "base64"           "B" t)
     ((ascii chinese-big5-1 chinese-big5-2)
-                              chinese-big5 "base64"           "B")
+                              chinese-big5 "base64"           "B" t)
     ((ascii japanese-jisx0208 japanese-jisx0213-1 japanese-jisx0213-2)
-                             iso-2022-jp-3 "7bit"             "B")
-    (nil utf-7 "7bit" "Q") ;; xxx
-    (nil utf-8 ,mew-charset-utf-8-encoding ,mew-charset-utf-8-header-encoding)
-    (nil iso-2022-jp-2 "7bit" "B")))
+                             iso-2022-jp-3 "7bit"             "B" t)
+    (nil utf-7 "7bit" "Q" t) ;; xxx
+    (nil utf-8 ,mew-charset-utf-8-encoding ,mew-charset-utf-8-header-encoding t)
+    (nil iso-2022-jp-2 "7bit" "B" t)))
 
 (defvar mew-cs-database-for-arg
   '((iso-2022-jp . euc-jp)
     (iso-2022-kr . euc-kr)))
 
+(defun mew-ecsdb-cs-for-arg (ecsdb)
+  (let* ((cs (mew-ecsdb-get-cs ecsdb))
+	 (acs (cdr (assoc cs mew-cs-database-for-arg))))
+    (or acs cs)))
+
 (if (and (not (mew-coding-system-p 'chinese-big5))
 	 (mew-coding-system-p 'big5))
     (define-coding-system-alias 'chinese-big5 'big5))
 
+(mew-defstruct dcsdb charset cs)
+
 (defvar mew-cs-database-for-decoding
-  `(("us-ascii"          . nil)
-    ("iso-8859-1"        . iso-8859-1)
-    ("iso-8859-2"        . iso-8859-2)
-    ("iso-8859-3"        . iso-8859-3)
-    ("iso-8859-4"        . iso-8859-4)
-    ("iso-8859-5"        . iso-8859-5)
-    ("iso-8859-6"        . iso-8859-6)
-    ("iso-8859-7"        . iso-8859-7)
-    ("iso-8859-8"        . iso-8859-8)
-    ("iso-8859-8-i"      . iso-8859-8) ;; temporary solution
-    ("iso-8859-9"        . iso-8859-9)
-    ("iso-8859-15"       . iso-8859-15)
-    ("iso-2022-cn"       . iso-2022-cn)
-    ("iso-2022-cn-ext"   . iso-2022-cn-ext)
-    ("gb2312"            . cn-gb-2312) ;; should be before cn-gb
-    ("cn-gb"             . cn-gb-2312)
-    ("hz-gb-2312"        . hz-gb-2312)
-    ("big5"              . chinese-big5)
-    ("cn-big5"           . chinese-big5)
-    ("iso-2022-kr"       . iso-2022-kr)
-    ("euc-kr"            . euc-kr)
-    ("ks_c_5601-1987"    . euc-kr)
-    ("iso-2022-jp"       . iso-2022-jp)
-    ("iso-2022-jp-2"     . iso-2022-jp-2)
-    ("iso-2022-jp-3"     . iso-2022-jp-3)
-    ("euc-jp"            . euc-japan)
-    ("shift_jis"         . shift_jis)
-    ("tis-620"           . tis-620)
-    ("koi8-r"            . koi8-r)
-    ("iso-2022-int-1"    . iso-2022-int-1)
-    ("utf-7"             . utf-7)
-    ("unicode-1-1-utf-7" . utf-7) ;; the old UTF-7 name, RFC 1642
-    ("utf-8"             . utf-8)
+  `(("us-ascii"          nil)
+    ("iso-8859-1"        iso-8859-1)
+    ("iso-8859-2"        iso-8859-2)
+    ("iso-8859-3"        iso-8859-3)
+    ("iso-8859-4"        iso-8859-4)
+    ("iso-8859-5"        iso-8859-5)
+    ("iso-8859-6"        iso-8859-6)
+    ("iso-8859-7"        iso-8859-7)
+    ("iso-8859-8"        iso-8859-8)
+    ("iso-8859-8-i"      iso-8859-8) ;; temporary solution
+    ("iso-8859-9"        iso-8859-9)
+    ("iso-8859-15"       iso-8859-15)
+    ("iso-2022-cn"       iso-2022-cn)
+    ("iso-2022-cn-ext"   iso-2022-cn-ext)
+    ("gb2312"            cn-gb-2312) ;; should be before cn-gb
+    ("cn-gb"             cn-gb-2312)
+    ("hz-gb-2312"        hz-gb-2312)
+    ("big5"              chinese-big5)
+    ("cn-big5"           chinese-big5)
+    ("iso-2022-kr"       iso-2022-kr)
+    ("euc-kr"            euc-kr)
+    ("ks_c_5601-1987"    euc-kr)
+    ("iso-2022-jp"       iso-2022-jp)
+    ("iso-2022-jp-2"     iso-2022-jp-2)
+    ("iso-2022-jp-3"     iso-2022-jp-3)
+    ("euc-jp"            euc-japan)
+    ("shift_jis"         shift_jis)
+    ("tis-620"           tis-620)
+    ("koi8-r"            koi8-r)
+    ("iso-2022-int-1"    iso-2022-int-1)
+    ("utf-7"             utf-7)
+    ("unicode-1-1-utf-7" utf-7) ;; the old UTF-7 name, RFC 1642
+    ("utf-8"             utf-8)
     ,@(if (mew-coding-system-p 'windows-1251)
-	  '(("windows-1250" . windows-1250)
-	    ("windows-1251" . windows-1251)
-	    ("windows-1252" . windows-1252)
-	    ("windows-1253" . windows-1253)
-	    ("windows-1254" . windows-1254)
-	    ("windows-1255" . windows-1255)
-	    ("windows-1256" . windows-1256)
-	    ("windows-1257" . windows-1257)
-	    ("windows-1258" . windows-1258))
-	'(("windows-1250" . cp1250)
-	  ("windows-1251" . cp1251)
-	  ("windows-1252" . cp1252)
-	  ("windows-1253" . cp1253)
-	  ("windows-1254" . cp1254)
-	  ("windows-1255" . cp1255)
-	  ("windows-1256" . cp1256)
-	  ("windows-1257" . cp1257)
-	  ("windows-1258" . cp1258)))))
+	  '(("windows-1250" windows-1250)
+	    ("windows-1251" windows-1251)
+	    ("windows-1252" windows-1252)
+	    ("windows-1253" windows-1253)
+	    ("windows-1254" windows-1254)
+	    ("windows-1255" windows-1255)
+	    ("windows-1256" windows-1256)
+	    ("windows-1257" windows-1257)
+	    ("windows-1258" windows-1258))
+	'(("windows-1250" cp1250)
+	  ("windows-1251" cp1251)
+	  ("windows-1252" cp1252)
+	  ("windows-1253" cp1253)
+	  ("windows-1254" cp1254)
+	  ("windows-1255" cp1255)
+	  ("windows-1256" cp1256)
+	  ("windows-1257" cp1257)
+	  ("windows-1258" cp1258)))))
 
 ;;
 ;; CS
@@ -204,27 +203,8 @@
   (if cs (encode-coding-region beg end cs)))
 
 ;; to internal
-;;(defun mew-cs-decode-string (str cs)
-;;  (if cs (decode-coding-string str cs) str))
-
-;; This code should be obsoleted when Mule 4.1 goes away.
 (defun mew-cs-decode-string (str cs)
-  (if (null cs)
-      str
-    (let ((post-conv (coding-system-post-read-conversion cs)))
-      (if (null post-conv)
-	  (decode-coding-string str cs)
-	;; Mule 4.1 (Emacs 20.7 + Mule-UCS-0.84) has a bug.
-	;; It applies post-conv just to a buffer, not to a string.
-	;; So, we should use a buffer.
-	(with-temp-buffer
-	  ;; The old base64-decode-string() returns a multibyte string.
-	  ;; The new base64-decode-string() returns a unibyte string.
-	  (mew-set-buffer-multibyte (mew-multibyte-string-p str))
-	  (insert str)
-	  (or (mew-multibyte-string-p str) (mew-set-buffer-multibyte t))
-	  (decode-coding-region (point-min) (point-max) cs)
-	  (buffer-substring (point-min) (point-max))))))) ;; xxx
+  (if cs (decode-coding-string str cs) str))
 
 ;; to external
 (defun mew-cs-encode-string (str cs)
@@ -234,10 +214,10 @@
 ;; Process environment
 ;;
 
-(defsubst mew-set-process-cs (process read write)
+(defun mew-set-process-cs (process read write)
   (set-process-coding-system process read write))
 
-(defsubst mew-set-buffer-cs (write)
+(defun mew-set-buffer-cs (write)
   (setq buffer-file-coding-system write))
 
 (defmacro mew-plet (&rest body)
@@ -246,9 +226,11 @@
      ,@body))
 
 (defmacro mew-piolet (read write &rest body)
+  ;; (declare (indent 2))
   `(let ((coding-system-for-read  ,read)
 	 (coding-system-for-write ,write))
      ,@body))
+(put 'mew-piolet 'lisp-indent-function 2)
 
 (defmacro mew-flet (&rest body)
   `(let ((coding-system-for-read  'binary)
@@ -259,12 +241,14 @@
      ,@body))
 
 (defmacro mew-frwlet (read write &rest body)
+  ;; (declare (indent 2))
   `(let ((coding-system-for-read  ,read)
 	 (coding-system-for-write ,write)
 	 (format-alist nil)
 	 (auto-image-file-mode nil)
 	 (jka-compr-inhibit t))
      ,@body))
+(put 'mew-frwlet 'lisp-indent-function 2)
 
 (defmacro mew-alet (&rest body)
   `(let ((default-file-name-coding-system nil)
@@ -277,7 +261,7 @@
 
 (defun mew-substring (str width &optional cnt nopad)
   (let ((sw (if (null str) 0 (string-width str)))
-	(i 0) (w 0) cw safe-w)
+	(i 0) (w 0) cw wid)
     (condition-case nil
 	(cond
 	 ((= sw width)
@@ -303,16 +287,15 @@
 		(concat (substring str 0 i) "..")
 	      (substring str 0 i)))
 	   (t
-	    ;; workaround for the bug of Emacs 20.7
-	    (setq safe-w (logand (- width w) 255))
+	    (setq wid (- width w))
 	    (if cnt
-		(concat (substring str 0 i) (make-string safe-w mew-sp) "..")
-	      (concat (substring str 0 i) (make-string safe-w mew-sp)))))))
+		(concat (substring str 0 i) (make-string wid mew-sp) "..")
+	      (concat (substring str 0 i) (make-string wid mew-sp)))))))
       (error
        (if (> (length mew-error-broken-string) width)
 	   (substring mew-error-broken-string 0 width)
 	 mew-error-broken-string)))))
-    
+
 ;;
 ;; Language specific
 ;;
@@ -324,33 +307,16 @@
 ;; Stolen from mule-cmd.el
 ;;
 
-(cond
- (mew-xemacs-p
-  (defsubst mew-set-coding-priority (pri) (set-coding-priority-list pri))
-  (defsubst mew-coding-category-list () (coding-priority-list))
-  (defsubst mew-coding-category-system (cat) (coding-category-system cat)))
- (t
-  (defsubst mew-set-coding-priority (pri) (set-coding-priority pri))
-  (defsubst mew-coding-category-list () coding-category-list)
-  (defsubst mew-coding-category-system (cat) (eval cat))))
+(defun mew-coding-category-list () coding-category-list)
+(defun mew-coding-category-system (cat) (eval cat))
 
 (defun mew-reset-coding-systems (priority categories)
-  (let ((rest-ctgs categories) checked-ctgs)
-    (while priority
-      (cond
-       (mew-xemacs-p
-	(unless (memq (car rest-ctgs) checked-ctgs)
-	  (when (car priority)
-	    (set-coding-category-system (car rest-ctgs) (car priority)))
-	  (setq checked-ctgs (cons (car rest-ctgs) checked-ctgs))))
-       (t
-	(set (car rest-ctgs) (car priority))))
-      (setq priority (cdr priority) rest-ctgs (cdr rest-ctgs)))
-    (cond
-     (mew-xemacs-p
-      (setq categories (nreverse checked-ctgs)))
-     (t
-      (update-coding-systems-internal)))
+  (let ((rest-ctgs categories))
+    (dolist (pri priority)
+      (set (car rest-ctgs) pri)
+      (setq rest-ctgs (cdr rest-ctgs)))
+    (if (fboundp 'update-coding-systems-internal) 
+	(update-coding-systems-internal))
     (mew-set-coding-priority categories)))
 
 (defun mew-set-language-environment-coding-systems (language-name)
@@ -371,13 +337,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2005 Mew developing team.
+;; Copyright (C) 1997-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -386,7 +352,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
