@@ -16,7 +16,13 @@
 (defvar mew-prog-wds       "wdsgrep.exe")
 (defvar mew-prog-google    "gdgrep.rb")
 (defvar mew-prog-est       "estcmd")
-(defvar mew-prog-namazu    "namazu")
+
+(mew-defstruct search
+	       key name prog
+	       func-search func-virtual
+	       func-index-folder func-index-all
+	       func-canonicalize-pattern
+	       func-register func-unregister func-filter)
 
 (defvar mew-search-switch
   `((spotlight "Spotlight" ,mew-prog-spotlight
@@ -34,16 +40,12 @@
      mew-google-index-folder mew-google-index-all
      mew-pick-canonicalize-pattern-google
      mew-google-register mew-google-unregister)
-    (est "Hyperestraier" ,mew-prog-est
+    (est "Hyper Estraier" ,mew-prog-est
      mew-search-with-est mew-search-virtual-with-est
      mew-est-index-folder mew-est-index-all
      mew-pick-canonicalize-pattern-est
-     nil nil)
-    (namazu "Namazu" ,mew-prog-namazu
-     mew-search-with-namazu nil
-     mew-nmz-index-folder nil
-     mew-pick-canonicalize-pattern-namazu
-     nil nil)))
+     nil nil
+     mew-est-input-filter)))
 
 (defun mew-search-get-list (func)
   (let ((sw mew-search-switch)
@@ -58,35 +60,6 @@
 (defun mew-search-get-ent (method)
   (assoc method mew-search-switch))
 
-(defun mew-search-get-key (ent)
-  (nth 0 ent))
-
-(defun mew-search-get-name (ent)
-  (nth 1 ent))
-
-(defun mew-search-get-prog (ent)
-  (nth 2 ent))
-
-(defun mew-search-get-func-search (ent)
-  (nth 3 ent))
-(defun mew-search-get-func-virtual (ent)
-  (nth 4 ent))
-
-(defun mew-search-get-func-index-folder (ent)
-  (nth 5 ent))
-
-(defun mew-search-get-func-index-all (ent)
-  (nth 6 ent))
-
-(defun mew-search-get-func-canonicalize-pattern (ent)
-  (nth 7 ent))
-
-(defun mew-search-get-func-register (ent)
-  (nth 8 ent))
-
-(defun mew-search-get-func-unregister (ent)
-  (nth 9 ent))
-
 (defvar mew-search-method (car (mew-search-get-list 'mew-search-get-key)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -109,62 +82,86 @@
       (setq mew-search-method (nth 0 (mew-assoc-equal name mew-search-switch 1))))))
 
 (defun mew-summary-search ()
-  "Pick messages according to a specified pick pattern 
+  "Pick messages according to a specified pick pattern
 with a search method. Then put the '*' mark onto them. "
   (interactive)
-  (if (not mew-search-method)
-      (message "No search method")
-    (let* ((ent (mew-search-get-ent mew-search-method))
-	   (func (mew-search-get-func-search ent))
-	   (name (mew-search-get-name ent))
-	   (canon-func (mew-search-get-func-canonicalize-pattern ent))
-	   (folder (mew-pickable-folder))
-	   pattern msgs)
-      (mew-pickable
+  (mew-pickable
+   (if (not mew-search-method)
+       (message "No search method")
+     (let* ((ent (mew-search-get-ent mew-search-method))
+	    (func (mew-search-get-func-search ent))
+	    (name (mew-search-get-name ent))
+	    (canon-func (mew-search-get-func-canonicalize-pattern ent))
+	    (flt-func (mew-search-get-func-filter ent))
+	    (folder (mew-summary-physical-folder))
+	    pattern msgs filter)
        (if (not (fboundp func))
 	   (message "This command cannot be used")
-	 (setq pattern (mew-input-pick-pattern (concat name " pick")))
-	 (when (and canon-func (fboundp canon-func))
-	   (setq pattern (funcall canon-func pattern)))
-	 (setq msgs (funcall func pattern folder))
-	 (mew-summary-pick-ls folder msgs))))))
+	 (setq pattern (if flt-func
+			   (read-string (concat name " pick pattern: "))
+			 (mew-input-pick-pattern (concat name " pick"))))
+	 (if (and (string= pattern "") (not (fboundp flt-func)))
+	     (message (mew-substitute-for-summary "Keyword must be specified. You may use '\\[mew-summary-pick]' instead"))
+	   (when (and canon-func (fboundp canon-func))
+	     (setq pattern (funcall canon-func pattern)))
+	   (if (fboundp flt-func)
+	       (setq filter (funcall flt-func)))
+	   (setq msgs (funcall func pattern folder filter))
+	   (mew-summary-pick-ls folder msgs)))))))
 
-(defun mew-summary-search-virtual (&optional ask-folder)
-  "Making Virtual mode according to a specified pick pattern
+(defun mew-summary-selection-by-search (&optional ask-folder)
+  "Making selection according to a specified pick pattern
 with a search method."
   (interactive "P")
   (if (not mew-search-method)
       (message "No search method")
-    (let* ((ent (mew-search-get-ent mew-search-method))
+    (let* ((ofolder (mew-summary-folder-name 'ext))
+	   (ent (mew-search-get-ent mew-search-method))
 	   (func (mew-search-get-func-virtual ent))
 	   (name (mew-search-get-name ent))
 	   (canon-func (mew-search-get-func-canonicalize-pattern ent))
-	   vfolder opattern pattern dfunc file opts rttl file-rttl flds)
+	   (flt-func (mew-search-get-func-filter ent))
+	   vfolder opattern pattern dfunc file opts rttl file-rttl flds filter)
       (if (not (fboundp func))
 	  (message "This command cannot be used")
 	(if ask-folder
-	    (setq flds (mew-input-folders (mew-summary-folder-name))))
-	(setq opattern (mew-input-pick-pattern (concat name " virtual")))
-	(if (and canon-func (fboundp canon-func))
-	    (setq pattern (funcall canon-func opattern))
-	  (setq pattern opattern))
-	(setq vfolder (mew-folder-to-virtual opattern))
-	(mew-summary-switch-to-folder vfolder)
-	(mew-sinfo-set-find-key opattern)
-	(make-local-variable 'mew-summary-form-mark-delete)
-	(setq mew-summary-form-mark-delete nil)
-	(make-local-variable 'mew-summary-form-mark-spam)
-	(setq mew-summary-form-mark-spam nil)
-	(when (mew-summary-exclusive-p)
-	  (with-temp-buffer
-	    (mew-set-buffer-multibyte t)
-	    (mew-piolet
-	     mew-cs-text-for-read mew-cs-text-for-write
-	     (setq file-rttl (funcall func pattern flds)))))
-	(mew-set '(file rttl) file-rttl)
-	(setq dfunc `(lambda () (mew-delete-file ,file)))
-	(setq opts (list "-i" file))
-	(mew-local-retrieve 'vir opts dfunc nil nil rttl)))))
+	    (setq flds (mew-input-folders
+			(or (mew-summary-folder-name)
+			    (mew-case-folder
+			     mew-case
+			     (mew-proto-inbox-folder (mew-proto mew-case)))))))
+	(setq opattern (if flt-func
+			   (read-string (concat name " virtual pattern: "))
+			 (mew-input-pick-pattern (concat name " virtual"))))
+	(if (and (string= opattern "") (not (fboundp flt-func)))
+	    (message (mew-substitute-for-summary "Keyword must be specified"))
+	  (if (string= opattern "") (setq opattern " "))
+	  (if (and canon-func (fboundp canon-func))
+	      (setq pattern (funcall canon-func opattern))
+	    (setq pattern opattern))
+	  (when (fboundp flt-func)
+	    (setq filter (funcall flt-func))
+	    (if (string= opattern " ") (setq opattern ""))
+	    (setq opattern (concat opattern filter)))
+	  (setq vfolder (mew-folder-to-selection opattern))
+	  (mew-summary-switch-to-folder vfolder)
+	  (mew-vinfo-set-mode 'selection)
+	  (mew-vinfo-set-physical-folder nil)
+	  (mew-vinfo-set-original-folder ofolder)
+	  (mew-sinfo-set-find-key opattern)
+	  (make-local-variable 'mew-summary-form-mark-delete)
+	  (setq mew-summary-form-mark-delete nil)
+	  (make-local-variable 'mew-summary-form-mark-spam)
+	  (setq mew-summary-form-mark-spam nil)
+	  (when (mew-summary-exclusive-p)
+	    (with-temp-buffer
+	      (mew-set-buffer-multibyte t)
+	      (mew-piolet mew-cs-text-for-read mew-cs-text-for-write
+		(setq file-rttl (funcall func pattern flds filter)))))
+	  (mew-set '(file rttl) file-rttl)
+	  (setq dfunc `(lambda () (mew-delete-file ,file)))
+	  (setq opts (list "-i" file))
+	  (mew-local-retrieve 'vir opts dfunc nil nil rttl))))))
 
 (defun mew-summary-make-index-folder ()
   "Make index for this folder."
@@ -259,7 +256,7 @@ with a search method."
      (mew-alet
       (call-process prog nil t nil "-onlyin" path pattern)))))
 
-(defun mew-search-with-spotlight (pattern folder)
+(defun mew-search-with-spotlight (pattern folder &optional dummy)
   (let ((path (mew-expand-folder folder))
 	msgs)
     (with-temp-buffer
@@ -273,7 +270,7 @@ with a search method."
       (setq msgs (sort (mapcar 'string-to-number msgs) '<))
       (mapcar 'number-to-string msgs))))
 
-(defun mew-search-virtual-with-spotlight (pattern flds)
+(defun mew-search-virtual-with-spotlight (pattern flds &optional dummy)
   (let* ((mpath (mew-expand-folder mew-folder-local))
 	 (mail-regex (regexp-quote (file-name-as-directory mpath)))
 	 (regex (format "^%s%s\\([0-9]+\\)\\(%s\\)?$" mail-regex (file-name-as-directory "\\(.*\\)") mew-suffix))
@@ -286,18 +283,18 @@ with a search method."
       (setq flds (cdr flds))
       (mew-search-spotlight pattern path)
       (goto-char (point-min))
-      (while (looking-at regex)
-	(setq rttl (1+ rttl))
-	(setq crnt (match-string 1))
-	(delete-region (match-beginning 0) (match-beginning 2))
-	(when (not (string= crnt prev))
-	  (beginning-of-line)
-	  (insert "CD:" mew-folder-local crnt "\n"))
-	(setq prev crnt)
-	(forward-line))
-      (mew-frwlet
-       mew-cs-text-for-read mew-cs-text-for-write
-       (write-region (point-min) (point-max) file (> n 1) 'no-msg))
+      (while (not (eobp))
+	(when (looking-at regex)
+	  (setq rttl (1+ rttl))
+	  (setq crnt (match-string 1))
+	  (delete-region (match-beginning 0) (match-beginning 2))
+	  (when (not (string= crnt prev))
+	    (beginning-of-line)
+	    (insert "CD:" mew-folder-local crnt "\n"))
+	  (setq prev crnt)
+	  (forward-line)))
+      (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	(write-region (point-min) (point-max) file (> n 1) 'no-msg))
       (mew-erase-buffer)
       (setq n (1+ n)))
     (list file rttl)))
@@ -320,7 +317,6 @@ with a search method."
 
 (defun mew-spotlight-index-folder (folder)
   "Making spotlight index for this folder."
-  (interactive)
   (let* ((dir (mew-expand-folder folder))
 	 (msgs (mew-dir-messages dir mew-regex-message-files3 'full)))
     (message "Spotlight indexing for %s..." folder)
@@ -355,7 +351,7 @@ with a search method."
      (mew-alet
       (call-process prog nil t nil "-m" "-p" path "-s" "-q" pattern)))))
 
-(defun mew-search-with-google (pattern folder)
+(defun mew-search-with-google (pattern folder &optional dummy)
   (let* ((path (mew-expand-folder folder))
 	 msgs)
     (with-temp-buffer
@@ -370,7 +366,7 @@ with a search method."
       (mapcar 'number-to-string msgs))))
 
 ;; xxx flds are not used at this moment
-(defun mew-search-virtual-with-google (pattern flds)
+(defun mew-search-virtual-with-google (pattern flds &optional dummy)
   (let* ((path (mew-expand-folder mew-folder-local))
 	 (mail-regex (regexp-quote (file-name-as-directory path)))
 	 (regex (concat "^" mail-regex "\\(.*\\)/" "\\([0-9]+\\)"))
@@ -379,23 +375,22 @@ with a search method."
 	 crnt)
     (mew-search-google pattern path)
     (goto-char (point-min))
-    (while (looking-at regex)
-      (setq rttl (1+ rttl))
-      (setq crnt (match-string 1))
-      (delete-region (match-beginning 0) (match-beginning 2))
-      (when (not (string= crnt prev))
-	(beginning-of-line)
-	(insert "CD:" mew-folder-local crnt "\n"))
-      (setq prev crnt)
-      (forward-line))
-    (mew-frwlet
-     mew-cs-text-for-read mew-cs-text-for-write
-     (write-region (point-min) (point-max) file nil 'no-msg))
+    (while (not (eobp))
+      (when (looking-at regex)
+	(setq rttl (1+ rttl))
+	(setq crnt (match-string 1))
+	(delete-region (match-beginning 0) (match-beginning 2))
+	(when (not (string= crnt prev))
+	  (beginning-of-line)
+	  (insert "CD:" mew-folder-local crnt "\n"))
+	(setq prev crnt)
+	(forward-line)))
+    (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+      (write-region (point-min) (point-max) file nil 'no-msg))
     (list file rttl)))
 
 (defun mew-google-index-folder (folder)
   "Make Google index for this folder."
-  (interactive)
   (let* ((ent (mew-search-get-ent mew-search-method))
 	 (prog (mew-search-get-prog ent)))
     (start-process prog nil prog "-m" "-i" (mew-expand-folder folder))
@@ -437,7 +432,7 @@ with a search method."
      (mew-alet
       (call-process prog nil t nil "-e" mew-suffix "-p" path "-s" pattern)))))
 
-(defun mew-search-with-wds (pattern folder)
+(defun mew-search-with-wds (pattern folder &optional dummy)
   (let* ((path (mew-expand-folder folder))
 	 msgs)
     (with-temp-buffer
@@ -452,7 +447,7 @@ with a search method."
       (mapcar 'number-to-string msgs))))
 
 ;; xxx flds are not used at this moment
-(defun mew-search-virtual-with-wds (pattern flds)
+(defun mew-search-virtual-with-wds (pattern flds &optional dummy)
   (let* ((path (mew-expand-folder mew-folder-local))
 	 (mail-regex (regexp-quote (file-name-as-directory path)))
 	 (regex (concat "^" mail-regex "\\(.*\\)/" "\\([0-9]+\\)"))
@@ -461,23 +456,22 @@ with a search method."
 	 crnt)
     (mew-search-wds pattern path)
     (goto-char (point-min))
-    (while (looking-at regex)
-      (setq rttl (1+ rttl))
-      (setq crnt (match-string 1))
-      (delete-region (match-beginning 0) (match-beginning 2))
-      (when (not (string= crnt prev))
-	(beginning-of-line)
-	(insert "CD:" mew-folder-local crnt "\n"))
-      (setq prev crnt)
+    (while (not (eobp))
+      (when (looking-at regex)
+	(setq rttl (1+ rttl))
+	(setq crnt (match-string 1))
+	(delete-region (match-beginning 0) (match-beginning 2))
+	(when (not (string= crnt prev))
+	  (beginning-of-line)
+	  (insert "CD:" mew-folder-local crnt "\n"))
+	(setq prev crnt))
       (forward-line))
-    (mew-frwlet
-     mew-cs-text-for-read mew-cs-text-for-write
-     (write-region (point-min) (point-max) file nil 'no-msg))
+    (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+      (write-region (point-min) (point-max) file nil 'no-msg))
     (list file rttl)))
 
 (defun mew-wds-index-folder (folder)
   "Make WDS index for all folders."
-  (interactive)
   (mew-wds-index-all))
 
 (defun mew-wds-index-all ()
@@ -500,49 +494,67 @@ with a search method."
   (interactive)
   (message "This command cannot be used"))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Namazu
-;;;
-
-(defun mew-search-with-namazu (pattern folder)
-  (mew-nmz-multi-pick (list (mew-nmz-expand-folder folder)) pattern nil 'single))
-
-(defun mew-nmz-index-folder (folder)
-  (mew-nmz-mknmz folder nil 'msg))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Hyper Estraier
 ;;;
 
+(defun mew-est-input-filter ()
+  (mew-input-pick-pattern "Hyper Estraier filter"))
+
 (defvar mew-search-est-db "casket")
 (defvar mew-prog-est-update "mewest")
 
-(defun mew-search-est (pattern path)
-  (setq pattern (mew-cs-encode-string pattern 'utf-8))
+(defun mew-search-est (pattern path filter)
+  (setq pattern (mew-cs-encode-string pattern mew-cs-est))
+  (if (string= filter "")
+      (setq filter nil)
+    (setq filter (mew-cs-encode-string filter mew-cs-est))
+    (setq filter (let ((mew-inherit-pick-omit-and t))
+		   (mew-pick-parse (mew-pick-lex filter))))
+    (setq filter (mapcar 'mew-pick-filter-est-kyvl filter)))
   (let* ((ent (mew-search-get-ent mew-search-method))
 	 (prog (mew-search-get-prog ent))
 	 (casket (expand-file-name mew-search-est-db mew-conf-path))
-	 maildir len attr)
-    (mew-plet
-     (mew-alet
-      (setq maildir (expand-file-name mew-home))
-      (setq len (length (file-name-as-directory maildir)))
-      (setq path (substring path len))
-      (setq path (file-name-as-directory path))
-      (setq attr (format "@uri STRINC %s" (mew-q-encode-string path ?%)))
-      (call-process prog nil t nil "search" "-vu" "-max" "-1" "-ord" "@cdate NUMA" "-attr" attr casket pattern)))))
+	 ;; mew-home is not good for Zaurus
+	 (pregex (concat "^"
+			 (regexp-quote
+			  (file-name-directory
+			   (directory-file-name
+			    (file-truename
+			     (expand-file-name mew-mail-path)))))))
+	 attr args)
+    (setq path (file-truename (file-name-as-directory path)))
+    (when (string-match pregex path)
+      (setq path (substring path (match-end 0)))
+      (mew-plet
+       (mew-alet
+	(setq attr (format "@uri STRINC %s" (mew-q-encode-string path ?%)))
+	(cond
+	 ((string-match "^ *ANDNOT " pattern)
+	  (setq pattern (concat "[UVSET] " pattern)))
+	 ((string-match "^ *$" pattern)
+	  (setq pattern "[UVSET]")))
+	(setq args (list "-vu" "-max" "-1" "-ord" "@cdate NUMA" casket pattern))
+	(unless (eq mew-cs-est 'utf-8)
+	  (setq args (cons (mew-cs-to-charset mew-cs-est) args))
+	  (setq args (cons "-ic" args)))
+	(setq filter (nreverse filter))
+	(while filter
+	  (setq args (cons "-attr" (cons (car filter) args)))
+	  (setq filter (cdr filter)))
+	(setq args (cons "-attr" (cons attr args)))
+	(apply 'call-process prog nil t nil "search" args))))))
 
-(defun mew-search-with-est (pattern folder)
+(defun mew-search-with-est (pattern folder filter)
   (let* ((path (mew-expand-folder folder))
-	 (regex (format "file://.*/%s/.*/\\([0-9]+\\)\\(%s\\)?$" 
+	 (regex (format "file://.*/%s/.*/\\([0-9]+\\)\\(%s\\)?$"
 			(file-name-nondirectory mew-mail-path)
 			(regexp-quote mew-suffix)))
 	 msgs)
     (with-temp-buffer
       (mew-set-buffer-multibyte t)
-      (mew-search-est pattern path)
+      (mew-search-est pattern path filter)
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
 	(setq msgs (cons (mew-match-string 1) msgs))
@@ -551,8 +563,8 @@ with a search method."
       (setq msgs (sort (mapcar 'string-to-number msgs) '<))
       (mapcar 'number-to-string msgs))))
 
-(defun mew-search-virtual-with-est (pattern flds)
-  (let* ((regex (format "file://.*/%s/\\(.*\\)/\\([0-9]+\\)\\(%s\\)?$" 
+(defun mew-search-virtual-with-est (pattern flds filter)
+  (let* ((regex (format "file://.*/%s/\\(.*\\)/\\([0-9]+\\)\\(%s\\)?$"
 			(file-name-nondirectory mew-mail-path)
 			(regexp-quote mew-suffix)))
 	 (file (mew-make-temp-name))
@@ -563,7 +575,7 @@ with a search method."
     (while flds
       (setq path (mew-expand-folder (car flds)))
       (setq flds (cdr flds))
-      (mew-search-est pattern path)
+      (mew-search-est pattern path filter)
       (goto-char (point-min))
       (setq start (point))
       (while (re-search-forward regex nil t)
@@ -577,9 +589,8 @@ with a search method."
 	(forward-line)
 	(setq start (point)))
       (delete-region start (point-max))
-      (mew-frwlet
-       mew-cs-text-for-read mew-cs-text-for-write
-       (write-region (point-min) (point-max) file (> n 1) 'no-msg))
+      (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	(write-region (point-min) (point-max) file (> n 1) 'no-msg))
       (mew-erase-buffer)
       (setq n (1+ n)))
     (list file rttl)))
@@ -591,7 +602,7 @@ with a search method."
       (message "\"%s\" does not exist" mew-prog-est-update)
     (message "Hyper Estraier indexing for %s..." folder)
     (let* ((path (file-truename (mew-expand-folder folder)))
-	   (pro (start-process "*Mew EST*" nil mew-prog-est-update path)))
+	   (pro (start-process "*Mew EST*" nil mew-prog-est-update "-s" mew-suffix path)))
       (set-process-filter pro 'mew-est-index-filter)
       (set-process-sentinel pro 'mew-est-index-sentinel))))
 
@@ -599,35 +610,188 @@ with a search method."
   "Make Hyper Estraier index for all folders."
   (interactive)
   (if (not (mew-which-exec mew-prog-est-update))
-      (message "")
+      (message "'%s' does not exist" mew-prog-est-update)
     (message "Hyper Estraier indexing...")
-    (let ((pro (start-process "*Mew EST*" nil mew-prog-est-update)))
+    (let ((pro (start-process "*Mew EST*" nil mew-prog-est-update "-s" mew-suffix)))
       (set-process-filter pro 'mew-est-index-filter)
       (set-process-sentinel pro 'mew-est-index-sentinel))))
 
 (defun mew-est-index-filter (process string)
-  (cond
-   ((string-match "exists" string)
-    (message "Another Hyper Estraier indexer is running"))
-   ((string-match "\.\.\.failed" string)
-    (message "Hyper Estraier indexing ...failed"))
-   ((string-match "\.\.\.done" string)
-    (message "Hyper Estraier indexing ...done"))))
+  (save-excursion
+    (cond
+     ((string-match "exists" string)
+      (message "Another Hyper Estraier indexer is running"))
+     ((string-match "\.\.\.failed" string)
+      (message "Hyper Estraier indexing ...failed"))
+     ((string-match "old messages\.\.\.done" string)
+      (message "Hyper Estraier purging ...done"))
+     ((string-match "new messages\.\.\.done" string)
+      (message "Hyper Estraier indexing ...done")))))
 
 (defun mew-est-index-sentinel (process event)
-  )
+  (save-excursion ))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; View
+;;;
+
+(defvar mew-prog-smew "smew")
+(defvar mew-prog-cmew "cmew")
+(defvar mew-id-db-file "id.db")
+(defvar mew-id-db-ignore-regex "^\\./casket$|^\\./casket/|/\\.")
+
+(defmacro mew-msgid-check (&rest body)
+  `(let ((db (mew-expand-file "+" mew-id-db-file)))
+     (cond
+      ((not (file-exists-p db))
+       (message "%s not found" mew-id-db-file))
+      ((not (mew-which-exec mew-prog-smew))
+       (message "%s not found" mew-prog-smew))
+      (t
+      ,@body))))
+
+(defun mew-summary-selection-by-msgid ()
+  "Creating Virtual mode with messages relating to the current message"
+  (interactive)
+  (mew-msgid-check
+   (mew-summary-msg
+    (let* ((ofolder (mew-summary-folder-name 'ext))
+	   (subj (or (mew-summary-get-subject) mew-error-no-subject))
+	   (str (mew-subject-simplify2 subj))
+	   (vfolder (mew-folder-to-selection
+		     (if (string= str "") mew-error-no-subject subj)))
+	   (regex (format "\\(.*\\)/\\([0-9]+\\)\\(%s\\)?$" (regexp-quote mew-suffix)))
+	   (rttl 0)
+	   (file (mew-make-temp-name))
+	   (db (mew-expand-file "+" mew-id-db-file))
+	   (mydir (substring (mew-path-to-folder (mew-expand-folder (mew-summary-folder-name))) 1))
+	   crnt start prev opts dfunc myid)
+      (mew-sumsyn-match mew-regex-sumsyn-long)
+      (setq myid (mew-sumsyn-my-id))
+      ;;
+      (mew-summary-switch-to-folder vfolder)
+      (mew-vinfo-set-mode 'selection)
+      (mew-vinfo-set-physical-folder nil)
+      (mew-vinfo-set-original-folder ofolder)
+      (mew-erase-buffer)
+      ;;
+      (make-local-variable 'mew-summary-form-mark-delete)
+      (setq mew-summary-form-mark-delete nil)
+      (make-local-variable 'mew-summary-form-mark-spam)
+      (setq mew-summary-form-mark-spam nil)
+      (when (mew-summary-exclusive-p)
+	(message "Creating selection by message-id...")
+	(mew-redraw)
+	(with-temp-buffer
+	  (mew-set-buffer-multibyte t)
+	  (mew-piolet mew-cs-text-for-read mew-cs-text-for-write
+	    (call-process mew-prog-smew nil t nil myid db mydir)
+	    (goto-char (point-min))
+	    (setq start (point))
+	    (while (re-search-forward regex nil t)
+	      (setq rttl (1+ rttl))
+	      (setq crnt (match-string 1))
+	      (delete-region start (match-beginning 2))
+	      (when (not (string= crnt prev))
+		(beginning-of-line)
+		(insert "CD:" mew-folder-local crnt "\n"))
+	      (setq prev crnt)
+	      (forward-line)
+	      (setq start (point)))
+	    (delete-region start (point-max))
+	    (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	      (write-region (point-min) (point-max) file nil 'no-msg))))
+	(setq dfunc `(lambda () (mew-delete-file ,file)))
+	(setq opts (list "-i" file))
+	(mew-local-retrieve 'vir opts dfunc nil nil rttl))))))
+
+(defun mew-summary-get-subject ()
+  (let* ((fld (mew-summary-folder-name))
+	 (msg (mew-summary-message-number))
+	 (cache (mew-cache-hit fld msg))
+	 subj)
+    (if (and cache (get-buffer cache))
+	(save-excursion
+	  (set-buffer cache)
+	  (setq subj (mew-header-get-value mew-subj:)))
+      (with-temp-buffer
+	(mew-insert-message
+	 fld msg mew-cs-text-for-read mew-header-reasonable-size)
+	(mew-refile-decode-subject)
+	(setq subj (mew-header-get-value mew-subj:))))
+    subj))
+
+(defun mew-summary-make-id-index-folder ()
+  "Make ID database for this folder."
+  (interactive)
+  (mew-summary-only
+   (cond
+    ((not (file-exists-p (expand-file-name mew-id-db-file mew-mail-path)))
+     (message (mew-substitute-for-summary "Type '\\[mew-summary-make-id-index-all]' to create ID database")))
+    ((not (mew-which-exec mew-prog-cmew))
+     (message "%s not found" mew-prog-cmew))
+    (t
+     (let* ((dbfile (expand-file-name mew-id-db-file mew-mail-path))
+	    (path (expand-file-name mew-mail-path))
+	    (regex mew-id-db-ignore-regex)
+	    (target (mew-folder-string
+		     (mew-path-to-folder
+		      (mew-expand-folder (mew-summary-folder-name 'ext)))))
+	    (options (list dbfile path regex target))
+	    (buf (generate-new-buffer (concat mew-buffer-prefix "cmew")))
+	    pro)
+       (message "Updating ID database...")
+       (setq pro (apply 'start-process "cmew" buf mew-prog-cmew options))
+       (set-process-filter pro 'mew-summary-make-id-index-filter)
+       (set-process-sentinel pro 'mew-summary-make-id-index-sentinel))))))
+
+(defun mew-summary-make-id-index-all (&optional full-update)
+  "Make ID database for all folders.
+If called with '\\[universal-argument]', the database is created 
+from scratch."
+  (interactive "P")
+  (if (not (mew-which-exec mew-prog-cmew))
+      (message "%s not found" mew-prog-cmew)
+    (let* ((dbfile (expand-file-name mew-id-db-file mew-mail-path))
+	   (path (expand-file-name mew-mail-path))
+	   (options (list dbfile path mew-id-db-ignore-regex))
+	   (buf (generate-new-buffer (concat mew-buffer-prefix "cmew")))
+	   pro)
+      (if full-update (setq options (cons "-f" options)))
+      (message "Updating ID database...")
+      (setq pro (apply 'start-process "cmew" buf mew-prog-cmew options))
+      (set-process-filter pro 'mew-summary-make-id-index-filter)
+      (set-process-sentinel pro 'mew-summary-make-id-index-sentinel))))
+
+(defun mew-summary-make-id-index-filter (process string)
+  (save-excursion
+    (mew-filter
+     (goto-char (point-max))
+     (insert string))))
+
+(defun mew-summary-make-id-index-sentinel (process event)
+  (let ((buf (process-buffer process)))
+    (save-excursion
+      (set-buffer buf)
+      (goto-char (point-max))
+      (forward-line -1)
+      (let ((str (mew-buffer-substring (line-beginning-position)
+				       (line-end-position))))
+	(message "Updating ID database...done (%s)" str)))
+    (mew-remove-buffer buf)))
 
 (provide 'mew-search)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2005-2006 Mew developing team.
+;; Copyright (C) 2005-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -636,7 +800,7 @@ with a search method."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
