@@ -9,10 +9,10 @@
 
 (defvar mew-ssh-prog-ver nil)		;; 0-SSH1, 1-SSH2, 2-OpenSSH, 3-PuTTY
 
-(defsubst mew-ssh-get (case list-or-vec)
+(defun mew-ssh-get (case list-or-vec)
   (elt list-or-vec (mew-ssh-prog-ver case)))
 
-(defconst mew-ssh-msg-passwd 
+(defconst mew-ssh-msg-passwd
   '("password:\\|Enter passphrase"
     "password:\\|Passphrase for\\|Enter passphrase"
     "password:\\|Enter passphrase"
@@ -52,20 +52,20 @@
 
 (defconst mew-ssh-info-prefix "mew-ssh-info-")
 
-(defsubst mew-ssh-info-name (sshserver server remoteport localport)
-  (format "%s:%s:%s:%s:%d"
+(defun mew-ssh-info-name (sshserver server remoteport localport)
+  (format "%s:%s:%s:%d:%d"
 	  mew-ssh-info-prefix sshserver server remoteport localport))
 
-(defsubst mew-ssh-info-name-regex (sshserver server remoteport)
-  (format "^%s:%s:%s:%s" mew-ssh-info-prefix sshserver server remoteport))
+(defun mew-ssh-info-name-regex (sshserver server remoteport)
+  (format "^%s:%s:%s:%d" mew-ssh-info-prefix sshserver server remoteport))
 
-(defsubst mew-ssh-pnm-to-lport (pnm)
+(defun mew-ssh-pnm-to-lport (pnm)
   (if (string-match ":\\([0-9]+\\)$" pnm) (match-string 1 pnm)))
 
 ;; user@host:port should be used here.
 ;; However, SSH does not symbolic port name "ssh" for the "-p" option
 ;; while OpenSSH does. Sigh...
-(defsubst mew-ssh-passtag (name)
+(defun mew-ssh-passtag (name)
   (if (string-match (format "^%s:[^:]+:" mew-ssh-info-prefix) name)
       (match-string 0 name)))
 
@@ -84,35 +84,32 @@ after ':'."
 	 (remoteport (mew-serv-to-port serv))
 	 (localport (+ 8000 (% (mew-random) 4000)))
 	 (process-connection-type mew-connection-type2)
-	 (i 0) (N mew-ssh-process-exec-cnt)
+	 (N mew-ssh-process-exec-cnt)
 	 (pros (process-list))
 	 (regex (mew-ssh-info-name-regex sshserver server remoteport))
 	 name pnm pro status)
     (cond
-     ((not (and (stringp server) (stringp serv) (stringp sshserver)))
-      ())
      ((not (mew-which-exec ssh))
       (message "'%s' is not found" ssh))
      (t
       (catch 'find
-	(while pros
-	  (when (string-match regex (process-name (car pros)))
-	    (if (memq (process-status (car pros)) '(run))
-		(setq pro (car pros))
-	      (delete-process (car pros)))
-	    (throw 'find nil))
-	  (setq pros (cdr pros))))
+	(dolist (pr pros)
+	  (when (string-match regex (process-name pr))
+	    (if (memq (process-status pr) '(run))
+		(setq pro pr)
+	      (delete-process pr))
+	    (throw 'find nil))))
       (if pro
 	  pro
 	(message "Connecting to the SSH server...")
 	(setq pro nil)
 	(catch 'loop
-	  (while (< i N)
+	  (dotimes (i N)
 	    (setq name (mew-ssh-info-name sshserver server remoteport localport))
 	    (setq pro (apply 'start-process
 			     name nil
 			     ssh "-x" "-v" "-L"
-			     (format "%d:%s:%s" localport server remoteport)
+			     (format "%d:%s:%d" localport server remoteport)
 			     (append args (list sshserver))))
 	    ;; An error would occur. So, let's exit in the case.
 	    (mew-timing)
@@ -125,7 +122,7 @@ after ':'."
 	      (message "Connecting to the SSH server...FAILED")
 	      (throw 'loop nil)))
 	    ;; ssh is now running.
-	    (process-kill-without-query pro)
+	    (mew-process-silent-exit pro)
 	    (setq pnm (process-name pro))
 	    (mew-info-clean-up pnm)
 	    (mew-ssh-set-try pnm 0)
@@ -144,7 +141,6 @@ after ':'."
 	      (throw 'loop nil))) ;; return value
 	    ;; 'bound
 	    (setq localport (1+ localport))
-	    (setq i (1+ i))
 	    (message "Connecting to the SSH server...FAILED")
 	    nil))))))) ;; return value
 
@@ -161,48 +157,50 @@ after ':'."
       (insert (format "\n<%s>\n%s\n" label string)))))
 
 (defun mew-ssh-filter (process string)
-  (mew-ssh-debug "SSH: " string)
-  (let* ((pnm (process-name process))
-	 (case (mew-ssh-get-case pnm))
-	 pass)
-    (cond
-     ((string-match (mew-ssh-get case mew-ssh-msg-passwd) string)
-      (if (= (mew-ssh-get-try pnm) 0)
-	  (setq pass (mew-input-passwd (format "SSH password (%s): "
-                                               (mew-ssh-get-account pnm))
-                                       (mew-ssh-passtag pnm)))
+  (save-excursion
+    (mew-ssh-debug "SSH: " string)
+    (let* ((pnm (process-name process))
+	   (case (mew-ssh-get-case pnm))
+	   pass)
+      (cond
+       ((string-match (mew-ssh-get case mew-ssh-msg-passwd) string)
+	(if (= (mew-ssh-get-try pnm) 0)
+	    (setq pass (mew-input-passwd (format "SSH password (%s): "
+						 (mew-ssh-get-account pnm))
+					 (mew-ssh-passtag pnm)))
+	  (mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
+	  (setq pass (mew-input-passwd (format "SSH password again (%s): "
+					       (mew-ssh-get-account pnm))
+				       (mew-ssh-passtag pnm))))
+	(mew-ssh-set-try pnm (1+ (mew-ssh-get-try pnm)))
+	(if (and (processp process)
+		 (memq (process-status process) '(run)))
+	    (progn
+	      (message "Sending password to the SSH server...")
+	      (process-send-string process (concat pass "\n")))
+	  (mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
+	  (message "Cannot find SSH process")))
+       ((or (string-match (mew-ssh-get case mew-ssh-msg-denied) string)
+	    (string-match (mew-ssh-get case mew-ssh-msg-refused) string))
+	(message "Cannot connect to the SSH server")
 	(mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
-	(setq pass (mew-input-passwd (format "SSH password again (%s): "
-                                             (mew-ssh-get-account pnm))
-				     (mew-ssh-passtag pnm))))
-      (mew-ssh-set-try pnm (1+ (mew-ssh-get-try pnm)))
-      (if (and (processp process)
-	       (memq (process-status process) '(run)))
-	  (progn
-	    (message "Sending password to the SSH server...")
-	    (process-send-string process (concat pass "\n")))
-	(mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
-	(message "Cannot find SSH process")))
-     ((or (string-match (mew-ssh-get case mew-ssh-msg-denied) string)
-	  (string-match (mew-ssh-get case mew-ssh-msg-refused) string))
-      (message "Cannot connect to the SSH server")
-      (mew-passwd-set-passwd (mew-ssh-passtag pnm) nil)
-      (mew-ssh-set-status pnm 'denied))
-     ((string-match (mew-ssh-get case mew-ssh-msg-connected) string)
-      (mew-ssh-set-status pnm t)
-      (message "Connecting to the SSH server...done"))
-     ((string-match "Local: bind: Address already in use" string)
-      (mew-ssh-set-status pnm 'bound)
-      (message "Connecting to the SSH server...done")))))
+	(mew-ssh-set-status pnm 'denied))
+       ((string-match (mew-ssh-get case mew-ssh-msg-connected) string)
+	(mew-ssh-set-status pnm t)
+	(message "Connecting to the SSH server...done"))
+       ((string-match "Local: bind: Address already in use" string)
+	(mew-ssh-set-status pnm 'bound)
+	(message "Connecting to the SSH server...done"))))))
 
 (defun mew-ssh-sentinel (process event)
   (let ((pnm (process-name process)))
-    (unless (mew-ssh-get-status pnm)
-      (mew-ssh-set-status pnm 'exit))))
+    (save-excursion
+      (unless (mew-ssh-get-status pnm)
+	(mew-ssh-set-status pnm 'exit)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; SSH version check 
+;;; SSH version check
 ;;;
 
 (defun mew-ssh-setup ()
@@ -230,13 +228,13 @@ after ':'."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2006 Mew developing team.
+;; Copyright (C) 1999-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -245,7 +243,7 @@ after ':'."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
