@@ -28,6 +28,7 @@ A file name of a certificate should be 'cert-hash.0'.
 (defvar mew-prog-ssl-arg nil) ;; xxx what about v4?
 
 (defvar mew-ssl-ver nil)
+(defvar mew-ssl-minor-ver nil)
 
 (defconst mew-ssl-process-exec-cnt 3)
 
@@ -41,12 +42,15 @@ A file name of a certificate should be 'cert-hash.0'.
 (defconst mew-tls-nntp "nntp")
 (defconst mew-tls-imap "imap") ;; xxx stunnel does not support this.
 
+;; stunnel does not support IPv6, sigh
+(defconst mew-ssl-localhost "127.0.0.1")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; SSL/TLS info
 ;;;
 
-(defvar mew-ssl-info-list '("status" "try" "file"))
+(defvar mew-ssl-info-list '("status" "try" "file" "string"))
 
 (mew-info-defun "mew-ssl-" mew-ssl-info-list)
 
@@ -57,13 +61,13 @@ A file name of a certificate should be 'cert-hash.0'.
 
 (defconst mew-ssl-info-prefix "mew-ssl-info-")
 
-(defsubst mew-ssl-info-name (server remoteport localport)
-  (format "%s:%s:%s:%d" mew-ssl-info-prefix server remoteport localport))
+(defun mew-ssl-info-name (server remoteport localport)
+  (format "%s:%s:%d:%d" mew-ssl-info-prefix server remoteport localport))
 
-(defsubst mew-ssl-info-name-regex (server remoteport)
-  (format "^%s:%s:%s" mew-ssl-info-prefix server remoteport))
+(defun mew-ssl-info-name-regex (server remoteport)
+  (format "^%s:%s:%d" mew-ssl-info-prefix server remoteport))
 
-(defsubst mew-ssl-pnm-to-lport (pnm)
+(defun mew-ssl-pnm-to-lport (pnm)
   (if (string-match ":\\([0-9]+\\)$" pnm) (match-string 1 pnm)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -71,17 +75,23 @@ A file name of a certificate should be 'cert-hash.0'.
 ;;; Executing SSL/TLS
 ;;;
 
+(defun mew-ssl-server (server)
+  (if (string= server "localhost")
+      mew-ssl-localhost
+    server))
+
 (defun mew-ssl-options (case server remoteport localport tls)
+  (setq server (mew-ssl-server server))
   (if (= mew-ssl-ver 3)
       (let (args)
 	(setq args
 	      `("-c" "-f"
 		"-a" ,(expand-file-name (mew-ssl-cert-directory case))
-		"-d" ,(format "localhost:%d" localport)
-		"-v" ,(int-to-string (mew-ssl-verify-level case))
+		"-d" ,(format "%s:%d" mew-ssl-localhost localport)
+		"-v" ,(number-to-string (mew-ssl-verify-level case))
 		"-D" "debug"
 		"-P" "none"
-		"-r" ,(format "%s:%s" server remoteport)
+		"-r" ,(format "%s:%d" server remoteport)
 		,@mew-prog-ssl-arg))
 	(if tls (setq args (cons "-n" (cons tls args))))
 	args)
@@ -92,27 +102,26 @@ A file name of a certificate should be 'cert-hash.0'.
 	(insert (format "verify=%d\n" (mew-ssl-verify-level case)))
 	(insert "foreground=yes\n")
 	(insert "debug=debug\n")
+	(if (>= mew-ssl-minor-ver 22)
+	    (insert "syslog=no\n"))
 	(insert "CApath=" (expand-file-name (mew-ssl-cert-directory case)) "\n")
 	(insert (format "[%d]\n" localport))
-	(insert (format "accept=localhost:%d\n" localport))
-	(insert (format "connect=%s:%s\n" server remoteport))
-	(if tls (insert (format "protocol=%s\n" tls)))
-	(mew-frwlet
-	 mew-cs-dummy mew-cs-text-for-write
-	 ;; NEVER use call-process-region for privacy reasons
-	 (write-region (point-min) (point-max) file nil 'no-msg))
+	(insert (format "accept=%s:%d\n" mew-ssl-localhost localport))
+	(insert (format "connect=%s:%d\n" server remoteport))
+	(if tls (insert (format "protocol=%s\nsslVersion=TLSv1\n" tls)))
+	(mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	  ;; NEVER use call-process-region for privacy reasons
+	  (write-region (point-min) (point-max) file nil 'no-msg))
 	(list file)))))
 
 (defun mew-open-ssl-stream (case server serv tls)
   "Open an SSL/TLS stream for SERVER's SERV.
 This function returns a process when an SSL/TLS connection is created
-successfully. 
+successfully.
 If TLS is nil, an SSL connection is created.
 If TLS is a magic word for 'stunnel', a TLS connection is created.
 A local port number can be obtained the process name after ':'. "
   (cond
-   ((not (and (stringp server) (stringp serv)))
-    nil)
    ((or (null mew-ssl-ver) (not (mew-which-exec mew-prog-ssl)))
     (message "'%s' is not found" mew-prog-ssl)
     nil)
@@ -120,28 +129,26 @@ A local port number can be obtained the process name after ':'. "
     (let* ((remoteport (mew-serv-to-port serv))
 	   (localport (+ 8000 (% (mew-random) 4000)))
 	   (process-connection-type mew-connection-type2)
-	   (i 0) (N mew-ssl-process-exec-cnt)
+	   (N mew-ssl-process-exec-cnt)
 	   (pros (process-list))
 	   (regex (mew-ssl-info-name-regex server remoteport))
-	   buf name pnm pro dummy bound opts)
+	   name pnm pro dummy bound opts)
       (catch 'find
-	(while pros
-	  (when (string-match regex (process-name (car pros)))
-	    (if (memq (process-status (car pros)) '(run))
-		(setq pro (car pros))
-	      (delete-process (car pros)))
-	    (throw 'find nil))
-	  (setq pros (cdr pros))))
+	(dolist (pr pros)
+	  (when (string-match regex (process-name pr))
+	    (if (memq (process-status pr) '(run))
+		(setq pro pr)
+	      (delete-process pr))
+	    (throw 'find nil))))
       (if pro
 	  pro
 	(message "Creating an SSL/TLS connection...")
 	(setq pro nil)
-	(setq buf (generate-new-buffer (concat mew-buffer-prefix "ssl")))
 	(catch 'loop
-	  (while (< i N)
+	  (dotimes (i N)
 	    (setq name (mew-ssl-info-name server remoteport localport))
 	    (setq opts (mew-ssl-options case server remoteport localport tls))
-	    (setq pro (apply 'start-process name buf mew-prog-ssl opts))
+	    (setq pro (apply 'start-process name nil mew-prog-ssl opts))
 	    ;; An error would occur. So, let's exit in the case.
 	    (cond
 	     ((not (processp pro))
@@ -152,7 +159,7 @@ A local port number can be obtained the process name after ':'. "
 	      (message "Creating an SSL/TLS connection...FAILED")
 	      (throw 'loop nil)))
 	    ;; stunnel is now running.
-	    (process-kill-without-query pro)
+	    (mew-process-silent-exit pro)
 	    (setq pnm (process-name pro))
 	    (mew-info-clean-up pnm)
 	    (mew-ssl-set-try pnm 0)
@@ -164,8 +171,7 @@ A local port number can be obtained the process name after ':'. "
 	    (if (eq (mew-ssl-get-status pnm) t)
 		(throw 'loop (setq bound t)))
 	    ;; bind-failure
-	    (setq localport (1+ localport))
-	    (setq i (1+ i))))
+	    (setq localport (1+ localport))))
 	(mew-ssl-set-status pnm nil)
 	(if (not bound)
 	    (progn
@@ -176,7 +182,7 @@ A local port number can be obtained the process name after ':'. "
 	  ;; So, we need to check the SSL/TLS tunnel with a dummy
 	  ;; tunneled connection here.
 	  (set-process-filter pro 'mew-ssl-filter2)
-	  (setq dummy (open-network-stream " *Mew dummy*" nil "localhost" localport))
+	  (setq dummy (open-network-stream " *Mew dummy*" nil mew-ssl-localhost localport))
 	  (mew-rendezvous (null (mew-ssl-get-status pnm)))
 	  (if (processp dummy) (delete-process dummy))
 	  (if (eq (mew-ssl-get-status pnm) t)
@@ -202,62 +208,75 @@ A local port number can be obtained the process name after ':'. "
       (insert (format "\n<%s>\n%s\n" label string)))))
 
 (defun mew-ssl-filter1 (process string)
-  (mew-ssl-debug "SSL/TLS: " string)
-  (let ((pnm (process-name process)))
-    (cond
-     ((string-match "bound to" string)
-      (mew-ssl-set-status pnm t))
-     ((string-match "gethostbyname: Valid name, no data record of requested type" string)
-      (mew-ssl-set-status pnm 'gethostbyname-failure))
-     ((string-match "gethostbyname: Host not found" string)
-      (mew-ssl-set-status pnm 'gethostbyname-failure))
-     ((string-match "Local: bind: Address already in use" string)
-      (mew-ssl-set-status pnm 'bind-failure)))))
+  (let* ((pnm (process-name process))
+	 (prev-str (mew-ssl-get-string pnm)))
+    (save-excursion
+      (mew-ssl-debug "SSL/TLS: " string)
+      (mew-ssl-set-string pnm string)
+      (setq string (concat prev-str string))
+      (cond
+       ((string-match "bound to" string)
+	(mew-ssl-set-status pnm t))
+       ((string-match "gethostbyname: Valid name, no data record of requested type" string)
+	(mew-ssl-set-status pnm 'gethostbyname-failure))
+       ((string-match "gethostbyname: Host not found" string)
+	(mew-ssl-set-status pnm 'gethostbyname-failure))
+       ((string-match "Local: bind: Address already in use" string)
+	(mew-ssl-set-status pnm 'bind-failure))))))
 
 (defun mew-ssl-filter2 (process string)
-  (mew-ssl-debug "SSL/TLS: " string)
-  (let ((pnm (process-name process)))
-    (cond
-     ((string-match "Negotiated ciphers\\|opened with SSL" string)
-      (mew-ssl-set-status pnm t))
-     ((string-match "Failed to initialize" string)
-      (mew-ssl-set-status pnm t)) ;; xxx
-     ((string-match "verify failed" string)
-      (mew-ssl-set-status pnm 'verify-failure)))))
+  (let* ((pnm (process-name process))
+	 (prev-str (mew-ssl-get-string pnm)))
+    (save-excursion
+      (mew-ssl-debug "SSL/TLS: " string)
+      (mew-ssl-set-string pnm string)
+      (setq string (concat prev-str string))
+      (cond
+       ((string-match "Negotiated ciphers\\|opened with SSL" string)
+	(mew-ssl-set-status pnm t))
+       ((string-match "Failed to initialize" string)
+	(mew-ssl-set-status pnm t)) ;; xxx
+       ((string-match "verify failed" string)
+	(mew-ssl-set-status pnm 'verify-failure))))))
 
 (defun mew-ssl-filter3 (process string)
-  (mew-ssl-debug "SSL/TLS: " string))
+  (save-excursion
+    (mew-ssl-debug "SSL/TLS: " string)))
 
 (defun mew-ssl-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (file (mew-ssl-get-file pnm)))
-    (mew-delete-file file)
-    (mew-remove-buffer (process-buffer process))))
+    (save-excursion
+      (mew-delete-file file))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; stunnel version check 
+;;; stunnel version check
 ;;;
 
 (defun mew-ssl-setup ()
   (if (not (mew-which-exec mew-prog-ssl))
       (setq mew-ssl-ver nil)
-    (let ((status (call-process mew-prog-ssl nil nil nil "-V")))
-      (if (= status 0)
-	  (setq mew-ssl-ver 3)
-	(setq mew-ssl-ver 4)))))
+    (with-temp-buffer
+      (call-process mew-prog-ssl nil t nil "-version")
+      (goto-char (point-min))
+      (if (looking-at "stunnel 4\\.\\([0-9]+\\)")
+	  (progn
+	    (setq mew-ssl-ver 4)
+	    (setq mew-ssl-minor-ver (string-to-number (mew-match-string 1))))
+	(setq mew-ssl-ver 3)))))
 
 (provide 'mew-ssl)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2005 Mew developing team.
+;; Copyright (C) 2002-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -266,7 +285,7 @@ A local port number can be obtained the process name after ':'. "
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
