@@ -1122,13 +1122,17 @@ Executing this command enables searching such information."
 (defun mew-summary-line ()
   "Toggle normal lines, wrapped lines and long lines."
   (interactive)
-  (let ((cwin (get-buffer-window (current-buffer)))
+  (let ((fld (mew-summary-folder-name))
+	(msg (mew-summary-message-number))
+	(cwin (get-buffer-window (current-buffer)))
 	(mwin (get-buffer-window (mew-buffer-message))))
     (if (not mwin)
 	(message "No message is displayed")
       (select-window mwin)
       (unwind-protect
-	  (mew-message-line)
+	  (progn
+	    (mew-message-line)
+	    (mew-message-mode-line fld msg))
 	(select-window cwin)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
