@@ -13,9 +13,6 @@
 (defvar mew-prog-application/msexcel "xlhtml")
 (defvar mew-prog-application/mspowerpoint "ppthtml")
 (defvar mew-prog-application/rtf "rtf2html")
-(defvar mew-prog-msword-ext "ooffice")
-(defvar mew-prog-msexcel-ext "ooffice")
-(defvar mew-prog-mspowerpoint-ext "ooffice")
 
 ;;;
 ;;; Text/Html, Application/Xml
@@ -24,19 +21,23 @@
 (defvar mew-format-html "%s.html")
 (defvar mew-format-xml  "%s.xml")
 
+(defvar mew-unix-browser "firefox")
+(defvar mew-unix-browser-arg `("-a" "firefox" "-remote" "openFile(%s)"))
+(defvar mew-unix-browser-form `(,mew-unix-browser ,mew-unix-browser-arg t))
+
 (defvar mew-prog-text/html           'mew-mime-text/html-w3m) ;; See w3m.el
-(defvar mew-prog-text/html-ext       '("mozilla" ("-remote" "openFile(%s)") t))
+(defvar mew-prog-text/html-ext       mew-unix-browser-form)
 
 (defvar mew-prog-text/xml            'mew-mime-text/html-w3m) ;; See w3m.el
-(defvar mew-prog-text/xml-ext        '("mozilla" ("-remote" "openFile(%s)") t))
+(defvar mew-prog-text/xml-ext        mew-unix-browser-form)
 
 (defvar mew-prog-application/xml     nil)
-(defvar mew-prog-application/xml-ext '("mozilla" ("-remote" "openFile(%s)") t))
+(defvar mew-prog-application/xml-ext mew-unix-browser-form)
 
 
 ;;;
 ;;; Image
-;;; 
+;;;
 
 (defvar mew-prog-image/*         'mew-mime-image/*)
 (defvar mew-prog-image/*-ext     '("display" ("-geometry" "+0+0")))
@@ -48,6 +49,7 @@
 (defvar mew-prog-plain    'mew-mime-text/plain)
 (defvar mew-prog-html     '(mew-mime-text/html mew-mime-text/html-ext))
 (defvar mew-prog-xml      '(mew-mime-text/xml  mew-mime-text/xml-ext))
+(defvar mew-prog-patch    '(mew-mime-text/plain mew-mime-text/patch-ext))
 (defvar mew-prog-enriched 'mew-mime-text/enriched)
 (defvar mew-prog-text     'mew-mime-text/plain)
 (defvar mew-prog-audio
@@ -71,15 +73,20 @@
 (defvar mew-prog-oasys           nil)
 (defvar mew-prog-octet-stream    'mew-mime-application/octet-stream)
 
-(defvar mew-prog-msword  `(mew-mime-application/msword  ,mew-prog-msword-ext))
-(defvar mew-prog-msexcel `(mew-mime-application/msexcel ,mew-prog-msexcel-ext))
-(defvar mew-prog-mspowerpoint `(mew-mime-application/mspowerpoint ,mew-prog-mspowerpoint-ext))
+(defvar mew-prog-msword-ext nil)
+(defvar mew-prog-msword nil)
+(defvar mew-prog-msexcel-ext nil)
+(defvar mew-prog-msexcel nil)
+(defvar mew-prog-mspowerpoint-ext nil)
+(defvar mew-prog-mspowerpoint nil)
 (defvar mew-prog-ooffice         "ooffice")
 (defvar mew-prog-visio           'mew-mime-application/octet-stream)
 (defvar mew-prog-mstnef          '(mew-mime-application-ms-tnef
 				   mew-mime-application-ms-tnef-ext))
 (defvar mew-prog-rtf             '(mew-mime-application/rtf
 				   mew-mime-application/rtf-ext))
+(defvar mew-prog-zip             'mew-mime-application/octet-stream)
+
 ;;;
 ;;; Misc
 ;;;
@@ -89,17 +96,24 @@
 
 (defun mew-set-file-type (file) ())
 
+(defvar mew-cs-est 'utf-8)
+
+(defun mew-focus-frame (frame)
+  (when focus-follows-mouse
+    (set-mouse-position
+     (selected-frame) (1- (frame-width)) 0)))
+
 (provide 'mew-unix)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2005 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -108,7 +122,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
