@@ -10,7 +10,7 @@
 ;;; (1) Variables closer to constant.
 ;;; (2) Variables for soft-coding.
 ;;; (3) Difficult options with some macros.
-;;; Use mew-add-first, mew-insert-after, mew-replace-with, 
+;;; Use mew-add-first, mew-insert-after, mew-replace-with,
 ;;; and mew-remove-entry to modify the variables.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -52,28 +52,27 @@
   "Regexp of various (was ...) expression in Subject:")
 
 (defvar mew-subject-simplify-replace-alist
-  (list
-   ;; (REGEXP . NEW) --- NEW may be nil for deletion (same to "")
-   ;; replace multiple Re: and Fw: into single Re:
-   (cons (concat "^" ;; regexp
-		 mew-reply-regex
-		 "\\("
-		 mew-reply-regex
-		 "\\|"
-		 mew-forward-regex
-		 "\\)*")
-	 'mew-reply-string) ;; new text
-   ;; replace multiple Fw: and Re: into single Fw:
-   (cons (concat "^" ;; regexp
-		 mew-forward-regex
-		 "\\("
-		 mew-reply-regex
-		 "\\|"
-		 mew-forward-regex
-		 "\\)*")
-	 'mew-forward-string) ;; new text
-   ;; delete extra string (no new text means delete)
-   (cons mew-was-regex nil)) ;; regexp
+  `(;; (REGEXP . NEW) --- NEW may be nil for deletion (same to "")
+    ;; replace multiple Re: and Fw: into single Re:
+    (,(concat "^" ;; regexp
+	      mew-reply-regex
+	      "\\("
+	      mew-reply-regex
+	      "\\|"
+	      mew-forward-regex
+	      "\\)*")
+     mew-reply-string) ;; new text
+    ;; replace multiple Fw: and Re: into single Fw:
+    (,(concat "^" ;; regexp
+	      mew-forward-regex
+	      "\\("
+	      mew-reply-regex
+	      "\\|"
+	      mew-forward-regex
+	      "\\)*")
+     mew-forward-string) ;; new text
+    ;; delete extra string (no new text means delete)
+    (,mew-was-regex nil)) ;; regexp
   "*Replacement alist to simplify Subject: field body
 Each element is cons cell whos car is REGEXP to replace,
 cdr is new text.")
@@ -152,7 +151,7 @@ for privacy reasons.")
 (defvar mew-address-separator ":, \t\n")
 (defvar mew-page-delimiter "^\^L")
 (defvar mew-keyval "^\\([^ \t:]+:?\\)[ \t]*")
-;; "^\\([^ \t:]+:?\\)[ \t]*\\(.*\\)$" faces a stupid error. 
+;; "^\\([^ \t:]+:?\\)[ \t]*\\(.*\\)$" faces a stupid error.
 ;; "$" does not mean end-of-line if the second match is empty, sigh.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -171,39 +170,44 @@ for privacy reasons.")
 ;;;
 
 (defvar mew-addrbook-switch
-  '((shortname . mew-addrbook-shortname-get)
-    (address   . identity)
-    (username  . mew-addrstr-extract-user)
-    (nickname  . mew-addrbook-nickname-get)
-    (name      . mew-addrbook-name-get))
+  '((shortname mew-addrbook-shortname-get)
+    (address   identity)
+    (username  mew-addrstr-extract-user)
+    (nickname  mew-addrbook-nickname-get)
+    (name      mew-addrbook-name-get))
   "Function database to get each field of Addrbook.
 'shortname, 'address, 'username, 'nickname, and 'name is defined.")
 
-(defsubst mew-addrbook-func (key)
-  (cdr (assq key mew-addrbook-switch)))
-
+(defun mew-addrbook-func (key)
+  (mew-alist-get-value (assq key mew-addrbook-switch)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; MIME types
 ;;;
 
-(defsubst mew-ct-textp (ct)
+(defun mew-ct-textp (ct)
   (string-match "^Text" (capitalize ct)))
 
-(defsubst mew-ct-imagep (ct)
+(defun mew-ct-imagep (ct)
   (string-match "^Image" (capitalize ct)))
 
-(defsubst mew-ct-modelp (ct)
+(defun mew-ct-modelp (ct)
   (string-match "^Model" (capitalize ct)))
 
-(defsubst mew-ct-multipartp (ct)
+(defun mew-ct-multipartp (ct)
   (string-match "^Multipart" (capitalize ct)))
 
-(defsubst mew-ct-messagep (ct)
+(defun mew-ct-alternativep (ct)
+  (string-match "^Multipart/Alternative" (capitalize ct)))
+
+(defun mew-ct-partialp (ct)
+  (string-match "^Message/Partial" (capitalize ct)))
+
+(defun mew-ct-messagep (ct)
   (string-match "^Message" (capitalize ct)))
 
-(defsubst mew-ct-linebasep (ct)
+(defun mew-ct-linebasep (ct)
   (or (mew-ct-textp ct)
       (mew-case-equal ct mew-ct-aps)
       (mew-case-equal ct mew-ct-msg))) ;; xxx mew-ct-messagep?
@@ -219,58 +223,58 @@ in draft buffer.")
 ;;;
 
 (defconst mew-custom-type-of-field-completion
-  '(alist key-type: string :value-type function))
+  '(list (list string function)))
 
 (defcustom mew-field-completion-switch
-  '(("To:"          . mew-complete-address)
-    ("Cc:"          . mew-complete-address)
-    ("Dcc:"         . mew-complete-address)
-    ("Bcc:"         . mew-complete-address)
-    ("Reply-To:"    . mew-complete-address)
-    ("Fcc:"         . mew-complete-local-folder)
-    ("Resent-To:"   . mew-complete-address)
-    ("Resent-Cc:"   . mew-complete-address)
-    ("Resent-Dcc:"  . mew-complete-address)
-    ("Resent-Bcc:"  . mew-complete-address)
-    ("Resent-Fcc:"  . mew-complete-local-folder)
-    ("Newsgroups:"  . mew-complete-newsgroups)
-    ("Followup-To:" . mew-complete-newsgroups))
+  '(("To:"          mew-complete-address)
+    ("Cc:"          mew-complete-address)
+    ("Dcc:"         mew-complete-address)
+    ("Bcc:"         mew-complete-address)
+    ("Reply-To:"    mew-complete-address)
+    ("Fcc:"         mew-complete-fcc-folder)
+    ("Resent-To:"   mew-complete-address)
+    ("Resent-Cc:"   mew-complete-address)
+    ("Resent-Dcc:"  mew-complete-address)
+    ("Resent-Bcc:"  mew-complete-address)
+    ("Resent-Fcc:"  mew-complete-fcc-folder)
+    ("Newsgroups:"  mew-complete-newsgroups)
+    ("Followup-To:" mew-complete-newsgroups))
   "*Completion function alist concerned with the key."
   :group 'mew-complete
   :type mew-custom-type-of-field-completion)
 
 (defcustom mew-field-circular-completion-switch
-  '(("To:"          . mew-circular-complete-domain)
-    ("Cc:"          . mew-circular-complete-domain)
-    ("Dcc:"         . mew-circular-complete-domain)
-    ("Bcc:"         . mew-circular-complete-domain)
-    ("Reply-To:"    . mew-circular-complete-domain)
-    ("Resent-To:"   . mew-circular-complete-domain)
-    ("Resent-Cc:"   . mew-circular-complete-domain)
-    ("Resent-Dcc:"  . mew-circular-complete-domain)
-    ("Resent-Bcc:"  . mew-circular-complete-domain)
-    ("From:"        . mew-circular-complete-from)
-    ("Resent-From:" . mew-circular-complete-from))
+  '(("To:"          mew-circular-complete-domain)
+    ("Cc:"          mew-circular-complete-domain)
+    ("Dcc:"         mew-circular-complete-domain)
+    ("Bcc:"         mew-circular-complete-domain)
+    ("Reply-To:"    mew-circular-complete-domain)
+    ("Resent-To:"   mew-circular-complete-domain)
+    ("Resent-Cc:"   mew-circular-complete-domain)
+    ("Resent-Dcc:"  mew-circular-complete-domain)
+    ("Resent-Bcc:"  mew-circular-complete-domain)
+    ("From:"        mew-circular-complete-from)
+    ("Resent-From:" mew-circular-complete-from))
   "*Circular completion function alist concerned with the key."
   :group 'mew-complete
   :type mew-custom-type-of-field-completion)
 
 (defcustom mew-field-expansion-switch
-  '(("To:"         . mew-expand-address)
-    ("Cc:"         . mew-expand-address)
-    ("Dcc:"        . mew-expand-address)
-    ("Bcc:"        . mew-expand-address)
-    ("Reply-To:"   . mew-expand-address)
-    ("Resent-To:"  . mew-expand-address)
-    ("Resent-Cc:"  . mew-expand-address)
-    ("Resent-Dcc:" . mew-expand-address)
-    ("Resent-Bcc:" . mew-expand-address))
+  '(("To:"         mew-expand-address)
+    ("Cc:"         mew-expand-address)
+    ("Dcc:"        mew-expand-address)
+    ("Bcc:"        mew-expand-address)
+    ("Reply-To:"   mew-expand-address)
+    ("Resent-To:"  mew-expand-address)
+    ("Resent-Cc:"  mew-expand-address)
+    ("Resent-Dcc:" mew-expand-address)
+    ("Resent-Bcc:" mew-expand-address))
   "*Expansion function alist concerned with the key."
   :group 'mew-complete
   :type mew-custom-type-of-field-completion)
 
-(defsubst mew-field-get-func (key switch)
-  (cdr (mew-assoc-match key switch 0)))
+(defun mew-field-get-func (key switch)
+  (mew-alist-get-value (mew-assoc-match key switch 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -280,113 +284,10 @@ in draft buffer.")
 (defvar mew-content-type mew-ct-txt
   "*The default Content-Type: for a file whose suffix is unknown.")
 
-(defvar mew-mime-content-type
-  `(("multipart"    nil         nil     nil            mew-icon-multipart)
-    ;;
-    ("audio/basic"  "\\.au$"    mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-wav"  "\\.wav$"   mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-aiff" "\\.aif?f$" mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-midi" "\\.midi?$" mew-b64 mew-prog-audio mew-icon-audio)
-    ("audio/x-mpeg" "\\.mpga$\\|\\.mp[23]$" mew-b64 mew-prog-audio mew-icon-audio)
-    ;;
-    ("image/gif"   "\\.gif$"   mew-b64 mew-prog-image mew-icon-image gif)
-    ("image/tiff"  "\\.tif?f$" mew-b64 mew-prog-image mew-icon-image tiff)
-    ("image/jpeg"  "\\.jpe?g$" mew-b64 mew-prog-image mew-icon-image jpeg)
-    ("image/png"   "\\.png$"   mew-b64 mew-prog-image mew-icon-image png)
-    ("image/x-xwd" "\\.xwd$"   mew-b64 mew-prog-image mew-icon-image xwd)
-    ("image/x-xbm" "\\.xbm$"   mew-b64 mew-prog-image mew-icon-image xbm)
-    ("image/x-xpm" "\\.xpm$"   mew-b64 mew-prog-image mew-icon-image xpm)
-    ("image/x-bmp" "\\.bmp$"   mew-b64 mew-prog-image mew-icon-image bmp)
-    ("image/x-pcx" "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image PCX)
-    ("image/x-tga" "\\.tga$"   mew-b64 mew-prog-image mew-icon-image TGA)
-    ("image"       "^$"        mew-b64 mew-prog-image mew-icon-image)
-    ;;
-    ("model/iges" "\\.ige?s$" mew-b64 mew-prog-iges  mew-icon-image) ;; xxx
-    ("model/vrml" "\\.wrl$"   mew-b64 mew-prog-vrml  mew-icon-image)
-    ("model/mesh" "\\.me?sh$" mew-b64 mew-prog-mesh  mew-icon-image)
-    ("model"      "^$"        mew-b64 mew-prog-model mew-icon-image)
-    ;;
-    ("video/mpeg"      "\\.mpe?g$" mew-b64 mew-prog-video mew-icon-video)
-    ("video/quicktime" "\\.mov$"   mew-b64 mew-prog-video mew-icon-video)
-    ("video/x-msvideo" "\\.avi$"   mew-b64 mew-prog-video mew-icon-video)
-    ;;
-    ("message/rfc822"          ,(format "^[0-9]+\\(%s\\)?$" mew-suffix) nil
-     mew-prog-rfc822           mew-icon-message/rfc822)
-    ("message/external-body"   "\\.ext$"  nil
-     mew-prog-external-body    mew-icon-message/external-body)
-    ("message/delivery-status" "^$"       nil
-     mew-prog-delivery-status  mew-icon-text)
-    ;;
-    ("application/postscript"        "\\.e?ps$" mew-qp
-     mew-prog-postscript             mew-icon-application/postscript)
-    ("application/pdf"               "\\.pdf$"  mew-b64
-     mew-prog-pdf                    mew-icon-application/postscript)
-    ("application/msword"            "\\.doc$"  mew-b64
-     mew-prog-msword                 mew-icon-text)
-    ("application/vnd.ms-excel"      "\\.xl[st]$" mew-b64
-     mew-prog-msexcel                mew-icon-text)
-    ("application/vnd.ms-powerpoint" "\\.ppt$"  mew-b64
-     mew-prog-mspowerpoint           mew-icon-text)
-    ("application/vnd.visio"         "\\.vsd$"  mew-b64
-     mew-prog-visio                  mew-icon-text)
-    ("application/vnd.sun.xml.writer"           "\\.sxw$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.writer.template"  "\\.stw$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.calc"             "\\.sxc$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.calc.template"    "\\.stc$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.draw"             "\\.sxd$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.draw.template"    "\\.std$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.impress"          "\\.sxi$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.impress.template" "\\.sti$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.writer.global"    "\\.sxg$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/vnd.sun.xml.math"             "\\.sxm$"  mew-b64
-     mew-prog-ooffice                mew-icon-text)
-    ("application/ms-tnef"           "\\.dat$" mew-b64
-     mew-prog-mstnef                 mew-icon-unknown)
-    ("application/rtf"               "\\.rtf$" mew-b64
-     mew-prog-rtf                    mew-icon-text)
-    ("application/pgp-keys"          "\\.pka$"  nil
-     mew-prog-pgp-keys               mew-icon-unknown)
-    ("application/x-pkcs7-signature" "\\.p7s$"  mew-b64
-     nil			     mew-icon-unknown)
-    ("application/vnd.fujitsu.oasys" "\\.oas$"  mew-b64
-     mew-prog-oasys                  mew-icon-text)
-    ("application/vnd.fujitsu.oasys2" "\\.oa2$" mew-b64
-     mew-prog-oasys                  mew-icon-text)
-    ("application/vnd.fujitsu.oasys3" "\\.oa3$" mew-b64
-     mew-prog-oasys                  mew-icon-text)
-    ("application/octet-stream"
-     "\\.tar$\\|\\.tar\\.\\|\\.gz$\\|\\.Z$\\|\\.taz$\\|\\.tgz$\\|\\.tbz$\\|\\.bz2?$\\|\\.lzh$\\|\\.zip$\\|\\.bin$\\|\\.pgp$\\|\\.gpg$\\|\\.exe$\\|\\.dll$"
-      mew-b64 mew-prog-octet-stream mew-icon-application/octet-stream)
-    ;;
-    ("text/html"     "\\.html?$" nil     mew-prog-html      mew-icon-text)
-    ("text/enriched" "\\.rtf$"   nil     mew-prog-enriched  mew-icon-text)
-    ("text/css"      "\\.css$"   nil     mew-prog-text      mew-icon-text)
-    ("text/sgml"     "\\.sgml$"  nil     mew-prog-text      mew-icon-text)
-    ("text/plain"    "\\.txt$\\|\\.c$\\|\\.h$\\|\\.el$\\|\\.diff$\\|\\.patch$"
-                                 nil     mew-prog-plain     mew-icon-text)
-    ("text/xml"      "\\.xml$"   nil     mew-prog-xml       mew-icon-text)
-    ("text/rfc822-headers" "\\.hdr$" nil
-     mew-prog-rfc822-headers     mew-icon-message/rfc822)
-    ("text"         "^$"        nil     mew-prog-text      mew-icon-text)
-    ;; Unknown CT: matches here.
-    ("application/xml"               "\\.xml$"  mew-b64
-     mew-prog-xml2                   mew-icon-text)
-    (t               "^$"       nil   mew-prog-octet-stream mew-icon-unknown)
-    ;; Unknown suffix matches here and return the entry specified 
-    ;; by mew-content-type.
-    (nil             ".*"))
+(defvar mew-mime-content-type nil
   "\\<mew-summary-mode-map>Database for MIME content type.
-This is a list of entries which consist of <content-type>, 
-<regular expression of file suffix>, <content-transfer-encoding>, 
+This is a list of entries which consist of <content-type>,
+<regular expression of file suffix>, <content-transfer-encoding>,
 <symbol for visualizer>, <symbol for icon>, <auxiliary data>.
 
 Here is an example:
@@ -429,57 +330,61 @@ whose format is one of followings:
 	Equivalent to (\"prog\" args async).
 - (nil func2)
 	(1) Displayed 'type \\[mew-summary-execute-external]'.
-	(2) <func2> is called. 
+	(2) <func2> is called.
 - (func1 func2)
 	(1) <func1> is called.
 	(2) <func2> is called.
 	    If the original content-type is Application/Octet-Stream,
 	    you are asked to choose <func1> or <func2>.
 - (func1 \"prog\")
-	(1) <func1> is called. 
+	(1) <func1> is called.
 	(2) \"prog\" is executed asynchronously.
 	    If the original content-type is Application/Octet-Stream,
 	    you are asked to choose <func1> or \"prog\".
-- (func1 (\"prog\" args async)) 
-	(1) <func1> is called. 
+- (func1 (\"prog\" args async))
+	(1) <func1> is called.
 	(2) \"prog\" is executed asynchronously with <args>.
 	    If the original content-type is Application/Octet-Stream,
 	    you are asked to choose <func1> or \"prog\".
 
 Note that (1) indicates the action for '\\[mew-summary-display]' and
-(2) indicates the action for '\\[mew-summary-execute-external]'.
+\(2) indicates the action for '\\[mew-summary-execute-external]'.
 
 If you want to add an entry for new content-type, please let the author
 know. Such an entry should be shared by all users and be included in
 the next version of Mew.")
 
-(defsubst mew-ctdb-by-ct (ct)
+(defun mew-ctdb-by-ct (ct)
   (mew-assoc-match2 ct mew-mime-content-type 0))
 
-(defsubst mew-ctdb-by-file (file)
+(defun mew-ctdb-by-file (file)
   (mew-assoc-match2 file mew-mime-content-type 1))
 
-(defsubst mew-ctdb-ct (attr)
+(defun mew-ctdb-ct (attr)
   (and (nth 0 attr) (mew-capitalize (nth 0 attr))))
 
-(defsubst mew-ctdb-regex (attr)
+(defun mew-ctdb-regex (attr)
   (nth 1 attr))
 
-(defsubst mew-ctdb-cte (attr)
+(defun mew-ctdb-cte (attr)
   (symbol-value (nth 2 attr)))
 
-(defsubst mew-ctdb-prog (attr)
+(defun mew-ctdb-prog (attr)
   (let ((val (symbol-value (nth 3 attr))))
     (if (and (listp val) (eq 'if (car val)))
 	(eval val)
       val)))
 
-(defsubst mew-ctdb-icon (attr)
+(defun mew-ctdb-icon (attr)
   (symbol-value (nth 4 attr)))
 
-(defsubst mew-mime-image-format-name (ct)
+(defun mew-mime-image-format-name (ct)
   (nth 5 (mew-ctdb-by-ct ct)))
 
+(defvar mew-mime-content-type-for-ooffice nil
+  "A list of (content-type suffix) for OpenOffice.org to be prepended
+'mew-mime-content-type'")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; MIME Content-Disposition:
@@ -493,13 +398,13 @@ the next version of Mew.")
     (t           "attachment" t))
   "(content-type inline/attachment filename)")
 
-(defsubst mew-cdpdb-by-ct (ct)
+(defun mew-cdpdb-by-ct (ct)
   (mew-assoc-match2 ct mew-mime-content-disposition 0))
 
-(defsubst mew-cdpdb-val (attr)
+(defun mew-cdpdb-val (attr)
   (nth 1 attr))
 
-(defsubst mew-cdpdb-file (attr)
+(defun mew-cdpdb-file (attr)
   (nth 2 attr))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -518,22 +423,22 @@ the next version of Mew.")
     (smime-encryption ((,mew-ct-smm ,mew-ct-smm-enc)) "SE")
     (smime-signature-encryption
      ((,mew-ct-smm ,mew-ct-smm-sig) (,mew-ct-smm ,mew-ct-smm-enc)) "SSSE")
-    (smie-encryption-signature
+    (smime-encryption-signature
      ((,mew-ct-smm ,mew-ct-smm-enc) (,mew-ct-smm ,mew-ct-smm-sig)) "SESS"))
   "*Alist of key, a list of privacy Content-Type, and its mark."
   :group 'mew-privacy
   :type 'sexp)
 
-(defsubst mew-pcdb-services ()
+(defun mew-pcdb-services ()
   (mapcar 'car mew-privacy-database))
 
-(defsubst mew-pcdb-by-service (service)
+(defun mew-pcdb-by-service (service)
   (assq service mew-privacy-database))
 
-(defsubst mew-pcdb-ct (pcdb)
+(defun mew-pcdb-ct (pcdb)
   (nth 1 pcdb))
 
-(defsubst mew-pcdb-mark (pcdb)
+(defun mew-pcdb-mark (pcdb)
   (nth 2 pcdb))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -544,14 +449,14 @@ the next version of Mew.")
 (defvar mew-mime-multipart-alternative-list
   `(,mew-ct-txt ".*"))
 
-(defsubst mew-multipart-alternative-preference (part)
+(defun mew-multipart-alternative-preference (part)
   (let ((ct (mew-syntax-get-value (mew-syntax-get-ct part) 'cap)))
     (mew-member-match ct mew-mime-multipart-alternative-list)))
 
 (defvar mew-mime-external-body-list
   '("anon-ftp" "url" "mail-server"))
 
-(defsubst mew-mime-external-body-preference (part)
+(defun mew-mime-external-body-preference (part)
   (let* ((ctl (mew-syntax-get-ct part))
 	 (ct (mew-syntax-get-value ctl 'cap))
 	 (access-type (mew-syntax-get-param ctl "access-type")))
@@ -584,14 +489,15 @@ the next version of Mew.")
     (,mew-in-reply-to:   unstruct   unstruct)
     (,mew-x-face:	 unstruct   unstruct)
     (,mew-face:          unstruct   unstruct)
-    ("Domainkey-Signature:" unstruct unstruct) ;; capitalized
-    ("Dkim-Signature:"      unstruct unstruct)) ;; capitalized
+    ("Authentication-Results:" unstruct struct2) ;; capitalized
+    ("Domainkey-Signature:"    unstruct unstruct) ;; capitalized
+    ("Dkim-Signature:"         unstruct unstruct)) ;; capitalized
   "(field enc dec)")
 
-(defsubst mew-field-type-for-encoding (key)
+(defun mew-field-type-for-encoding (key)
   (or (nth 1 (assoc (mew-capitalize key) mew-field-database)) 'unstruct))
 
-(defsubst mew-field-type-for-decoding (key)
+(defun mew-field-type-for-decoding (key)
   (or (nth 2 (assoc (mew-capitalize key) mew-field-database)) 'text))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -641,10 +547,12 @@ the next version of Mew.")
     ("^\\(Message-Id\\|Posted\\|In-Reply-To\\|References\\|Precedence\\):$" nil)
     ("^DomainKey-Signature:$" nil)
     ("^DKIM-Signature:$" nil)
-    ("^Authentication-Results:$" t
+    ("^Authentication-Results:$"
+     mew-authentication-results
      mew-face-header-important
      mew-face-header-xmew
-     ("fail" mew-face-header-xmew-bad
+     ("hardfail" mew-face-header-xmew-bad
+      "fail" mew-face-header-xmew-bad
       "softfail" mew-face-header-xmew-bad))
     ("^Delivered-" nil)
     ("^List-" nil) ;; RFC 2369
@@ -659,12 +567,12 @@ the next version of Mew.")
      mew-face-header-private
      mew-face-header-marginal))
   "*An alist of field spec for Message mode. Each spec
-consists of field-regular-expression, visible-p, face-for-key, 
-and face-for-value. Fields whose visible-p is t are displayed in 
+consists of field-regular-expression, visible-p, face-for-key,
+and face-for-value. Fields whose visible-p is t are displayed in
 Message mode in the defined order. Fields whose visible-p is nil are
 hidden in Message mode. Type DEL to see them. Fields not matched
 to field-regular-expressions are operated by the value of
-'mew-field-other-visible'. If face-for-key is omitted, 
+'mew-field-other-visible'. If face-for-key is omitted,
 'mew-face-header-key' is used. If face-for-value is not
 present, mew-face-header-marginal is used."
   :group 'mew-highlight
@@ -673,25 +581,70 @@ present, mew-face-header-marginal is used."
                   (choice (list boolean)
                           (list boolean face face))))
 
-;; cons the position to the spec. 
-(defsubst mew-nspec-by-key (key)
+;; cons the position to the spec.
+(defun mew-nspec-by-key (key)
   (mew-assoc-match3 key mew-field-spec 0))
 
-(defsubst mew-nspec-n (nspec)
+(defun mew-nspec-n (nspec)
   (nth 0 nspec))
 
-(defsubst mew-nspec-visiblep (nspec)
+(defun mew-nspec-visiblep (nspec)
   (nth 2 nspec))
 
-(defsubst mew-nspec-keyface (nspec)
+(defun mew-nspec-keyface (nspec)
   (nth 3 nspec))
 
-(defsubst mew-nspec-valface (nspec)
+(defun mew-nspec-valface (nspec)
   (nth 4 nspec))
 
-(defsubst mew-nspec-extraface (nspec)
+(defun mew-nspec-extraface (nspec)
   (nth 5 nspec))
 
+(defun mew-authentication-results (val)
+  (save-match-data
+    (or (string-match "=hardfail" val)
+	(string-match "=fail" val)
+	(string-match "=softfail" val))))
+
+(mew-defstruct header-color field key val)
+(defvar mew-header-color-alist nil)
+
+(defun mew-header-color-base (field key-color val-color bold)
+  (unless val-color (setq val-color key-color))
+  (let ((face-key (intern (concat "mew-face-color-" key-color (if bold "-bold"))))
+	(face-val (intern (concat "mew-face-color-" val-color (if bold "-bold")))))
+    (unless (facep face-key)
+      (make-empty-face face-key)
+      (face-spec-set face-key `((t (:foreground ,key-color :bold ,bold)))))
+    (unless (facep face-val)
+      (make-empty-face face-val)
+      (face-spec-set face-val `((t (:foreground ,val-color :bold ,bold)))))
+    (setq mew-header-color-alist
+	  (cons (list (capitalize field) face-key face-val) mew-header-color-alist))))
+
+(defun mew-header-color (field key-color &optional val-color)
+  "Set the color of FIELD key to KEY-COLOR. 
+And set the color of FIELD value to VAL-COLOR.
+If VAL-COLOR is omitted, set the color of FIELD value to KEY-COLOR."
+  (mew-header-color-base field key-color val-color nil))
+
+(defun mew-header-color-bold (field key-color &optional val-color)
+  "Set the color of FIELD key to KEY-COLOR. 
+And set the color of FIELD value to VAL-COLOR.
+If VAL-COLOR is omitted, set the color of FIELD value to KEY-COLOR.
+The style becomes bold."
+  (mew-header-color-base field key-color val-color t))
+
+(defun mew-key-face (key nspec)
+  (or (mew-header-color-get-key (assoc key mew-header-color-alist))
+      (mew-nspec-keyface nspec)
+      'mew-face-header-key))
+
+(defun mew-val-face (key nspec)
+  (or (mew-header-color-get-val (assoc key mew-header-color-alist))
+      (mew-nspec-valface nspec)
+      'mew-face-header-marginal))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Highlight body
@@ -700,7 +653,7 @@ present, mew-face-header-marginal is used."
 (defvar mew-highlight-body-regex-comment "^#+.*")
 
 (defvar mew-highlight-body-regex-cite
-  "^\\(\\([ \t]*\\([>:|]\\|\\w+\\(['._-]+\\w+\\)*>+\\)\\)+\\).*")
+  "^\\(\\([ \t]\\{,7\\}\\([>:|]\\|\\w+\\([._-]+\\w+\\)*>+\\)\\)+\\).*")
 
 (defcustom mew-highlight-body-prefix-width 20
   "*Maximum string width assume prefix for fancy highlight a body."
@@ -720,16 +673,16 @@ present, mew-face-header-marginal is used."
 ;;;
 
 (defvar mew-highlight-mark-keywords
-  `((,mew-mark-review . mew-face-mark-review)
-    (,mew-mark-escape . mew-face-mark-escape)
-    (,mew-mark-delete . mew-face-mark-delete)
-    (,mew-mark-unlink . mew-face-mark-unlink)
-    (,mew-mark-refile . mew-face-mark-refile)
-    (,mew-mark-unread . mew-face-mark-unread))
+  `((,mew-mark-review mew-face-mark-review)
+    (,mew-mark-escape mew-face-mark-escape)
+    (,mew-mark-delete mew-face-mark-delete)
+    (,mew-mark-unlink mew-face-mark-unlink)
+    (,mew-mark-refile mew-face-mark-refile)
+    (,mew-mark-unread mew-face-mark-unread))
   "A list of mark-face pair to used in Summary/Virtual mode.")
 
-(defsubst mew-highlight-mark-get-face (mark)
-  (cdr (assoc mark mew-highlight-mark-keywords)))
+(defun mew-highlight-mark-get-face (mark)
+  (mew-alist-get-value (assoc mark mew-highlight-mark-keywords)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -749,32 +702,32 @@ actions after marking.
 
 0th (the first element of a list) is a mark value. A mark value is
 ASCII code of the mark. For example, the value of the '*' mark
-(mew-mark-review) is 42.
+\(mew-mark-review) is 42.
 
 Seven values following a mark value means as follows:
 
 1st is the case of no mark.
 2nd is the case where the new mark is equal to the old one.
-3rd is the case where level of the new mark is greater than that of the 
+3rd is the case where level of the new mark is greater than that of the
     old one.
 4th, 5th, and 6th is the case where levels are equal.
     4th and 5th is the case that the old mark has state.
         4th means that the old mark is overrode.
         5th means that the old mark remains.
     6th is the case that the old mark does not have state.
-7th is the case where level of the new mark is smaller than that of the 
+7th is the case where level of the new mark is smaller than that of the
     old one.
 
 The value of cursor actions means as follows:
 
 	0 means staying.
 	1 means moving according to the direction,
-	2 means moving according to the direction 
+	2 means moving according to the direction
 	  then displaying the next message.
 
 For more detail, see mew-mark-put-mark and mew-mark-afterstep.")
 
-(defsubst mew-markas-nth (mark case)
+(defun mew-markas-nth (mark case)
   (nth case (assoc mark mew-mark-afterstep-spec)))
 
 (defvar mew-mark-spec
@@ -787,7 +740,7 @@ For more detail, see mew-mark-put-mark and mew-mark-afterstep.")
     (,mew-mark-refile "refile" 2 t   mew-mark-kill-refile mew-mark-unrefile
 		                     mew-mark-exec-refile mew-mark-sanity-refile))
   "*A list of lists which consists of
-mark, name, level, statefullp, kill-line-p, 
+mark, name, level, statefullp, kill-line-p,
 undo-func, exec-func, and sanity-fucn.")
 
 ;;
@@ -796,28 +749,28 @@ undo-func, exec-func, and sanity-fucn.")
   "Collecting all defined marks."
   (mapcar 'car mew-mark-spec))
 
-(defsubst mew-markdb-by-mark (mark)
+(defun mew-markdb-by-mark (mark)
   (assoc mark mew-mark-spec))
 
-(defsubst mew-markdb-name (mark)
+(defun mew-markdb-name (mark)
   (nth 1 (mew-markdb-by-mark mark)))
 
-(defsubst mew-markdb-level (mark)
+(defun mew-markdb-level (mark)
   (nth 2 (mew-markdb-by-mark mark)))
 
-(defsubst mew-markdb-statefullp (mark)
+(defun mew-markdb-statefullp (mark)
   (nth 3 (mew-markdb-by-mark mark)))
 
-(defsubst mew-markdb-killp (mark)
+(defun mew-markdb-killp (mark)
   (nth 4 (mew-markdb-by-mark mark)))
 
-(defsubst mew-markdb-func-undo (mark)
+(defun mew-markdb-func-undo (mark)
   (nth 5 (mew-markdb-by-mark mark)))
 
-(defsubst mew-markdb-func-exec (mark)
+(defun mew-markdb-func-exec (mark)
   (nth 6 (mew-markdb-by-mark mark)))
 
-(defsubst mew-markdb-func-sanity (mark)
+(defun mew-markdb-func-sanity (mark)
   (nth 7 (mew-markdb-by-mark mark)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -832,7 +785,7 @@ You can put any marks, typically 'o' and 'D' according to message
 headers.
 
 This value must be a list of entries. An entry is a list whose first
-element is a field defined in 'mew-scan-fields'. 
+element is a field defined in 'mew-scan-fields'.
 
 If the second element is a list, the first element of the list must be
 a mark (e.g. ?D) or a string which is a comma separated folders (e.g
@@ -844,7 +797,7 @@ matches \"adult\" or \"money\" with 'D'. And Mew marks messages whose
 Subject is \"daily log\" with 'o' to be refiled \"+log\". Also Mew
 marks messages whose Content-Type: contains \"gb2312\" with 'D'.
 
-(setq mew-inbox-action-alist
+\(setq mew-inbox-action-alist
       '((\"Subject:\"
 	 (?D \"adult\" \"money\")
 	 (\"+log\" \"daily log\"))
@@ -857,13 +810,13 @@ function which returns nil or a mark or a string.
 If you are using 'spamassassin' which adds the \"X-Spam-Flag:\" field
 to messages and puts them in your mailbox, configure as follows:
 
-(setq mew-spam: \"X-Spam-Flag:\")
+\(setq mew-spam: \"X-Spam-Flag:\")
 
-(defun mew-spam-assassin (val)
+\(defun mew-spam-assassin (val)
   (let ((case-fold-search t))
     (if (string-match \"yes\" val) ?D)))
 
-(setq mew-inbox-action-alist
+\(setq mew-inbox-action-alist
       '((\"X-Spam-Flag:\" mew-spam-assassin)))
 
 With this configuration, messages with the \"X-Spam-Flag:\" whose
@@ -874,55 +827,16 @@ value is \"Yes\" are automatically marked with 'D' since the
 If you are using 'bogofilter' which adds the \"X-Bogosity:\" field
 to messages and puts them to your mailbox, configure as follows:
 
-(setq mew-spam: \"X-Bogosity:\")
+\(setq mew-spam: \"X-Bogosity:\")
 
-(defun mew-spam-bogofilter (val)
+\(defun mew-spam-bogofilter (val)
   (let ((case-fold-search t))
     (if (string-match \"yes\" val) ?D)))
 
-(setq mew-inbox-action-alist
+\(setq mew-inbox-action-alist
       '((\"X-Bogosity:\" mew-spam-bogofilter)))
 ")
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; External-body
-;;;
-
-(defvar mew-ext-url-alist
-  '(("^application" "Fetch by w3" mew-ext-url-fetch-by-w3 nil)
-    (t		    "Browse by mozilla" "mozilla" ("-install")))
-  "*Alist of (REGEXP DOC PROGRAM ARGS-LIST) to define
-an appropriate method for a content-type in external-body. Note
-this phantom body is defined RFC 2017.
-
-If REGEXP is 't', every content-type is matched to the list.
-If PROGRAM is a string, it is considered an external program.
-If PROGRAM is a symbol, the lisp function whose name is PROGRAM is called.
-
-'mew-ext-url-show-by-w3' and 'mew-ext-url-fetch-by-w3' are pre-defined
-as lisp function.
-
-If you want to use \"w3.el\" instead of \"mozilla\", configure like this:
-	(setq mew-ext-url-alist
-              '((t (\"Browse by w3\" mew-ext-url-show-by-w3 nil))))
-If you want to use \"lynx\" instead of \"mozilla\", configure like this:
-        (setq mew-ext-url-alist
-              '((t (\"Browse by lynx\" \"kterm\" (\"-e\" \"lynx\" \"-color\")))))
-")
-
-(defsubst mew-ext-url-by-ct (ct)
-  (mew-assoc-match2 ct mew-ext-url-alist 0))
-
-(defsubst mew-ext-url-get-doc (prog-list)
-  (nth 1 prog-list))
-
-(defsubst mew-ext-url-get-prog (prog-list)
-  (nth 2 prog-list))
-
-(defsubst mew-ext-url-get-args (prog-list)
-  (nth 3 prog-list))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Proto
@@ -940,7 +854,7 @@ If you want to use \"lynx\" instead of \"mozilla\", configure like this:
 
 (defun mew-proto-to-body (fld n)
   (if (mew-folder-virtualp fld)
-      (setq fld (mew-physical-folder fld)))
+      (setq fld (mew-folder-basename fld)))
   (setq fld (mew-case:folder-folder fld))
   (let ((proto (mew-folder-prefix fld)))
     (unless (member proto mew-folder-prefixes)
@@ -960,9 +874,9 @@ If you want to use \"lynx\" instead of \"mozilla\", configure like this:
   (or proto (setq proto (mew-proto case)))
   (cond
    ((mew-folder-popp  proto) mew-pop-inbox-folder)
-   ((mew-folder-imapp proto) mew-imap-inbox-folder)
+   ((mew-folder-imapp proto) (mew-imap-inbox-folder case))
    ((mew-folder-nntpp proto) (mew-nntp-newsgroup case))
-   (t ;; local 
+   (t ;; local
     (mew-inbox-folder case))))
 
 (defun mew-proto-queue-folder (proto &optional case)
@@ -974,13 +888,13 @@ If you want to use \"lynx\" instead of \"mozilla\", configure like this:
 (defun mew-proto-friend-folder (proto &optional case)
   (cond
    ((mew-folder-imapp proto) (mew-imap-friend-folder case))
-   (t ;; local 
+   (t ;; local
     mew-friend-folder)))
 
 (defun mew-proto-friend-folder-list (proto &optional case)
   (cond
    ((mew-folder-imapp proto) (mew-imap-friend-folder-list case))
-   (t ;; local 
+   (t ;; local
     (mew-local-friend-folder-list))))
 
 (defun mew-proto-folder-alist (proto &optional case)
@@ -988,7 +902,7 @@ If you want to use \"lynx\" instead of \"mozilla\", configure like this:
    ((mew-folder-popp  proto) (mew-pop-folder-alist))
    ((mew-folder-imapp proto) (mew-imap-folder-alist case))
    ((mew-folder-nntpp proto) (mew-nntp-folder-alist case))
-   (t ;; local 
+   (t ;; local
     (mew-local-folder-alist))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -997,34 +911,39 @@ If you want to use \"lynx\" instead of \"mozilla\", configure like this:
 ;;;
 
 (defcustom mew-config-alist nil
-  "*Alist of config. This is a list of 
-	(<case> (<key> . <value>) (<key> . <value>) ...).
+  "*Alist of config. This is a list of
+	(<case> (<key> <value>) (<key> <value>) ...).
   - <case> is a string of case.
-  - <key> is a string of Mew value with the \"mew-\" prefix removed.
+  - <key> is a symbol of Mew value with the 'mew-' prefix removed.
   - <value> is a string.
 
-Currently, the following keys are supported: 
-\"name\", \"user\", \"mail-domain\", 
-\"cc\", \"fcc\", \"dcc\", \"reply-to\", \"organization\", \"header-alist\", 
-\"proto\", 
-\"smtp-server\", \"smtp-port\", \"smtp-ssh-server\", \"smtp-ssl\", \"smtp-ssl-port\", 
-\"smtp-user\", \"smtp-auth-list\", 
-\"smtp-msgid-user\", \"smtp-msgid-domain\", \"smtp-helo-domain\", \"smtp-mail-from\", 
-\"pop-server\", \"pop-port\", \"pop-ssh-server\", \"pop-ssl\", \"pop-ssl-port\", 
-\"pop-user\", \"pop-auth\", \"pop-auth-list\",
-\"pop-size\", \"pop-body-lines\", \"pop-delete\", \"pop-body-lines\",
-\"pop-proxy-server\", \"pop-proxy-port\",
-\"imap-server\", \"imap-port\", \"imap-ssh-server\", \"imap-ssl\", \"imap-ssl-port\", 
-\"imap-user\", \"imap-auth\", \"imap-auth-list\", 
-\"imap-size\", \"imap-header-only\", \"imap-delete\", 
-\"imap-trash-folder\", \"imap-queue-folder\", \"imap-spam-field\", \"imap-spam-word\",
-\"nntp-server\", \"nntp-port\", \"nntp-ssh-server\", \"nntp-ssl\", \"nntp-ssl-port\", 
-\"nntp-user\", \"nntp-size\", \"nntp-header-only\", 
-\"nntp-msgid-user\", \"nntp-msgid-domain\", 
-\"inbox-folder\", \"queue-folder\", \"postq-folder\", 
-\"mailbox-type\", \"mbox-command\", \"mbox-command-arg\", 
-\"use-retrieve\", \"signature-file\", \"content-type\", \"refile-guess-alist\",
-\"spam-prog\", \"spam-prog-args\", \"ham-prog\", \"ham-prog-args\".
+Currently, the following keys are supported:
+name, user, mail-domain, 
+cc, fcc, dcc, reply-to, organization, header-alist, proto, 
+smtp-server, smtp-port, smtp-ssh-server, smtp-ssl, smtp-ssl-port, 
+smtp-user, smtp-auth-list, 
+smtp-msgid-user, smtp-msgid-domain, smtp-helo-domain, smtp-mail-from, 
+pop-server, pop-port, pop-ssh-server, pop-ssl, pop-ssl-port, 
+pop-user, pop-auth, pop-auth-list, 
+pop-size, pop-header-only, pop-delete, pop-body-lines,
+pop-proxy-server, pop-proxy-port,
+imap-server, imap-port, imap-ssh-server, imap-ssl, imap-ssl-port, 
+imap-user, imap-auth, imap-auth-list, 
+imap-size, imap-header-only, imap-delete, 
+imap-trash-folder, imap-queue-folder, imap-spam-field, imap-spam-word,
+imap-proxy-server, imap-proxy-port,
+nntp-server, nntp-port, nntp-ssh-server, nntp-ssl, nntp-ssl-port, 
+nntp-user, nntp-size, nntp-header-only, 
+nntp-msgid-user, nntp-msgid-domain, 
+ssl-cert-directory, ssl-verify-level, 
+inbox-folder, queue-folder, postq-folder, 
+mailbox-type, mbox-command, mbox-command-arg,
+signature-file, content-type, refile-guess-alist, 
+spam-prog, spam-prog-args, ham-prog, ham-prog-args, 
+use-old-pgp, pgp-signer, smime-signer, privacy-method, 
+protect-privacy-always, protect-privacy-always-type, 
+protect-privacy-encrypted, protect-privacy-encrypted-type, 
+protect-privacy-with-old-pgp-signature
 
 from = name <user@mail-domain>
 message-id = *random*.smtp-msgid-user@smtp-msgid-domain
@@ -1032,38 +951,40 @@ message-id = *random*.nntp-msgid-user@nntp-msgid-domain
 
 An example is as follows:
 
-(setq mew-config-alist
-      '((\"mew\"
-	 (\"mail-domain\"  . \"example.org\")
-	 (\"inbox-folder\" . \"+inbox-mew\"))
-	(\"keio\"
-	 (\"cc\"           . \"kazu@example.jp\")
-	 (\"user\"         . \"pooh\")
-	 (\"mail-domain\"  . \"example.net\"))
-	(\"default\"
-	 (\"name\"         . \"Kazu Yamamoto\")
-	 (\"mail-domain\"  . \"example.jp\"))))
+\(setq mew-config-alist
+      '((mew
+	 (mail-domain  \"example.org\")
+	 (inbox-folder \"+inbox-mew\"))
+	(keio
+	 (cc           \"kazu@example.jp\")
+	 (user         \"pooh\")
+	 (mail-domain  \"example.net\"))
+	(default
+	 (name         \"Kazu Yamamoto\")
+	 (mail-domain  \"example.jp\"))))
 "
   :group 'mew-env
   :type '(alist :key-type string
                 :value-type (repeat (cons string string))))
 
-(defsubst mew-cfent-by-case (case)
+(defun mew-cfent-by-case (case)
   (if (null case)
-      (assoc mew-case-default mew-config-alist)
-    (assoc case mew-config-alist)))
+      (or (assoc mew-case-default mew-config-alist)
+	  (assoc (intern mew-case-default) mew-config-alist))
+    (or (assoc case mew-config-alist)
+	(assoc (intern case) mew-config-alist))))
 
 (provide 'mew-vars2)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2006 Mew developing team.
+;; Copyright (C) 2000-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1072,7 +993,7 @@ An example is as follows:
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
