@@ -5,6 +5,14 @@
 
 ;;; Code:
 
+(eval-when-compile
+  (mew-no-waring-defvar w32-start-process-show-window)
+  (mew-no-waring-defun mw32script-init)
+  (mew-no-waring-defun mw32script-make-pathext-regexp)
+  (mew-no-waring-defun mw32script-openp)
+  (mew-no-waring-defun mw32script-resolve-script)
+  (mew-no-waring-defun dos-to-unix-filename))
+
 ;;; for NTEmacs User
 ;;
 ;; put mw32script.el(in Meadow's archive) into load-path.
@@ -38,74 +46,53 @@
 
 (defvar mew-w32-cs-print nil)
 
-;; Emacs version dependent variables.
-(cond
- ((featurep 'xemacs)			; XEmacs
-  )
- ((<= 20 emacs-major-version)
-  (if (featurep 'meadow) ;; Meadow
-      (progn
-	(require 'mw32script)
-	(mw32script-init))
-    (if (condition-case nil (require 'mw32script) (file-error nil)) ;; NTEmacs
-	(progn
-	  (mw32script-make-pathext-regexp)
-	  (defalias 'call-process-original 'call-process)
-	  (defun call-process (PROGRAM INFILE BUFFER DISPLAY &rest PROGRAM-ARGS)
-	    (let (prog sargs)
-	      (setq prog (mw32script-openp PROGRAM))
-	      (if (not prog)
-		  (progn
-		    (setq prog (mew-which-exec PROGRAM))
-		    (setq sargs (mw32script-resolve-script prog))))
-	      (if sargs
-		  (apply 'call-process-original
-			 (car sargs) INFILE BUFFER DISPLAY
-			 prog PROGRAM-ARGS)
-		(apply 'call-process-original
-		       PROGRAM INFILE BUFFER DISPLAY PROGRAM-ARGS))))
-	  (defalias 'start-process-original 'start-process)
-	  (defun start-process (NAME BUFFER PROGRAM &rest PROGRAM-ARGS)
-	    (let (prog sargs)
-	      (setq prog (mw32script-openp PROGRAM))
-	      (if (not prog)
-		  (progn
-		    (setq prog (mew-which-exec PROGRAM))
-		    (setq sargs (mw32script-resolve-script prog))))
-	      (if sargs
-		  (apply 'start-process-original
-			 NAME BUFFER (car sargs) prog PROGRAM-ARGS)
-		(apply 'start-process-original
-		       NAME BUFFER PROGRAM PROGRAM-ARGS))))))
-    )))
-
-(setq mew-which-exec-suffixes '("" ".exe" ".bat"))
-
-(when (and (featurep 'mw32script)
-	   (fboundp 'define-process-argument-editing))
-  ;; Argument setting for mknmz.bat, gcnmz.bat, rfnmz.bat
-  (let ((progs '("mknmz" "gcnmz" "rfnmz"))
-	prog)
-    (while progs
-      (setq prog (mew-which-exec (car progs)))
-      (setq progs (cdr progs))
-      (when (and prog (string-match "\\.bat$" prog))
-	(setq prog (regexp-quote prog))
-	;; (general-process-argument-editing-function
-	;;     ARGUMENT QUOTING ARGV0ISP &optional EP H2SP QP S2ISP)	
-	(define-process-argument-editing
-	  prog
-	  (lambda (x) (general-process-argument-editing-function x nil nil))
-	  'first)))))
+(setq mew-which-exec-suffixes (if (and (boundp 'exec-suffixes)
+				       exec-suffixes)
+				  exec-suffixes
+				'(".exe" ".com" ".bat" ".cmd" "")))
 
 (setq mew-delete-temp-file  nil)
 
+;; Emacs version dependent variables.
+(cond
+ ((featurep 'meadow) ;; Meadow
+  (require 'mw32script)
+  (mw32script-init))
+ ((condition-case nil
+      (require 'mw32script)
+    (file-error nil)) ;; NTEmacs
+  (mw32script-make-pathext-regexp)
+  (defalias 'call-process-original 'call-process)
+  (defun call-process (PROGRAM INFILE BUFFER DISPLAY &rest PROGRAM-ARGS)
+    (let (prog sargs)
+      (setq prog (mw32script-openp PROGRAM))
+      (unless prog
+	(setq prog (mew-which-exec PROGRAM))
+	(setq sargs (mw32script-resolve-script prog)))
+      (if sargs
+	  (apply 'call-process-original
+		 (car sargs) INFILE BUFFER DISPLAY
+		 prog PROGRAM-ARGS)
+	(apply 'call-process-original
+	       PROGRAM INFILE BUFFER DISPLAY PROGRAM-ARGS))))
+  (defalias 'start-process-original 'start-process)
+  (defun start-process (NAME BUFFER PROGRAM &rest PROGRAM-ARGS)
+    (let (prog sargs)
+      (setq prog (mw32script-openp PROGRAM))
+      (unless prog
+	(setq prog (mew-which-exec PROGRAM))
+	(setq sargs (mw32script-resolve-script prog)))
+      (if sargs
+	  (apply 'start-process-original
+		 NAME BUFFER (car sargs) prog PROGRAM-ARGS)
+	(apply 'start-process-original
+	       NAME BUFFER PROGRAM PROGRAM-ARGS))))))
+
 ;; printing
 (defun mew-w32-print-buffer ()
   (let ((tempfile (mew-make-temp-name)))
-    (mew-frwlet
-     mew-cs-dummy mew-w32-cs-print
-     (write-region (point-min) (point-max) tempfile nil nil))
+    (mew-frwlet mew-cs-dummy mew-w32-cs-print
+      (write-region (point-min) (point-max) tempfile nil nil))
     (setq w32-start-process-show-window t)
     (cond
      ((eq mew-w32-prog-print-arg nil)
@@ -121,6 +108,7 @@
 (defvar mew-prog-plain    'mew-mime-text/plain)
 (defvar mew-prog-html     '(mew-mime-text/html mew-mime-text/html-ext))
 (defvar mew-prog-xml      '(mew-mime-text/xml  mew-mime-text/xml-ext))
+(defvar mew-prog-patch    '(mew-mime-text/plain mew-mime-text/patch-ext))
 (defvar mew-prog-enriched 'mew-mime-text/enriched)
 (defvar mew-prog-text     'mew-mime-text/plain)
 (defvar mew-prog-audio           mew-w32-exec)
@@ -147,6 +135,7 @@
 (defvar mew-prog-visio           mew-w32-exec)
 (defvar mew-prog-ooffice         mew-w32-exec)
 (defvar mew-prog-rtf             mew-w32-exec)
+(defvar mew-prog-zip             mew-w32-exec)
 
 ;;;
 ;;; Text/Html, Application/Xml, Image
@@ -172,10 +161,9 @@
 (defvar mew-prog-application/mspowerpoint nil)
 (defvar mew-prog-application/rtf nil)
 
-(if (eq mew-mule-ver 3)
-    (setq mew-cs-database-for-arg
-	  '((iso-2022-jp . shift_jis-unix)
-	    (iso-2022-kr . euc-kr-unix))))
+(setq mew-cs-database-for-arg
+      '((iso-2022-jp . shift_jis-unix)
+	(iso-2022-kr . euc-kr-unix)))
 
 (setq mew-prog-grep-max-msgs 2000) ;; for external grep (pick & virtual)
 
@@ -185,11 +173,10 @@
   (interactive "e")
   (when (eq (posn-window (event-start event)) (selected-window))
     (let ((files (car (cdr (cdr event))))
-	  from to)
+	  to)
       (if (stringp files)
 	  (setq files (list files)))
-      (while (setq from (car files))
-	(setq files (cdr files))
+      (dolist (from files)
 	(setq from (dos-to-unix-filename from))
 	(when (file-exists-p from)
 	  (setq to (file-name-nondirectory from))
@@ -206,17 +193,22 @@
 
 (defun mew-set-file-type (file) ())
 
+(defvar mew-cs-est 'shift_jis)
+
+(defun mew-focus-frame (frame)
+  (if (fboundp 'w32-focus-frame) (w32-focus-frame frame)))
+
 (provide 'mew-win32)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2006 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -225,7 +217,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
